@@ -41,12 +41,13 @@ epochs = mne.Epochs(raw, events, event_ids, tmin, tmax, picks=picks)
 
 # Get the stim channel data
 pick_ch = mne.pick_channels(epochs.ch_names, ['STI 014'])[0]
-data = epochs.get_data()[:, pick_ch, :].astype(int)
-data = np.sum((data.astype(int) & 512) == 512, axis=0)
+data = epochs.get_data()[:, pick_ch, :]
+data = np.sum((data.astype(int) & eog_event_id) == eog_event_id, axis=0)
 
 ###############################################################################
 # Plot EOG artifact distribution
-plt.stem(1e3 * epochs.times, data)
-plt.xlabel('Times (ms)')
-plt.ylabel('Blink counts (from %s trials)' % len(epochs))
-plt.show()
+fig, ax = plt.subplots()
+ax.stem(1e3 * epochs.times, data, use_line_collection=True)
+ax.set(xlabel='Times (ms)',
+       ylabel='Blink counts (from %s trials)' % len(epochs))
+fig.tight_layout()
