@@ -1,7 +1,7 @@
 """Beamformers for source localization."""
 
 from ._lcmv import (make_lcmv, apply_lcmv, apply_lcmv_epochs, apply_lcmv_raw,
-                    tf_lcmv)
+                    tf_lcmv, apply_lcmv_cov)
 from ._dics import (make_dics, apply_dics, apply_dics_epochs, apply_dics_csd,
                     tf_dics)
 from ._rap_music import rap_music
