@@ -8,17 +8,16 @@
 # License: BSD (3-clause)
 import numpy as np
 
-from ..utils import logger, verbose, warn, _reg_pinv
+from ..utils import (logger, verbose, warn, _check_one_ch_type,
+                     _check_channels_spatial_filter, _check_rank,
+                     _check_option)
 from ..forward import _subject_from_forward
 from ..minimum_norm.inverse import combine_xyz, _check_reference
 from ..source_estimate import _make_stc, _get_src_type
 from ..time_frequency import csd_fourier, csd_multitaper, csd_morlet
-from ._compute_beamformer import (_setup_picks,
-                                  _pick_channels_spatial_filter,
-                                  _check_proj_match, _prepare_beamformer_input,
-                                  _compute_beamformer, _check_one_ch_type,
-                                  _check_src_type, Beamformer, _check_rank)
-from ..externals import six
+from ._compute_beamformer import (_check_proj_match, _prepare_beamformer_input,
+                                  _compute_beamformer, _check_src_type,
+                                  Beamformer, _compute_power)
 
 
 @verbose
@@ -101,9 +100,7 @@ def make_dics(info, forward, csd, reg=0.05, label=None, pick_ori=None,
         each spatial location, prior to inversion. This may be necessary when
         you use a single sphere model for MEG and ``mode='vertex'``.
         Defaults to ``False``.
-    verbose : bool, str, int, None
-        If not None, override default verbose level (see :func:`mne.verbose`
-        and :ref:`Logging documentation <tut_logging>` for more).
+    %(verbose)s
 
     Returns
     -------
@@ -163,8 +160,7 @@ def make_dics(info, forward, csd, reg=0.05, label=None, pick_ori=None,
         inversion='matrix', weight_norm='unit-gain', normalize_fwd=False
 
     For more information about ``real_filter``, see the
-    `supplemental information <http://www.cell.com/cms/attachment/616681/4982593/mmc1.pdf>`_
-    from [3]_.
+    supplemental information from [3]_.
 
     References
     ----------
@@ -185,10 +181,10 @@ def make_dics(info, forward, csd, reg=0.05, label=None, pick_ori=None,
            statistics in Python. bioRxiv, 245530.
            https://doi.org/10.1101/245530
     """  # noqa: E501
-    allowed_ori = [None, 'normal', 'max-power']
     rank = _check_rank(rank)
-    if pick_ori not in allowed_ori:
-        raise ValueError('"pick_ori" should be one of %s.' % allowed_ori)
+    _check_option('pick_ori', pick_ori, [None, 'normal', 'max-power'])
+    _check_option('inversion', inversion, ['single', 'matrix'])
+    _check_option('weight_norm', weight_norm, ['unit-noise-gain', 'nai', None])
 
     # Leadfield rank and optional rank reduction
     # (to deal with problems with complex eigenvalues within the computation
@@ -202,11 +198,6 @@ def make_dics(info, forward, csd, reg=0.05, label=None, pick_ori=None,
             'and inversion="matrix".'
         )
 
-    # Inversion mode
-    if inversion not in ['single', 'matrix']:
-        raise ValueError("The inversion parameter should be either 'single' "
-                         "or 'matrix'.")
-
     frequencies = [np.mean(freq_bin) for freq_bin in csd.frequencies]
     n_freqs = len(frequencies)
     n_orient = forward['sol']['ncol'] // forward['nsource']
@@ -214,21 +205,32 @@ def make_dics(info, forward, csd, reg=0.05, label=None, pick_ori=None,
     # Determine how to normalize the leadfield
     if normalize_fwd:
         if inversion == 'single':
-            fwd_norm = 'dipole'
+            if weight_norm == 'unit-noise-gain':
+                raise ValueError('The computation of a unit-noise-gain '
+                                 'beamformer with inversion="single" is not '
+                                 'stable with depth normalization, set  '
+                                 'normalize_fwd to False.')
+            combine_xyz = False
         else:
-            fwd_norm = 'vertex'
+            combine_xyz = 'fro'
+        exp = 1.  # turn on depth weighting with exponent 1
     else:
-        fwd_norm = None  # No normalization
+        exp = None  # turn off depth weighting entirely
+        combine_xyz = False
+
+    _check_one_ch_type('dics', info, forward)
+
+    # pick info, get gain matrix, etc.
+    _, info, proj, vertices, G, _, nn, orient_std = _prepare_beamformer_input(
+        info, forward, label, pick_ori,
+        combine_xyz=combine_xyz, exp=exp)
+    subject = _subject_from_forward(forward)
+    src_type = _get_src_type(forward['src'], vertices)
+    del forward
+    ch_names = list(info['ch_names'])
 
-    picks = _setup_picks(info=info, forward=forward)
-    _, ch_names, proj, vertices, G, nn = _prepare_beamformer_input(
-        info, forward, label, picks=picks, pick_ori=pick_ori,
-        fwd_norm=fwd_norm,
-    )
     csd_picks = [csd.ch_names.index(ch) for ch in ch_names]
 
-    _check_one_ch_type(info, picks, None, 'dics')
-
     logger.info('Computing DICS spatial filters...')
     Ws = []
     for i, freq in enumerate(frequencies):
@@ -247,13 +249,11 @@ def make_dics(info, forward, csd, reg=0.05, label=None, pick_ori=None,
         # compute spatial filter
         W = _compute_beamformer(G, Cm, reg, n_orient, weight_norm, pick_ori,
                                 reduce_rank, rank=rank, inversion=inversion,
-                                nn=nn)
+                                nn=nn, orient_std=orient_std)
         Ws.append(W)
 
     Ws = np.array(Ws)
 
-    subject = _subject_from_forward(forward)
-    src_type = _get_src_type(forward['src'], vertices)
     filters = Beamformer(
         kind='DICS', weights=Ws, csd=csd, ch_names=ch_names, proj=proj,
         vertices=vertices, subject=subject, pick_ori=pick_ori,
@@ -334,9 +334,7 @@ def apply_dics(evoked, filters, verbose=None):
     filters : instance of Beamformer
         DICS spatial filter (beamformer weights)
         Filter weights returned from :func:`make_dics`.
-    verbose : bool, str, int, None
-        If not None, override default verbose level (see :func:`mne.verbose`
-        and :ref:`Logging documentation <tut_logging>` for more).
+    %(verbose)s
 
     Returns
     -------
@@ -356,12 +354,12 @@ def apply_dics(evoked, filters, verbose=None):
     data = evoked.data
     tmin = evoked.times[0]
 
-    sel = _pick_channels_spatial_filter(evoked.ch_names, filters)
+    sel = _check_channels_spatial_filter(evoked.ch_names, filters)
     data = data[sel]
 
     stc = _apply_dics(data=data, filters=filters, info=info, tmin=tmin)
 
-    return six.advance_iterator(stc)
+    return next(stc)
 
 
 @verbose
@@ -391,9 +389,7 @@ def apply_dics_epochs(epochs, filters, return_generator=False, verbose=None):
     return_generator : bool
         Return a generator object instead of a list. This allows iterating
         over the stcs without having to keep them all in memory.
-    verbose : bool, str, int, None
-        If not None, override default verbose level (see :func:`mne.verbose`
-        and :ref:`Logging documentation <tut_logging>` for more).
+    %(verbose)s
 
     Returns
     -------
@@ -418,7 +414,7 @@ def apply_dics_epochs(epochs, filters, return_generator=False, verbose=None):
     info = epochs.info
     tmin = epochs.times[0]
 
-    sel = _pick_channels_spatial_filter(epochs.ch_names, filters)
+    sel = _check_channels_spatial_filter(epochs.ch_names, filters)
     data = epochs.get_data()[:, sel, :]
 
     stcs = _apply_dics(data=data, filters=filters, info=info, tmin=tmin)
@@ -446,9 +442,7 @@ def apply_dics_csd(csd, filters, verbose=None):
     filters : instance of Beamformer
         DICS spatial filter (beamformer weights)
         Filter weights returned from `make_dics`.
-    verbose : bool, str, int, None
-        If not None, override default verbose level (see :func:`mne.verbose`
-        and :ref:`Logging documentation <tut_logging>` for more).
+    %(verbose)s
 
     Returns
     -------
@@ -489,117 +483,19 @@ def apply_dics_csd(csd, filters, verbose=None):
         Cm = Cm[csd_picks, :][:, csd_picks]
         W = filters['weights'][i]
 
-        for k in range(n_sources):
-            Wk = W[n_orient * k: n_orient * k + n_orient]
-            power = Wk.dot(Cm).dot(Wk.T)
-
-            if n_orient > 1:  # Pool the orientations
-                source_power[k, i] = np.abs(power.trace())
-            else:
-                source_power[k, i] = np.abs(power)
+        source_power[:, i] = _compute_power(Cm, W, n_orient)
 
     logger.info('[done]')
 
     # compatibility with 0.16, add src_type as None if not present:
     filters, warn_text = _check_src_type(filters)
 
-    return (_make_stc(source_power.reshape(-1, n_freqs), vertices=vertices,
-                      src_type=filters['src_type'], tmin=0, tstep=1,
+    return (_make_stc(source_power, vertices=vertices,
+                      src_type=filters['src_type'], tmin=0., tstep=1.,
                       subject=subject, warn_text=warn_text),
             frequencies)
 
 
-def _apply_old_dics(data, info, tmin, forward, noise_csd, data_csd, reg,
-                    label=None, picks=None, pick_ori=None, real_filter=False,
-                    verbose=None):
-    """Old implementation of Dynamic Imaging of Coherent Sources (DICS)."""
-    from scipy import linalg  # Local import to keep 'import mne' fast
-
-    if len(noise_csd.frequencies) > 1 or len(data_csd.frequencies) > 1:
-        raise ValueError('CSD matrix object should only contain one '
-                         'frequency.')
-
-    is_free_ori, _, proj, vertno, G, _ =\
-        _prepare_beamformer_input(info, forward, label, picks, pick_ori)
-
-    Cm = data_csd.get_data(index=0)
-    Cm_noise = noise_csd.get_data(index=0)
-
-    # Take real part of Cm to compute real filters
-    if real_filter:
-        Cm = Cm.real
-
-    # Tikhonov regularization using reg parameter to control for
-    # trade-off between spatial resolution and noise sensitivity
-    # eq. 25 in Gross and Ioannides, 1999 Phys. Med. Biol. 44 2081
-    Cm_inv, _ = _reg_pinv(Cm, reg, rank=None)
-    del Cm
-
-    # Compute spatial filters
-    W = np.dot(G.T, Cm_inv)
-    n_orient = 3 if is_free_ori else 1
-    n_sources = G.shape[1] // n_orient
-
-    for k in range(n_sources):
-        Wk = W[n_orient * k: n_orient * k + n_orient]
-        Gk = G[:, n_orient * k: n_orient * k + n_orient]
-        Ck = np.dot(Wk, Gk)
-
-        if is_free_ori:
-            # Free source orientation
-            Wk[:] = np.dot(linalg.pinv(Ck, 0.1), Wk)
-        else:
-            # Fixed source orientation
-            Wk /= Ck
-
-        # Noise normalization
-        noise_norm = np.dot(np.dot(Wk.conj(), Cm_noise), Wk.T)
-        noise_norm = np.abs(noise_norm).trace()
-        Wk /= np.sqrt(noise_norm)
-
-    # Pick source orientation normal to cortical surface
-    if pick_ori == 'normal':
-        W = W[2::3]
-        is_free_ori = False
-
-    if isinstance(data, np.ndarray) and data.ndim == 2:
-        data = [data]
-        return_single = True
-    else:
-        return_single = False
-
-    subject = _subject_from_forward(forward)
-    for i, M in enumerate(data):
-        if len(M) != len(picks):
-            raise ValueError('data and picks must have the same length')
-
-        if not return_single:
-            logger.info("Processing epoch : %d" % (i + 1))
-
-        # Apply SSPs
-        if info['projs']:
-            M = np.dot(proj, M)
-
-        # project to source space using beamformer weights
-        if is_free_ori:
-            sol = np.dot(W, M)
-            logger.info('combining the current components...')
-            sol = combine_xyz(sol)
-        else:
-            # Linear inverse: do not delay compuation due to non-linear abs
-            sol = np.dot(W, M)
-
-        tstep = 1.0 / info['sfreq']
-        if np.iscomplexobj(sol):
-            sol = np.abs(sol)  # XXX : STC cannot contain (yet?) complex values
-
-        src_type = _get_src_type(forward['src_type'], vertno)
-        yield _make_stc(sol, vertices=vertno, src_type=src_type, tmin=tmin,
-                        tstep=tstep, subject=subject)
-
-    logger.info('[done]')
-
-
 @verbose
 def tf_dics(epochs, forward, noise_csds, tmin, tmax, tstep, win_lengths,
             subtract_evoked=False, mode='fourier', freq_bins=None,
@@ -643,11 +539,11 @@ def tf_dics(epochs, forward, noise_csds, tmin, tmax, tstep, win_lengths,
         tf source grid. Defaults to False.
     mode : 'fourier' | 'multitaper' | 'cwt_morlet'
         Spectrum estimation mode. Defaults to 'fourier'.
-    freq_bins : list of tuples of float
+    freq_bins : list of tuple of float
         Start and end point of frequency bins of interest.
         Only used in 'multitaper' or 'fourier' mode. For 'cwt_morlet' mode, use
         the ``frequencies`` parameter instead.
-    frequencies : list of float | list of lists of float
+    frequencies : list of float | list of list of float
         The frequencies to compute the source power for. If you want to compute
         the average power for multiple frequency bins, specify a list of
         lists: each list containing the frequencies for the corresponding bin.
@@ -665,7 +561,7 @@ def tf_dics(epochs, forward, noise_csds, tmin, tmax, tstep, win_lengths,
         Use adaptive weights to combine the tapered spectra into CSD. Only used
         in 'multitaper' mode. Defaults to False.
     mt_low_bias : bool
-        Only use tapers with more than 90% spectral concentration within
+        Only use tapers with more than 90%% spectral concentration within
         bandwidth. Only used in 'multitaper' mode. Defaults to True.
     cwt_n_cycles: float | list of float | None
         Number of cycles to use when constructing Morlet wavelets. Fixed number
@@ -735,9 +631,7 @@ def tf_dics(epochs, forward, noise_csds, tmin, tmax, tstep, win_lengths,
         each spatial location, prior to inversion. This may be necessary when
         you use a single sphere model for MEG and ``mode='vertex'``.
         Defaults to ``False``.
-    verbose : bool, str, int, None
-        If not None, override default verbose level (see :func:`mne.verbose`
-        and :ref:`Logging documentation <tut_logging>` for more).
+    %(verbose)s
 
     Returns
     -------
@@ -855,8 +749,6 @@ def tf_dics(epochs, forward, noise_csds, tmin, tmax, tstep, win_lengths,
                     'window %d to %d ms, in frequency range %d to %d Hz' %
                     (win_tmin * 1e3, win_tmax * 1e3, fmin, fmax)
                 )
-                win_tmin = win_tmin
-                win_tmax = win_tmax
 
                 # Calculating data CSD in current time window
                 if mode == 'fourier':
