@@ -1,18 +1,18 @@
 #!/usr/bin/env python
 r"""Compute SSP/PCA projections for ECG artifacts.
 
-You can do for example:
+Examples
+--------
+.. code-block:: console
+
+    $ mne compute_proj_ecg -i sample_audvis_raw.fif -c "MEG 1531" \
+                           --l-freq 1 --h-freq 100 \
+                           --rej-grad 3000 --rej-mag 4000 --rej-eeg 100
 
-$ mne compute_proj_ecg -i sample_audvis_raw.fif -c "MEG 1531" \
-                       --l-freq 1 --h-freq 100 \
-                       --rej-grad 3000 --rej-mag 4000 --rej-eeg 100
 """
-from __future__ import print_function
-
 # Authors : Alexandre Gramfort, Ph.D.
 #           Martin Luessi, Ph.D.
 
-from mne.externals.six import string_types
 import os
 import sys
 import mne
@@ -199,7 +199,7 @@ def run():
         # append the ecg projs, so they are last in the list
         projs = mne.read_proj(proj_fname) + projs
 
-    if isinstance(preload, string_types) and os.path.exists(preload):
+    if isinstance(preload, str) and os.path.exists(preload):
         os.remove(preload)
 
     print("Writing ECG projections in %s" % ecg_proj_fname)
@@ -208,6 +208,4 @@ def run():
     print("Writing ECG events in %s" % ecg_event_fname)
     mne.write_events(ecg_event_fname, events)
 
-is_main = (__name__ == '__main__')
-if is_main:
-    run()
+mne.utils.run_command_if_main()
