@@ -4,6 +4,7 @@
 
 import numpy as np
 
+from .fixes import rfft, irfft
 from .utils import (sizeof_fmt, logger, get_config, warn, _explain_exception,
                     verbose)
 
@@ -50,10 +51,7 @@ def init_cuda(ignore_config=False, verbose=None):
     ----------
     ignore_config : bool
         If True, ignore the config value MNE_USE_CUDA and force init.
-    verbose : bool, str, int, or None
-        If not None, override default verbose level (see :func:`mne.verbose`
-        and :ref:`Logging documentation <tut_logging>` for more). Defaults to
-        self.verbose.
+    %(verbose)s
     """
     global _cuda_capable
     if _cuda_capable:
@@ -65,13 +63,14 @@ def init_cuda(ignore_config=False, verbose=None):
     # Triage possible errors for informative messaging
     _cuda_capable = False
     try:
-        import cupy
+        import cupy  # noqa
     except ImportError:
         warn('module cupy not found, CUDA not enabled')
         return
+    device_id = int(get_config('MNE_CUDA_DEVICE', '0'))
     try:
         # Initialize CUDA
-        cupy.cuda.Device()
+        _set_cuda_device(device_id, verbose)
     except Exception:
         warn('so CUDA device could be initialized, likely a hardware error, '
              'CUDA not enabled%s' % _explain_exception())
@@ -82,10 +81,40 @@ def init_cuda(ignore_config=False, verbose=None):
     logger.info('Enabling CUDA with %s available memory' % get_cuda_memory())
 
 
+@verbose
+def set_cuda_device(device_id, verbose=None):
+    """Set the CUDA device temporarily for the current session.
+
+    Parameters
+    ----------
+    device_id : int
+        Numeric ID of the CUDA-capable device you want MNE-Python to use.
+    %(verbose)s
+    """
+    if _cuda_capable:
+        _set_cuda_device(device_id, verbose)
+    elif get_config('MNE_USE_CUDA', 'false').lower() == 'true':
+        init_cuda()
+        _set_cuda_device(device_id, verbose)
+    else:
+        warn('Could not set CUDA device because CUDA is not enabled; either '
+             'run mne.cuda.init_cuda() first, or set the MNE_USE_CUDA config '
+             'variable to "true".')
+
+
+@verbose
+def _set_cuda_device(device_id, verbose=None):
+    """Set the CUDA device."""
+    import cupy
+    cupy.cuda.Device(device_id).use()
+    logger.info('Now using CUDA device {}'.format(device_id))
+
+
 ###############################################################################
 # Repeated FFT multiplication
 
-def _setup_cuda_fft_multiply_repeated(n_jobs, h, n_fft):
+def _setup_cuda_fft_multiply_repeated(n_jobs, h, n_fft,
+                                      kind='FFT FIR filtering'):
     """Set up repeated CUDA FFT multiplication with a given filter.
 
     Parameters
@@ -97,6 +126,8 @@ def _setup_cuda_fft_multiply_repeated(n_jobs, h, n_fft):
         The filtering function that will be used repeatedly.
     n_fft : int
         The number of points in the FFT.
+    kind : str
+        The kind to report to the user.
 
     Returns
     -------
@@ -123,8 +154,8 @@ def _setup_cuda_fft_multiply_repeated(n_jobs, h, n_fft):
     -----
     This function is designed to be used with fft_multiply_repeated().
     """
-    cuda_dict = dict(n_fft=n_fft, rfft=np.fft.rfft, irfft=np.fft.irfft,
-                     h_fft=np.fft.rfft(h, n=n_fft))
+    cuda_dict = dict(n_fft=n_fft, rfft=rfft, irfft=irfft,
+                     h_fft=rfft(h, n=n_fft))
     if n_jobs == 'cuda':
         n_jobs = 1
         init_cuda()
@@ -133,7 +164,7 @@ def _setup_cuda_fft_multiply_repeated(n_jobs, h, n_fft):
             try:
                 # do the IFFT normalization now so we don't have to later
                 h_fft = cupy.array(cuda_dict['h_fft'])
-                logger.info('Using CUDA for FFT FIR filtering')
+                logger.info('Using CUDA for %s' % kind)
             except Exception as exp:
                 logger.info('CUDA not used, could not instantiate memory '
                             '(arrays may be too large: "%s"), falling back to '
@@ -215,7 +246,7 @@ def _setup_cuda_fft_resample(n_jobs, W, new_len):
     -----
     This function is designed to be used with fft_resample().
     """
-    cuda_dict = dict(use_cuda=False, rfft=np.fft.rfft, irfft=np.fft.irfft)
+    cuda_dict = dict(use_cuda=False, rfft=rfft, irfft=irfft)
     rfft_len_x = len(W) // 2 + 1
     # fold the window onto inself (should be symmetric) and truncate
     W = W.copy()
@@ -245,16 +276,16 @@ def _setup_cuda_fft_resample(n_jobs, W, new_len):
     return n_jobs, cuda_dict
 
 
-def _cuda_upload_rfft(x, n):
+def _cuda_upload_rfft(x, n, axis=-1):
     """Upload and compute rfft."""
     import cupy
-    return cupy.fft.rfft(cupy.array(x), n)
+    return cupy.fft.rfft(cupy.array(x), n=n, axis=axis)
 
 
-def _cuda_irfft_get(x, n):
+def _cuda_irfft_get(x, n, axis=-1):
     """Compute irfft and get."""
     import cupy
-    return cupy.fft.irfft(x, n).get()
+    return cupy.fft.irfft(x, n=n, axis=axis).get()
 
 
 def _fft_resample(x, new_len, npads, to_removes, cuda_dict=None,
@@ -324,7 +355,7 @@ def _smart_pad(x, n_pad, pad='reflect_limited'):
     if pad == 'reflect_limited':
         # need to pad with zeros if len(x) <= npad
         l_z_pad = np.zeros(max(n_pad[0] - len(x) + 1, 0), dtype=x.dtype)
-        r_z_pad = np.zeros(max(n_pad[0] - len(x) + 1, 0), dtype=x.dtype)
+        r_z_pad = np.zeros(max(n_pad[1] - len(x) + 1, 0), dtype=x.dtype)
         return np.concatenate([l_z_pad, 2 * x[0] - x[n_pad[0]:0:-1], x,
                                2 * x[-1] - x[-2:-n_pad[1] - 2:-1], r_z_pad])
     else:
