@@ -1,4 +1,4 @@
-# Authors: Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
+# Authors: Alexandre Gramfort <alexandre.gramfort@inria.fr>
 #          Denis A. Engemann <denis.engemann@gmail.com>
 #          Eric Larson <larson.eric.d@gmail.com>
 #
@@ -11,14 +11,16 @@ DEFAULTS = dict(
                ref_meg='steelblue', misc='k', stim='k', resp='k', chpi='k',
                exci='k', ias='k', syst='k', seeg='saddlebrown', dipole='k',
                gof='k', bio='k', ecog='k', hbo='darkblue', hbr='b'),
-    units=dict(eeg='uV', grad='fT/cm', mag='fT', eog='uV', misc='AU',
-               seeg='mV', dipole='nAm', gof='GOF', emg='uV', ecg='uV',
-               bio='uV', ecog='uV', hbo='uM', hbr='uM'),
+    units=dict(mag='fT', grad='fT/cm', eeg='uV', eog='uV', ecg='uV', emg='uV',
+               misc='AU', seeg='mV', dipole='nAm', gof='GOF', bio='uV',
+               ecog='uV', hbo='uM', hbr='uM', ref_meg='fT'),
+    # scalings for the units
     scalings=dict(mag=1e15, grad=1e13, eeg=1e6, eog=1e6, emg=1e6, ecg=1e6,
                   misc=1.0, seeg=1e3, dipole=1e9, gof=1.0, bio=1e6, ecog=1e6,
-                  hbo=1e6, hbr=1e6),
+                  hbo=1e6, hbr=1e6, ref_meg=1e15),
+    # rough guess for a good plot
     scalings_plot_raw=dict(mag=1e-12, grad=4e-11, eeg=20e-6, eog=150e-6,
-                           ecg=5e-4, emg=1e-3, ref_meg=1e-12, misc=1e-3,
+                           ecg=5e-4, emg=1e-3, ref_meg=1e-12, misc='auto',
                            stim=1, resp=1, chpi=1e-4, exci=1, ias=1, syst=1,
                            seeg=1e-4, bio=1e-6, ecog=1e-4, hbo=10e-6,
                            hbr=10e-6, whitened=10.),
@@ -28,10 +30,11 @@ DEFAULTS = dict(
               misc=(-5., 5.), seeg=(-20., 20.), dipole=(-100., 100.),
               gof=(0., 1.), bio=(-500., 500.), ecog=(-200., 200.), hbo=(0, 20),
               hbr=(0, 20)),
-    titles=dict(eeg='EEG', grad='Gradiometers', mag='Magnetometers',
-                misc='misc', seeg='sEEG', dipole='Dipole', eog='EOG',
-                gof='Goodness of fit', ecg='ECG', emg='EMG', bio='BIO',
-                ecog='ECoG', hbo='Oxyhemoglobin', hbr='Deoxyhemoglobin'),
+    titles=dict(mag='Magnetometers', grad='Gradiometers', eeg='EEG', eog='EOG',
+                ecg='ECG', emg='EMG', misc='misc', seeg='sEEG', bio='BIO',
+                dipole='Dipole', ecog='ECoG', hbo='Oxyhemoglobin',
+                ref_meg='Reference Magnetometers',
+                hbr='Deoxyhemoglobin', gof='Goodness of fit'),
     mask_params=dict(marker='o',
                      markerfacecolor='w',
                      markeredgecolor='k',
@@ -62,6 +65,10 @@ DEFAULTS = dict(
     ),
     noise_std=dict(grad=5e-13, mag=20e-15, eeg=0.2e-6),
     eloreta_options=dict(eps=1e-6, max_iter=20, force_equal=None),
+    depth_mne=dict(exp=0.8, limit=10., limit_depth_chs=True,
+                   combine_xyz='spectral', allow_fixed_depth=False),
+    depth_sparse=dict(exp=0.8, limit=None, limit_depth_chs='whiten',
+                      combine_xyz='fro', allow_fixed_depth=True),
 )
 
 
