@@ -1,4 +1,3 @@
-import os
 import os.path as op
 import gc
 
@@ -12,11 +11,13 @@ from mne import (read_forward_solution, apply_forward, apply_forward_raw,
                  average_forward_solutions, write_forward_solution,
                  convert_forward_solution, SourceEstimate, pick_types_forward,
                  read_evokeds)
+from mne.io import read_info
 from mne.label import read_label
-from mne.utils import (requires_mne, run_subprocess, _TempDir,
+from mne.utils import (requires_mne, run_subprocess,
                        run_tests_if_main)
 from mne.forward import (restrict_forward_to_stc, restrict_forward_to_label,
-                         Forward, is_fixed_orient)
+                         Forward, is_fixed_orient, compute_orient_prior,
+                         compute_depth_prior)
 
 data_path = testing.data_path(download=False)
 fname_meeg = op.join(data_path, 'MEG', 'sample',
@@ -24,15 +25,8 @@ fname_meeg = op.join(data_path, 'MEG', 'sample',
 fname_meeg_grad = op.join(data_path, 'MEG', 'sample',
                           'sample_audvis_trunc-meg-eeg-oct-2-grad-fwd.fif')
 
-fname_raw = op.join(op.dirname(__file__), '..', '..', 'io', 'tests', 'data',
-                    'test_raw.fif')
-
 fname_evoked = op.join(op.dirname(__file__), '..', '..', 'io', 'tests',
                        'data', 'test-ave.fif')
-fname_mri = op.join(data_path, 'MEG', 'sample',
-                    'sample_audvis_trunc-trans.fif')
-subjects_dir = os.path.join(data_path, 'subjects')
-fname_src = op.join(subjects_dir, 'sample', 'bem', 'sample-oct-4-src.fif')
 
 
 def compare_forwards(f1, f2):
@@ -91,9 +85,8 @@ def test_convert_forward():
 
 @pytest.mark.slowtest
 @testing.requires_testing_data
-def test_io_forward():
+def test_io_forward(tmpdir):
     """Test IO for forward solutions."""
-    temp_dir = _TempDir()
     # do extensive tests with MEEG + grad
     n_channels, n_src = 366, 108
     fwd = read_forward_solution(fname_meeg_grad)
@@ -103,7 +96,7 @@ def test_io_forward():
     leadfield = fwd['sol']['data']
     assert_equal(leadfield.shape, (n_channels, n_src))
     assert_equal(len(fwd['sol']['row_names']), n_channels)
-    fname_temp = op.join(temp_dir, 'test-fwd.fif')
+    fname_temp = tmpdir.join('test-fwd.fif')
     with pytest.warns(RuntimeWarning, match='stored on disk'):
         write_forward_solution(fname_temp, fwd, overwrite=True)
 
@@ -173,7 +166,7 @@ def test_io_forward():
 
     # test warnings on bad filenames
     fwd = read_forward_solution(fname_meeg_grad)
-    fwd_badname = op.join(temp_dir, 'test-bad-name.fif.gz')
+    fwd_badname = tmpdir.join('test-bad-name.fif.gz')
     with pytest.warns(RuntimeWarning, match='end with'):
         write_forward_solution(fwd_badname, fwd)
     with pytest.warns(RuntimeWarning, match='end with'):
@@ -236,7 +229,7 @@ def test_apply_forward():
 
 
 @testing.requires_testing_data
-def test_restrict_forward_to_stc():
+def test_restrict_forward_to_stc(tmpdir):
     """Test restriction of source space to source SourceEstimate."""
     start = 0
     stop = 5
@@ -280,8 +273,7 @@ def test_restrict_forward_to_stc():
 
     # Test saving the restricted forward object. This only works if all fields
     # are properly accounted for.
-    temp_dir = _TempDir()
-    fname_copy = op.join(temp_dir, 'copy-fwd.fif')
+    fname_copy = tmpdir.join('copy-fwd.fif')
     with pytest.warns(RuntimeWarning, match='stored on disk'):
         write_forward_solution(fname_copy, fwd_out, overwrite=True)
     fwd_out_read = read_forward_solution(fname_copy)
@@ -291,7 +283,7 @@ def test_restrict_forward_to_stc():
 
 
 @testing.requires_testing_data
-def test_restrict_forward_to_label():
+def test_restrict_forward_to_label(tmpdir):
     """Test restriction of source space to label."""
     fwd = read_forward_solution(fname_meeg)
     fwd = convert_forward_solution(fwd, surf_ori=True, force_fixed=True,
@@ -350,8 +342,7 @@ def test_restrict_forward_to_label():
 
     # Test saving the restricted forward object. This only works if all fields
     # are properly accounted for.
-    temp_dir = _TempDir()
-    fname_copy = op.join(temp_dir, 'copy-fwd.fif')
+    fname_copy = tmpdir.join('copy-fwd.fif')
     write_forward_solution(fname_copy, fwd_out, overwrite=True)
     fwd_out_read = read_forward_solution(fname_copy)
     compare_forwards(fwd_out, fwd_out_read)
@@ -359,9 +350,8 @@ def test_restrict_forward_to_label():
 
 @testing.requires_testing_data
 @requires_mne
-def test_average_forward_solution():
+def test_average_forward_solution(tmpdir):
     """Test averaging forward solutions."""
-    temp_dir = _TempDir()
     fwd = read_forward_solution(fname_meeg)
     # input not a list
     pytest.raises(TypeError, average_forward_solutions, 1)
@@ -383,7 +373,7 @@ def test_average_forward_solution():
 
     # modify a fwd solution, save it, use MNE to average with old one
     fwd_copy['sol']['data'] *= 0.5
-    fname_copy = op.join(temp_dir, 'copy-fwd.fif')
+    fname_copy = str(tmpdir.join('copy-fwd.fif'))
     write_forward_solution(fname_copy, fwd_copy, overwrite=True)
     cmd = ('mne_average_forward_solutions', '--fwd', fname_meeg, '--fwd',
            fname_copy, '--out', fname_copy)
@@ -401,4 +391,41 @@ def test_average_forward_solution():
     compare_forwards(fwd, fwd_ave)
 
 
+@testing.requires_testing_data
+def test_priors():
+    """Test prior computations."""
+    # Depth prior
+    fwd = read_forward_solution(fname_meeg)
+    assert not is_fixed_orient(fwd)
+    n_sources = fwd['nsource']
+    info = read_info(fname_evoked)
+    depth_prior = compute_depth_prior(fwd, info, exp=0.8)
+    assert depth_prior.shape == (3 * n_sources,)
+    depth_prior = compute_depth_prior(fwd, info, exp=0.)
+    assert_array_equal(depth_prior, 1.)
+    with pytest.raises(ValueError, match='must be "whiten"'):
+        compute_depth_prior(fwd, info, limit_depth_chs='foo')
+    with pytest.raises(ValueError, match='noise_cov must be a Covariance'):
+        compute_depth_prior(fwd, info, limit_depth_chs='whiten')
+    fwd_fixed = convert_forward_solution(fwd, force_fixed=True)
+    with pytest.deprecated_call():
+        depth_prior = compute_depth_prior(
+            fwd_fixed['sol']['data'], info, is_fixed_ori=True)
+    assert depth_prior.shape == (n_sources,)
+    # Orientation prior
+    orient_prior = compute_orient_prior(fwd, 1.)
+    assert_array_equal(orient_prior, 1.)
+    orient_prior = compute_orient_prior(fwd_fixed, 0.)
+    assert_array_equal(orient_prior, 1.)
+    with pytest.raises(ValueError, match='oriented in surface coordinates'):
+        compute_orient_prior(fwd, 0.5)
+    fwd_surf_ori = convert_forward_solution(fwd, surf_ori=True)
+    orient_prior = compute_orient_prior(fwd_surf_ori, 0.5)
+    assert all(np.in1d(orient_prior, (0.5, 1.)))
+    with pytest.raises(ValueError, match='between 0 and 1'):
+        compute_orient_prior(fwd_surf_ori, -0.5)
+    with pytest.raises(ValueError, match='with fixed orientation'):
+        compute_orient_prior(fwd_fixed, 0.5)
+
+
 run_tests_if_main()
