@@ -7,8 +7,38 @@ import os
 from ..utils import warn
 
 
+def _check_pyface_backend():
+    """Check the currently selected Pyface backend.
+
+    Returns
+    -------
+    backend : str
+        Name of the backend.
+    result : 0 | 1 | 2
+        0: the backend has been tested and works.
+        1: the backend has not been tested.
+        2: the backend not been tested.
+
+    Notes
+    -----
+    See also http://docs.enthought.com/pyface/.
+    """
+    try:
+        from traitsui.toolkit import toolkit
+        from traits.etsconfig.api import ETSConfig
+    except ImportError:
+        return None, 2
+
+    toolkit()
+    backend = ETSConfig.toolkit
+    if backend == 'qt4':
+        status = 0
+    else:
+        status = 1
+    return backend, status
+
+
 def _check_backend():
-    from ..utils import _check_pyface_backend
     try:
         from pyface.api import warning
     except ImportError:
