@@ -6,14 +6,14 @@
 
 from collections import Counter
 import os
+import queue
 import sys
 
 import numpy as np
 from scipy.linalg import inv
 from threading import Thread
 
-from ..externals.six.moves import queue
-from ..io.meas_info import _read_dig_points, _make_dig_points
+from .._digitization._utils import _read_dig_points, _make_dig_points
 from ..utils import get_config, set_config, logger, warn
 
 from mayavi.core.ui.mayavi_scene import MayaviScene
@@ -35,6 +35,7 @@ from ..io.kit.kit import (RawKIT, KIT, _make_stim_channel, _default_stim_chs,
 from ..transforms import (apply_trans, als_ras_trans,
                           get_ras_to_neuromag_trans, Transform)
 from ..coreg import _decimate_points, fit_matched_points
+from ._backend import _check_pyface_backend
 from ..event import _find_events
 from ._marker_gui import CombineMarkersPanel, CombineMarkersModel
 from ._help import read_tooltips
@@ -42,13 +43,13 @@ from ._viewer import HeadViewController, PointObject
 
 
 use_editor = CheckListEditor(cols=5, values=[(i, str(i)) for i in range(5)])
-backend_is_wx = False  # is there a way to determine this?
-if backend_is_wx:
+
+if _check_pyface_backend()[0] == 'wx':
     # wx backend allows labels for wildcards
     hsp_wildcard = ['Head Shape Points (*.hsp;*.txt)|*.hsp;*.txt']
     elp_wildcard = ['Head Shape Fiducials (*.elp;*.txt)|*.elp;*.txt']
     kit_con_wildcard = ['Continuous KIT Files (*.sqd;*.con)|*.sqd;*.con']
-elif sys.platform in ('win32',  'linux2'):
+if sys.platform in ('win32',  'linux2'):
     # on Windows and Ubuntu, multiple wildcards does not seem to work
     hsp_wildcard = ['*.hsp', '*.txt']
     elp_wildcard = ['*.elp', '*.txt']
@@ -416,7 +417,8 @@ class Kit2FiffModel(HasPrivateTraits):
                     stim_code, self.stim_threshold)
         raw = RawKIT(self.sqd_file, preload=preload, stim=self.stim_chs_array,
                      slope=self.stim_slope, stim_code=stim_code,
-                     stimthresh=self.stim_threshold)
+                     stimthresh=self.stim_threshold,
+                     allow_unknown_format=self.allow_unknown_format)
 
         if np.any(self.fid):
             raw.info['dig'] = _make_dig_points(self.fid[0], self.fid[1],
