@@ -1,6 +1,5 @@
 # -*- coding: utf-8 -*-
 """Conversion tool from Brain Vision EEG to FIF."""
-
 # Authors: Teon Brooks <teon.brooks@gmail.com>
 #          Christian Brodbeck <christianbrodbeck@nyu.edu>
 #          Eric Larson <larson.eric.d@gmail.com>
@@ -11,27 +10,26 @@
 #
 # License: BSD (3-clause)
 
+import configparser
 import os
 import os.path as op
 import re
 from datetime import datetime
 from math import modf
-from functools import partial
+from io import StringIO
 
 import numpy as np
 
-from ...utils import verbose, logger, warn
+from ...utils import verbose, logger, warn, fill_doc, _DefaultEventParser
 from ..constants import FIFF
 from ..meas_info import _empty_info
-from ..base import BaseRaw, _check_update_montage
-from ..utils import (_read_segments_file, _synthesize_stim_channel,
-                     _mult_cal_one, _deprecate_stim_channel)
-from ...annotations import (Annotations, events_from_annotations,
-                            read_annotations)
-from ...externals.six import StringIO, string_types
-from ...externals.six.moves import configparser
+from ..base import BaseRaw
+from ..utils import _read_segments_file, _mult_cal_one, _deprecate_montage
+from ...annotations import Annotations, read_annotations
+from ...channels import make_dig_montage
 
 
+@fill_doc
 class RawBrainVision(BaseRaw):
     """Raw object from Brain Vision EEG file.
 
@@ -39,11 +37,7 @@ class RawBrainVision(BaseRaw):
     ----------
     vhdr_fname : str
         Path to the EEG header file.
-    montage : str | None | instance of Montage
-        Path or instance of montage containing electrode positions. If None,
-        read sensor locations from header file if present, otherwise (0, 0, 0).
-        See the documentation of :func:`mne.channels.read_montage` for more
-        information.
+    %(montage_deprecated)s
     eog : list or tuple
         Names of channels or list of indices that should be designated
         EOG channels. Values should correspond to the vhdr file.
@@ -56,47 +50,8 @@ class RawBrainVision(BaseRaw):
     scale : float
         The scaling factor for EEG data. Unless specified otherwise by
         header file, units are in microvolts. Default scale factor is 1.
-    preload : bool
-        If True, all data are loaded at initialization.
-        If False, data are not read until save.
-    response_trig_shift : int | None
-        An integer that will be added to all response triggers when reading
-        events (stimulus triggers will be unaffected). Use
-        ``trig_shift_by_type={'response': ...}`` instead. If None, response
-        triggers will be ignored. Default is 0 for backwards compatibility,
-        but typically another value or None will be necessary.
-        This is deprecated in 0.17 and will be removed in 0.18.
-    event_id : dict | None
-        Special events to consider in addition to those that follow the normal
-        BrainVision trigger format ('###' with an optional single character
-        prefix, e.g., "111", "S  1", "R128", "S 21"). If dict, the keys will be
-        mapped to trigger values on the stimulus channel.
-        Example: {'SyncStatus': 1, 'Pulse Artifact': 3}.
-        If None or an empty dict (default), only BrainVision format events are
-        added to the stimulus channel. Keys are case sensitive. "New Segment"
-        markers are always dropped.
-        This is deprecated in 0.17 and will be removed in 0.18.
-    trig_shift_by_type: dict | None
-        The names of marker types to which an offset should be added.
-        If dict, the keys specify marker types (case is ignored), so that the
-        corresponding value (an integer) will be added to the trigger value of
-        all events of this type. If the value for a key is in the dict is None,
-        all markers of this type will be ignored. If None (default), no offset
-        is added, which may lead to different marker types being mapped to the
-        same event id.
-        This is deprecated in 0.17 and will be removed in 0.18.
-    stim_channel : bool (default True)
-        Add a stim channel from the events.
-
-        .. warning:: This defaults to True in 0.17 but will change to False in
-                     0.18 (when no stim channel synthesis will be allowed)
-                     and be removed in 0.19; migrate code to use
-                     :func:`mne.events_from_annotations` instead.
-
-        .. versionadded:: 0.17
-    verbose : bool, str, int, or None
-        If not None, override default verbose level (see :func:`mne.verbose`
-        and :ref:`Logging documentation <tut_logging>` for more).
+    %(preload)s
+    %(verbose)s
 
     See Also
     --------
@@ -104,38 +59,17 @@ class RawBrainVision(BaseRaw):
     """
 
     @verbose
-    def __init__(self, vhdr_fname, montage=None,
+    def __init__(self, vhdr_fname, montage='deprecated',
                  eog=('HEOGL', 'HEOGR', 'VEOGb'), misc='auto',
-                 scale=1., preload=False, response_trig_shift=0,
-                 event_id=None, trig_shift_by_type=None, stim_channel=None,
-                 verbose=None):  # noqa: D107
-        stim_channel = _deprecate_stim_channel(stim_channel)
-        if stim_channel and response_trig_shift != 0:
-            warn(
-                "'response_trig_shift' was deprecated in version "
-                "0.17 and will be removed in 0.18. Use "
-                "trig_shift_by_type={{'response': {} }} instead".format(
-                    response_trig_shift), DeprecationWarning)
-            if trig_shift_by_type and 'response' in (
-                    key.lower() for key in trig_shift_by_type):
-                raise ValueError(
-                    'offset for response markers has been specified twice, '
-                    'both using "trig_shift_by_type" and '
-                    '"response_trig_shift"')
-            else:
-                if trig_shift_by_type is None:
-                    trig_shift_by_type = dict()
-                trig_shift_by_type['response'] = response_trig_shift
+                 scale=1., preload=False, verbose=None):  # noqa: D107
         # Channel info and events
         logger.info('Extracting parameters from %s...' % vhdr_fname)
         vhdr_fname = op.abspath(vhdr_fname)
         (info, data_fname, fmt, order, n_samples, mrk_fname, montage,
-         orig_units) = _get_vhdr_info(vhdr_fname, eog, misc, scale, montage,
-                                      stim_channel)
+         orig_units) = _get_vhdr_info(vhdr_fname, eog, misc, scale, montage)
         self._order = order
         self._n_samples = n_samples
 
-        _check_update_montage(info, montage)
         with open(data_fname, 'rb') as f:
             if isinstance(fmt, dict):  # ASCII, this will be slow :(
                 if self._order == 'F':  # multiplexed, channels in columns
@@ -148,20 +82,12 @@ class RawBrainVision(BaseRaw):
                     raise NotImplementedError()
             else:
                 n_data_ch = int(info['nchan'])
-                if stim_channel:
-                    n_data_ch -= 1
                 f.seek(0, os.SEEK_END)
                 n_samples = f.tell()
                 dtype_bytes = _fmt_byte_dict[fmt]
                 offsets = None
                 n_samples = n_samples // (dtype_bytes * n_data_ch)
 
-        # Create a dummy event channel first
-        if stim_channel:
-            self._create_event_ch(np.empty((0, 3)), n_samples)
-        else:
-            self._event_ch = None
-
         super(RawBrainVision, self).__init__(
             info, last_samps=[n_samples - 1], filenames=[data_fname],
             orig_format=fmt, preload=preload, verbose=verbose,
@@ -171,37 +97,18 @@ class RawBrainVision(BaseRaw):
         annots = read_annotations(mrk_fname, info['sfreq'])
         self.set_annotations(annots)
 
-        if stim_channel:
-            # Use events_from_annotations to properly set the events
-            trig_shift_by_type = _check_trig_shift_by_type(trig_shift_by_type)
-            dropped_desc = []  # use to collect dropped descriptions
-            event_id = dict() if event_id is None else event_id
-            event_id = partial(_event_id_func, event_id=event_id,
-                               trig_shift_by_type=trig_shift_by_type,
-                               dropped_desc=dropped_desc)
-            events, _ = events_from_annotations(self, event_id)
-            if len(dropped_desc) > 0:
-                dropped = list(set(dropped_desc))
-                warn("{0} event(s) will be dropped, such as {1}. "
-                     "Consider using the event_id parameter to parse events "
-                     "that do not follow the BrainVision format. For more "
-                     "information, see the docstring of read_raw_brainvision."
-                     .format(len(dropped), dropped[:5]))
-            self._create_event_ch(events, n_samples)
+        _deprecate_montage(self, "read_raw_brainvision", montage)
 
     def _read_segment_file(self, data, idx, fi, start, stop, cals, mult):
         """Read a chunk of raw data."""
         # read data
         n_data_ch = len(self.ch_names)
-        if self._event_ch is not None:
-            n_data_ch -= 1
         if self._order == 'C':
             _read_segments_c(self, data, idx, fi, start, stop, cals, mult)
-        elif isinstance(self.orig_format, string_types):
+        elif isinstance(self.orig_format, str):
             dtype = _fmt_dtype_dict[self.orig_format]
             _read_segments_file(self, data, idx, fi, start, stop, cals, mult,
-                                dtype=dtype, n_channels=n_data_ch,
-                                trigger_ch=self._event_ch)
+                                dtype=dtype, n_channels=n_data_ch)
         else:
             offsets = self._raw_extras[fi]
             with open(self._filenames[fi], 'rb') as fid:
@@ -211,46 +118,8 @@ class RawBrainVision(BaseRaw):
                     line = fid.readline().decode('ASCII')
                     line = line.strip().replace(',', '.').split()
                     block[:n_data_ch, ii] = [float(l) for l in line]
-            if self._event_ch is not None:
-                block[-1] = self._event_ch[start:stop]
             _mult_cal_one(data, block, idx, cals, mult)
 
-    def _get_brainvision_events(self):
-        """Retrieve the events associated with the Brain Vision Raw object.
-
-        Returns
-        -------
-        events : array, shape (n_events, 3)
-            Events, each row consisting of an (onset, duration, trigger)
-            sequence.
-        """
-        return self._events.copy()
-
-    def _set_brainvision_events(self, events):
-        """Set the events and update the synthesized stim channel.
-
-        Parameters
-        ----------
-        events : array, shape (n_events, 3)
-            Events, each row consisting of an (onset, duration, trigger)
-            sequence.
-        """
-        self._create_event_ch(events)
-
-    def _create_event_ch(self, events, n_samp=None):
-        """Create the event channel."""
-        if n_samp is None:
-            n_samp = self.last_samp - self.first_samp + 1
-        events = np.array(events, int)
-
-        if events.ndim != 2 or events.shape[1] != 3:
-            raise ValueError("[n_events x 3] shaped array required")
-        # update events
-        self._event_ch = _synthesize_stim_channel(events, n_samp)
-        self._events = events
-        if getattr(self, 'preload', False):
-            self._data[-1] = self._event_ch
-
 
 def _read_segments_c(raw, data, idx, fi, start, stop, cals, mult):
     """Read chunk of vectorized raw data."""
@@ -258,13 +127,9 @@ def _read_segments_c(raw, data, idx, fi, start, stop, cals, mult):
     dtype = _fmt_dtype_dict[raw.orig_format]
     n_bytes = _fmt_byte_dict[raw.orig_format]
     n_channels = len(raw.ch_names)
-    trigger_ch = raw._event_ch
     block = np.zeros((n_channels, stop - start))
     with open(raw._filenames[fi], 'rb', buffering=0) as fid:
         for ch_id in np.arange(n_channels)[idx]:
-            if trigger_ch is not None and ch_id == n_channels - 1:  # stim
-                block[ch_id] = trigger_ch[start:stop]
-                continue
             fid.seek(start * n_bytes + ch_id * n_bytes * n_samples)
             block[ch_id] = np.fromfile(fid, dtype, stop - start)
 
@@ -287,8 +152,9 @@ def _read_vmrk(fname):
         The onsets in seconds.
     description : array, shape (n_annots,)
         The description of each annotation.
-    orig_time : str
-        The origin time as a string.
+    date_str : str
+        The recording time as a string. Defaults to empty string if no
+        recording time is found.
     """
     # read vmrk file
     with open(fname, 'rb') as fid:
@@ -330,7 +196,8 @@ def _read_vmrk(fname):
     # extract Marker Infos block
     m = re.search(r"\[Marker Infos\]", txt, re.IGNORECASE)
     if not m:
-        return np.zeros((0, 3))
+        return np.array(list()), np.array(list()), np.array(list()), ''
+
     mk_txt = txt[m.end():]
     m = re.search(r"^\[.*\]$", mk_txt)
     if m:
@@ -339,13 +206,15 @@ def _read_vmrk(fname):
     # extract event information
     items = re.findall(r"^Mk\d+=(.*)", mk_txt, re.MULTILINE)
     onset, duration, description = list(), list(), list()
-    date_str = None
+    date_str = ''
     for info in items:
-        mtype, mdesc, this_onset, this_duration = info.split(',')[:4]
-        if date_str is None and mtype == 'New Segment':
+        info_data = info.split(',')
+        mtype, mdesc, this_onset, this_duration = info_data[:4]
+        if date_str == '' and len(info_data) == 5 and mtype == 'New Segment':
             # to handle the origin of time and handle the presence of multiple
-            # New Segment annotations. We only keep the first one for date_str.
-            date_str = info.split(',')[-1]
+            # New Segment annotations. We only keep the first one that is
+            # different from an empty string for date_str.
+            date_str = info_data[-1]
 
         this_duration = (int(this_duration)
                          if this_duration.isdigit() else 0)
@@ -356,98 +225,6 @@ def _read_vmrk(fname):
     return np.array(onset), np.array(duration), np.array(description), date_str
 
 
-def _event_id_func(desc, event_id, trig_shift_by_type, dropped_desc):
-    """Get integers from string description.
-
-    This function can be passed as event_id to events_from_annotations
-    function.
-
-    Parameters
-    ----------
-    desc : str
-        The description of the event.
-    event_id : dict
-        The default mapping from desc to integer.
-    trig_shift_by_type: dict | None
-        The names of marker types to which an offset should be added.
-        If dict, the keys specify marker types (case is ignored), so that the
-        corresponding value (an integer) will be added to the trigger value of
-        all events of this type. If the value for a key is in the dict is None,
-        all markers of this type will be ignored. If None (default), no offset
-        is added, which may lead to different marker types being mapped to the
-        same event id.
-    dropped_desc : list
-        Used to log the dropped descriptions.
-
-    Returns
-    -------
-    trigger : int | None
-        The integer corresponding to the specific event. If None,
-        then a proper integer cannot be found and the event is typically
-        ignored.
-    """
-    mtype, mdesc = desc.split('/')
-    found = False
-    if (mdesc in event_id) or (mtype == "New Segment"):
-        trigger = event_id.get(mdesc, None)
-        found = True
-    else:
-        try:
-            # Match any three digit marker value (padded with whitespace).
-            # In BrainVision Recorder, the markers sometimes have a prefix
-            # depending on the type, e.g., Stimulus=S, Response=R,
-            # Optical=O, ... Note that any arbitrary stimulus type can be
-            # defined. So we match any single character that is not
-            # forbidden by BrainVision Recorder: [^a-z$%\-@/\\|;,:.\s]
-            marker_regexp = r'^[^a-z$%\-@/\\|;,:.\s]{0,1}([\s\d]{2}\d{1})$'
-            trigger = int(re.findall(marker_regexp, mdesc)[0])
-        except IndexError:
-            trigger = None
-
-    if mtype.lower() in trig_shift_by_type:
-        cur_shift = trig_shift_by_type[mtype.lower()]
-        if cur_shift is not None:
-            trigger += cur_shift
-        else:
-            # The trigger has been deliberately shifted to None. Do not
-            # add this to "dropped" so we do not warn about something
-            # that was done deliberately. Just continue with next item.
-            trigger = None
-            found = True
-
-    if trigger is None and not found:
-        dropped_desc.append(desc)
-
-    return trigger
-
-
-def _check_trig_shift_by_type(trig_shift_by_type):
-    """Check the trig_shift_by_type parameter.
-
-    trig_shift_by_type is used to offset event numbers depending
-    of the type of marker (eg. Response, Stimulus).
-    """
-    if trig_shift_by_type is None:
-        trig_shift_by_type = dict()
-    elif not isinstance(trig_shift_by_type, dict):
-        raise TypeError("'trig_shift_by_type' must be None or dict")
-
-    for mrk_type in list(trig_shift_by_type.keys()):
-        cur_shift = trig_shift_by_type[mrk_type]
-        if not isinstance(cur_shift, int) and cur_shift is not None:
-            raise TypeError('shift for type {} must be int or None'.format(
-                mrk_type
-            ))
-        mrk_type_lc = mrk_type.lower()
-        if mrk_type_lc != mrk_type:
-            if mrk_type_lc in trig_shift_by_type:
-                raise ValueError('marker type {} specified twice with'
-                                 'different case'.format(mrk_type_lc))
-            trig_shift_by_type[mrk_type_lc] = cur_shift
-            del trig_shift_by_type[mrk_type]
-    return trig_shift_by_type
-
-
 def _read_annotations_brainvision(fname, sfreq='auto'):
     """Create Annotations from BrainVision vrmk.
 
@@ -485,7 +262,6 @@ def _read_annotations_brainvision(fname, sfreq='auto'):
     annotations = Annotations(onset=onset, duration=duration,
                               description=description,
                               orig_time=orig_time)
-
     return annotations
 
 
@@ -514,7 +290,7 @@ def _check_mrk_version(header):
             'Brain Vision Data Exchange Marker File, Version 2.0',
             'BrainVision Data Exchange Marker File, Version 1.0']
     if header not in tags:
-        raise ValueError("Currently only support %r, not %r"
+        raise ValueError("Currently, MNE-Python only supports %r, not %r"
                          "Contact MNE-Developers for support."
                          % (str(tags), header))
 
@@ -538,10 +314,16 @@ _unit_dict = {'V': 1.,  # V stands for Volt
 def _str_to_meas_date(date_str):
     date_str = date_str.strip()
 
-    if date_str in ['0', '00000000000000000000']:
+    if date_str in ['', '0', '00000000000000000000']:
         return None
 
-    meas_date = datetime.strptime(date_str, '%Y%m%d%H%M%S%f')
+    try:
+        meas_date = datetime.strptime(date_str, '%Y%m%d%H%M%S%f')
+    except ValueError as e:
+        if 'does not match format' in str(e):
+            return None
+        else:
+            raise
 
     # We need list of unix time in milliseconds and as second entry
     # the additional amount of microseconds
@@ -590,10 +372,7 @@ def _aux_vhdr_info(vhdr_fname):
     else:
         params, settings = settings, ''
     cfg = configparser.ConfigParser()
-    if hasattr(cfg, 'read_file'):  # newer API
-        cfg.read_file(StringIO(params))
-    else:
-        cfg.readfp(StringIO(params))
+    cfg.read_file(StringIO(params))
 
     # get sampling info
     # Sampling interval is given in microsec
@@ -608,7 +387,7 @@ def _aux_vhdr_info(vhdr_fname):
     return settings, cfg, cinfostr, info
 
 
-def _get_vhdr_info(vhdr_fname, eog, misc, scale, montage, stim_channel):
+def _get_vhdr_info(vhdr_fname, eog, misc, scale, montage):
     """Extract all the information from the header file.
 
     Parameters
@@ -629,10 +408,8 @@ def _get_vhdr_info(vhdr_fname, eog, misc, scale, montage, stim_channel):
     montage : str | None | instance of Montage
         Path or instance of montage containing electrode positions. If None,
         read sensor locations from header file if present, otherwise (0, 0, 0).
-        See the documentation of :func:`mne.channels.read_montage` for more
-        information.
-    stim_channel : bool
-        See reader.
+        See the documentation of :func:`mne.channels.read_dig_captrack` for
+        more information.
 
     Returns
     -------
@@ -679,8 +456,8 @@ def _get_vhdr_info(vhdr_fname, eog, misc, scale, montage, stim_channel):
             raise NotImplementedError('BrainVision files with ASCII data in '
                                       'vectorized order (i.e. channels in rows'
                                       ') are not supported yet.')
-        fmt = dict((key, cfg.get('ASCII Infos', key))
-                   for key in cfg.options('ASCII Infos'))
+        fmt = {key: cfg.get('ASCII Infos', key)
+               for key in cfg.options('ASCII Infos')}
 
     # locate EEG binary file and marker file for the stim channel
     path = op.dirname(vhdr_fname)
@@ -689,7 +466,7 @@ def _get_vhdr_info(vhdr_fname, eog, misc, scale, montage, stim_channel):
 
     # Try to get measurement date from marker file
     # Usually saved with a marker "New Segment", see BrainVision documentation
-    regexp = r'^Mk\d+=New Segment,.*,\d+,\d+,\d+,(\d{20})$'
+    regexp = r'^Mk\d+=New Segment,.*,\d+,\d+,-?\d+,(\d{20})$'
     with open(mrk_fname, 'r') as tmp_mrk_f:
         lines = tmp_mrk_f.readlines()
 
@@ -697,7 +474,7 @@ def _get_vhdr_info(vhdr_fname, eog, misc, scale, montage, stim_channel):
         match = re.findall(regexp, line.strip())
 
         # Always take first measurement date we find
-        if match and match[0] != '00000000000000000000':
+        if match:
             date_str = match[0]
             info['meas_date'] = _str_to_meas_date(date_str)
             break
@@ -718,7 +495,6 @@ def _get_vhdr_info(vhdr_fname, eog, misc, scale, montage, stim_channel):
                 fid.seek(0, 2)
                 n_bytes = fid.tell()
                 n_samples = n_bytes // _fmt_byte_dict[fmt] // nchan
-    nchan = nchan + int(stim_channel)  # augment with stim channel
 
     ch_names = [''] * nchan
     cals = np.empty(nchan)
@@ -752,37 +528,45 @@ def _get_vhdr_info(vhdr_fname, eog, misc, scale, montage, stim_channel):
                               else FIFF.FIFF_UNIT_NONE)
     misc = list(misc_chs.keys()) if misc == 'auto' else misc
 
-    # create montage
-    if cfg.has_section('Coordinates') and montage is None:
+    # create montage: 'Coordinates' section in VHDR file corresponds to "BVEF"
+    # BrainVision Electrode File. The data are based on BrainVision Analyzer
+    # coordinate system: Defined between standard electrode positions: X-axis
+    # from T7 to T8, Y-axis from Oz to Fpz, Z-axis orthogonal from XY-plane
+    # through Cz, fit to a sphere if idealized (when radius=1), specified in mm
+    if cfg.has_section('Coordinates') and montage in (None, 'deprecated'):
         from ...transforms import _sph_to_cart
-        from ...channels.montage import Montage
         montage_pos = list()
         montage_names = list()
         to_misc = list()
+        # Go through channels
         for ch in cfg.items('Coordinates'):
             ch_name = ch_dict[ch[0]]
             montage_names.append(ch_name)
-            radius, theta, phi = [float(c) for c in ch[1].split(',')]
             # 1: radius, 2: theta, 3: phi
+            rad, theta, phi = [float(c) for c in ch[1].split(',')]
             pol = np.deg2rad(theta)
             az = np.deg2rad(phi)
-            pos = _sph_to_cart(np.array([[radius * 85., az, pol]]))[0]
+            # Coordinates could be "idealized" (spherical head model)
+            if rad == 1:
+                # scale up to realistic head radius (8.5cm == 85mm)
+                rad *= 85.
+            pos = _sph_to_cart(np.array([[rad, az, pol]]))[0]
             if (pos == 0).all() and ch_name not in list(eog) + misc:
                 to_misc.append(ch_name)
             montage_pos.append(pos)
-        montage_sel = np.arange(len(montage_pos))
-        montage = Montage(montage_pos, montage_names, 'Brainvision',
-                          montage_sel)
+        # Make a montage, normalizing from BrainVision units "mm" to "m", the
+        # unit used for montages in MNE
+        montage_pos = np.array(montage_pos) / 1e3
+        montage = make_dig_montage(
+            ch_pos=dict(zip(montage_names, montage_pos)),
+            coord_frame='head'
+        )
         if len(to_misc) > 0:
             misc += to_misc
             warn('No coordinate information found for channels {}. '
                  'Setting channel types to misc. To avoid this warning, set '
                  'channel types explicitly.'.format(to_misc))
 
-    if stim_channel:
-        ch_names[-1] = 'STI 014'
-        cals[-1] = 1.
-        ranges[-1] = 1.
     if np.isnan(cals).any():
         raise RuntimeError('Missing channel units')
 
@@ -843,7 +627,7 @@ def _get_vhdr_info(vhdr_fname, eog, misc, scale, montage, stim_channel):
         hp_s = '[s]' in header[hp_col]
         lp_s = '[s]' in header[lp_col]
 
-        for i, ch in enumerate(ch_names[:-1], 1):
+        for i, ch in enumerate(ch_names, 1):
             line = re.split(divider, settings[idx + i])
             # double check alignment with channel by using the hw settings
             if idx == idx_amp:
@@ -1000,21 +784,17 @@ def _get_vhdr_info(vhdr_fname, eog, misc, scale, montage, stim_channel):
             orig_units)
 
 
-def read_raw_brainvision(vhdr_fname, montage=None,
+@fill_doc
+def read_raw_brainvision(vhdr_fname, montage='deprecated',
                          eog=('HEOGL', 'HEOGR', 'VEOGb'), misc='auto',
-                         scale=1., preload=False, response_trig_shift=0,
-                         event_id=None, trig_shift_by_type=None,
-                         stim_channel=None, verbose=None):
+                         scale=1., preload=False, verbose=None):
     """Reader for Brain Vision EEG file.
 
     Parameters
     ----------
     vhdr_fname : str
         Path to the EEG header file.
-    montage : str | None | instance of Montage
-        Path or instance of montage containing electrode positions.
-        If None, sensor locations are (0,0,0). See the documentation of
-        :func:`mne.channels.read_montage` for more information.
+    %(montage_deprecated)s
     eog : list or tuple of str
         Names of channels or list of indices that should be designated
         EOG channels. Values should correspond to the vhdr file
@@ -1027,46 +807,8 @@ def read_raw_brainvision(vhdr_fname, montage=None,
     scale : float
         The scaling factor for EEG data. Unless specified otherwise by
         header file, units are in microvolts. Default scale factor is 1.
-    preload : bool
-        If True, all data are loaded at initialization.
-        If False, data are not read until save.
-    response_trig_shift : int | None
-        An integer that will be added to all response triggers when reading
-        events (stimulus triggers will be unaffected). Use
-        ``trig_shift_by_type={'response': ...}`` instead. If None, response
-        triggers will be ignored. Default is 0 for backwards compatibility,
-        but typically another value or None will be necessary.
-        This was deprecated in 0.17 and will be removed in 0.18.
-    event_id : dict | None
-        Special events to consider in addition to those that follow the normal
-        BrainVision trigger format ('###' with an optional single character
-        prefix). If dict, the keys will be mapped to trigger values on the
-        stimulus channel. Example: {'SyncStatus': 1, 'Pulse Artifact': 3}.
-        If None or an empty dict (default), only BrainVision format events are
-        added to the stimulus channel. Keys are case sensitive. "New Segment"
-        markers are always dropped.
-        This was deprecated in 0.17 and will be removed in 0.18.
-    trig_shift_by_type : dict | None
-        The names of marker types to which an offset should be added.
-        If dict, the keys specify marker types (case is ignored), so that the
-        corresponding value (an integer) will be added to the trigger value of
-        all events of this type. If the value for a key is in the dict is None,
-        all markers of this type will be ignored. If None (default), no offset
-        is added, which may lead to different marker types being mapped to the
-        same event id.
-        This was deprecated in 0.17 and will be removed in 0.18.
-    stim_channel : bool (default True)
-        Add a stim channel from the events.
-
-        .. warning:: This defaults to True in 0.17 but will change to False in
-                     0.18 (when no stim channel synthesis will be allowed)
-                     and be removed in 0.19; migrate code to use
-                     :func:`mne.events_from_annotations` instead.
-
-        .. versionadded:: 0.17
-    verbose : bool, str, int, or None
-        If not None, override default verbose level (see :func:`mne.verbose`
-        and :ref:`Logging documentation <tut_logging>` for more).
+    %(preload)s
+    %(verbose)s
 
     Returns
     -------
@@ -1080,7 +822,38 @@ def read_raw_brainvision(vhdr_fname, montage=None,
     """
     return RawBrainVision(vhdr_fname=vhdr_fname, montage=montage, eog=eog,
                           misc=misc, scale=scale, preload=preload,
-                          response_trig_shift=response_trig_shift,
-                          event_id=event_id, verbose=verbose,
-                          trig_shift_by_type=trig_shift_by_type,
-                          stim_channel=stim_channel)
+                          verbose=verbose)
+
+
+_BV_EVENT_IO_OFFSETS = {'Event/': 0, 'Stimulus/S': 0, 'Response/R': 1000,
+                        'Optic/O': 2000}
+_OTHER_ACCEPTED_MARKERS = {
+    'New Segment/': 99999, 'SyncStatus/Sync On': 99998
+}
+_OTHER_OFFSET = 10001  # where to start "unknown" event_ids
+
+
+class _BVEventParser(_DefaultEventParser):
+    """Parse standard brainvision events, accounting for non-standard ones."""
+
+    def __call__(self, description):
+        """Parse BrainVision event codes (like `Stimulus/S 11`) to ints."""
+        offsets = _BV_EVENT_IO_OFFSETS
+
+        maybe_digit = description[-3:].strip()
+        kind = description[:-3]
+        if maybe_digit.isdigit() and kind in offsets:
+            code = int(maybe_digit) + offsets[kind]
+        elif description in _OTHER_ACCEPTED_MARKERS:
+            code = _OTHER_ACCEPTED_MARKERS[description]
+        else:
+            code = (super(_BVEventParser, self)
+                    .__call__(description, offset=_OTHER_OFFSET))
+        return code
+
+
+def _check_bv_annot(descriptions):
+    markers_basename = set([dd.rstrip('0123456789 ') for dd in descriptions])
+    bv_markers = (set(_BV_EVENT_IO_OFFSETS.keys())
+                  .union(set(_OTHER_ACCEPTED_MARKERS.keys())))
+    return len(markers_basename - bv_markers) == 0
