@@ -1,34 +1,28 @@
 # -*- coding: utf-8 -*-
+"""Test reading of BrainVision format."""
 # Author: Teon Brooks <teon.brooks@gmail.com>
 #         Stefan Appelhoff <stefan.appelhoff@mailbox.org>
 #
 # License: BSD (3-clause)
-
-from functools import partial
-import inspect
 import os.path as op
+from os import unlink
 import shutil
 
 import numpy as np
 from numpy.testing import (assert_array_almost_equal, assert_array_equal,
                            assert_allclose, assert_equal)
 import pytest
+from tempfile import NamedTemporaryFile
 
 from mne.utils import _TempDir, run_tests_if_main
-from mne import pick_types, find_events, read_annotations
+from mne import pick_types, read_annotations, concatenate_raws
 from mne.io.constants import FIFF
 from mne.io import read_raw_fif, read_raw_brainvision
 from mne.io.tests.test_raw import _test_raw_reader
 from mne.datasets import testing
+from mne.annotations import events_from_annotations
 
-# Helper for deprecation
-read_raw_brainvision_orig = read_raw_brainvision
-read_raw_brainvision = partial(read_raw_brainvision_orig, stim_channel=False)
-read_raw_brainvision_old = partial(read_raw_brainvision_orig,
-                                   stim_channel=True)
-
-FILE = inspect.getfile(inspect.currentframe())
-data_dir = op.join(op.dirname(op.abspath(FILE)), 'data')
+data_dir = op.join(op.dirname(__file__), 'data')
 vhdr_path = op.join(data_dir, 'test.vhdr')
 vmrk_path = op.join(data_dir, 'test.vmrk')
 eeg_path = op.join(data_dir, 'test.eeg')
@@ -60,10 +54,18 @@ neuroone_vhdr = op.join(data_path, 'Brainvision', 'test_NO.vhdr')
 # Test for nanovolts as unit
 vhdr_nV_path = op.join(data_dir, 'test_nV.vhdr')
 
-montage = op.join(data_dir, 'test.hpts')
+# Test bad date
+vhdr_bad_date = op.join(data_dir, 'test_bad_date.vhdr')
+
 eeg_bin = op.join(data_dir, 'test_bin_raw.fif')
 eog = ['HL', 'HR', 'Vb']
-event_id = {'Sync On': 5}
+
+# XXX: BUG we cannot parse test.hpts FastSCAN file to create a DigMontage
+#      (plus I've removed montage from all the read_raw_brainvision and nothing
+#       broke, so we were not testing that set_montage in brainvision was
+#       working)
+#      This should be amend in its own PR.
+montage = op.join(data_dir, 'test.hpts')
 
 
 def test_orig_units(recwarn):
@@ -80,29 +82,79 @@ def test_orig_units(recwarn):
     assert orig_units['ReRef'] == 'C'
 
 
-def test_vmrk_meas_date():
+DATE_TEST_CASES = np.array([
+    ('Mk1=New Segment,,1,1,0,20131113161403794232\n',  # content
+     [1384359243, 794231],  # meas_date internal representation
+     '2013-11-13 16:14:03 GMT'),  # meas_date representation
+
+    (('Mk1=New Segment,,1,1,0,20070716122240937454\n'
+      'Mk2=New Segment,,2,1,0,20070716122240937455\n'),
+     [1184588560, 937453],
+     '2007-07-16 12:22:40 GMT'),
+
+    ('Mk1=New Segment,,1,1,0,\nMk2=New Segment,,2,1,0,20070716122240937454\n',
+     [1184588560, 937453],
+     '2007-07-16 12:22:40 GMT'),
+
+    ('Mk1=STATUS,,1,1,0\n', None, 'unspecified'),
+    ('Mk1=New Segment,,1,1,0,\n', None, 'unspecified'),
+    ('Mk1=New Segment,,1,1,0\n', None, 'unspecified'),
+    ('Mk1=New Segment,,1,1,0,00000000000304125000', None, 'unspecified'),
+
+], dtype=np.dtype({
+    'names': ['content', 'meas_date', 'meas_date_repr'],
+    'formats': [object, object, 'U22']
+}))
+
+
+@pytest.fixture(scope='session')
+def _mocked_meas_date_data(tmpdir_factory):
+    """Prepare files for mocked_meas_date_file fixture."""
+    # Prepare the files
+    tmpdir = str(tmpdir_factory.mktemp('brainvision_mocked_meas_date'))
+    vhdr_fname, vmrk_fname, eeg_fname = [
+        op.join(tmpdir, op.basename(ff))
+        for ff in [vhdr_path, vmrk_path, eeg_path]
+    ]
+    for orig, dest in zip([vhdr_path, eeg_path], [vhdr_fname, eeg_fname]):
+        shutil.copyfile(orig, dest)
+
+    # Get the marker information
+    with open(vmrk_path, 'r') as fin:
+        lines = fin.readlines()
+
+    return vhdr_fname, vmrk_fname, lines
+
+
+@pytest.fixture(scope='session', params=[tt for tt in DATE_TEST_CASES])
+def mocked_meas_date_file(_mocked_meas_date_data, request):
+    """Prepare a generator for use in test_meas_date."""
+    MEAS_DATE_LINE = 11  # see test.vmrk file
+    vhdr_fname, vmrk_fname, lines = _mocked_meas_date_data
+
+    lines[MEAS_DATE_LINE] = request.param['content']
+    with open(vmrk_fname, 'w') as fout:
+        fout.writelines(lines)
+
+    yield (
+        vhdr_fname, request.param['meas_date'], request.param['meas_date_repr']
+    )
+
+
+def test_meas_date(mocked_meas_date_file):
     """Test successful extraction of measurement date."""
-    # Test file that does have a specific date
-    raw = read_raw_brainvision(vhdr_path)
-    assert_allclose(raw.info['meas_date'], [1384359243, 794231])
-    assert '2013-11-13 16:14:03 GMT' in repr(raw.info)
-
-    # Test file with multiple dates ... we should only take the first
-    with pytest.warns(RuntimeWarning, match='software filter'):
-        raw = read_raw_brainvision(vhdr_old_path)
-    assert_allclose(raw.info['meas_date'], [1184588560, 937453])
-    assert '2007-07-16 12:22:40 GMT' in repr(raw.info)
-
-    # Test files with no date, we should get DATE_NONE from mne.io.write
-    with pytest.warns(RuntimeWarning, match='coordinate information'):
-        raw = read_raw_brainvision(vhdr_v2_path)
-    assert raw.info['meas_date'] is None
-    assert 'unspecified' in repr(raw.info)
+    vhdr_f, expected_meas, expected_meas_repr = mocked_meas_date_file
+    raw = read_raw_brainvision(vhdr_f)
+    assert expected_meas_repr in repr(raw.info)
+    if expected_meas is None:
+        assert raw.info['meas_date'] is None
+    else:
+        assert_allclose(raw.info['meas_date'], expected_meas)
 
 
 def test_vhdr_codepage_ansi():
     """Test BV reading with ANSI codepage."""
-    raw_init = read_raw_brainvision(vhdr_path, event_id=event_id)
+    raw_init = read_raw_brainvision(vhdr_path)
     data_expected, times_expected = raw_init[:]
     tempdir = _TempDir()
     ansi_vhdr_path = op.join(tempdir, op.split(vhdr_path)[-1])
@@ -127,7 +179,7 @@ def test_vhdr_codepage_ansi():
                     line = b'Codepage=ANSI\n'
                 fout.write(line)
 
-    raw = read_raw_brainvision(ansi_vhdr_path, event_id=event_id)
+    raw = read_raw_brainvision(ansi_vhdr_path)
     data_new, times_new = raw[:]
 
     assert_equal(raw_init.ch_names, raw.ch_names)
@@ -137,7 +189,7 @@ def test_vhdr_codepage_ansi():
 
 def test_ascii():
     """Test ASCII BV reading."""
-    raw = read_raw_brainvision(vhdr_path, event_id=event_id)
+    raw = read_raw_brainvision(vhdr_path)
     tempdir = _TempDir()
     ascii_vhdr_path = op.join(tempdir, op.split(vhdr_path)[-1])
     # copy marker file
@@ -169,7 +221,7 @@ def test_ascii():
                             for ch_name in raw.ch_names) + b'\n')
         fid.write(b'\n'.join(b' '.join(b'%.3f' % dd for dd in d)
                              for d in data.T / raw._cals))
-    raw = read_raw_brainvision(ascii_vhdr_path, event_id=event_id)
+    raw = read_raw_brainvision(ascii_vhdr_path)
     data_new, times_new = raw[:]
     assert_allclose(data_new, data, atol=1e-15)
     assert_allclose(times_new, times)
@@ -179,8 +231,8 @@ def test_brainvision_data_highpass_filters():
     """Test reading raw Brain Vision files with amplifier filter settings."""
     # Homogeneous highpass in seconds (default measurement unit)
     raw = _test_raw_reader(
-        read_raw_brainvision, vhdr_fname=vhdr_highpass_path,
-        montage=montage, eog=eog)
+        read_raw_brainvision, vhdr_fname=vhdr_highpass_path, eog=eog
+    )
 
     assert_equal(raw.info['highpass'], 1. / (2 * np.pi * 10))
     assert_equal(raw.info['lowpass'], 250.)
@@ -188,8 +240,8 @@ def test_brainvision_data_highpass_filters():
     # Heterogeneous highpass in seconds (default measurement unit)
     with pytest.warns(RuntimeWarning, match='different .*pass filters') as w:
         raw = _test_raw_reader(
-            read_raw_brainvision_old, vhdr_fname=vhdr_mixed_highpass_path,
-            montage=montage, eog=eog, event_id=event_id)
+            read_raw_brainvision, vhdr_fname=vhdr_mixed_highpass_path,
+            eog=eog)
 
     lowpass_warning = ['different lowpass filters' in str(ww.message)
                        for ww in w]
@@ -206,7 +258,7 @@ def test_brainvision_data_highpass_filters():
     # Homogeneous highpass in Hertz
     raw = _test_raw_reader(
         read_raw_brainvision, vhdr_fname=vhdr_highpass_hz_path,
-        montage=montage, eog=eog, event_id=event_id)
+        eog=eog)
 
     assert_equal(raw.info['highpass'], 10.)
     assert_equal(raw.info['lowpass'], 250.)
@@ -214,8 +266,8 @@ def test_brainvision_data_highpass_filters():
     # Heterogeneous highpass in Hertz
     with pytest.warns(RuntimeWarning, match='different .*pass filters') as w:
         raw = _test_raw_reader(
-            read_raw_brainvision_old, vhdr_fname=vhdr_mixed_highpass_hz_path,
-            montage=montage, eog=eog, event_id=event_id)
+            read_raw_brainvision, vhdr_fname=vhdr_mixed_highpass_hz_path,
+            eog=eog)
 
     trigger_warning = ['will be dropped' in str(ww.message)
                        for ww in w]
@@ -236,8 +288,8 @@ def test_brainvision_data_lowpass_filters():
     """Test files with amplifier LP filter settings."""
     # Homogeneous lowpass in Hertz (default measurement unit)
     raw = _test_raw_reader(
-        read_raw_brainvision, vhdr_fname=vhdr_lowpass_path,
-        montage=montage, eog=eog, event_id=event_id)
+        read_raw_brainvision, vhdr_fname=vhdr_lowpass_path, eog=eog
+    )
 
     assert_equal(raw.info['highpass'], 1. / (2 * np.pi * 10))
     assert_equal(raw.info['lowpass'], 250.)
@@ -245,8 +297,8 @@ def test_brainvision_data_lowpass_filters():
     # Heterogeneous lowpass in Hertz (default measurement unit)
     with pytest.warns(RuntimeWarning) as w:  # event parsing
         raw = _test_raw_reader(
-            read_raw_brainvision_old, vhdr_fname=vhdr_mixed_lowpass_path,
-            montage=montage, eog=eog, event_id=event_id)
+            read_raw_brainvision, vhdr_fname=vhdr_mixed_lowpass_path, eog=eog
+        )
 
     lowpass_warning = ['different lowpass filters' in str(ww.message)
                        for ww in w]
@@ -262,8 +314,8 @@ def test_brainvision_data_lowpass_filters():
 
     # Homogeneous lowpass in seconds
     raw = _test_raw_reader(
-        read_raw_brainvision, vhdr_fname=vhdr_lowpass_s_path,
-        montage=montage, eog=eog, event_id=event_id)
+        read_raw_brainvision, vhdr_fname=vhdr_lowpass_s_path, eog=eog
+    )
 
     assert_equal(raw.info['highpass'], 1. / (2 * np.pi * 10))
     assert_equal(raw.info['lowpass'], 1. / (2 * np.pi * 0.004))
@@ -271,8 +323,8 @@ def test_brainvision_data_lowpass_filters():
     # Heterogeneous lowpass in seconds
     with pytest.warns(RuntimeWarning) as w:  # filter settings
         raw = _test_raw_reader(
-            read_raw_brainvision_old, vhdr_fname=vhdr_mixed_lowpass_s_path,
-            montage=montage, eog=eog, event_id=event_id)
+            read_raw_brainvision, vhdr_fname=vhdr_mixed_lowpass_s_path, eog=eog
+        )
 
     lowpass_warning = ['different lowpass filters' in str(ww.message)
                        for ww in w]
@@ -292,8 +344,8 @@ def test_brainvision_data_partially_disabled_hw_filters():
     with pytest.warns(RuntimeWarning) as w:  # event parsing
         raw = _test_raw_reader(
             read_raw_brainvision,
-            vhdr_fname=vhdr_partially_disabled_hw_filter_path,
-            montage=montage, eog=eog)
+            vhdr_fname=vhdr_partially_disabled_hw_filter_path, eog=eog
+        )
 
     trigger_warning = ['will be dropped' in str(ww.message)
                        for ww in w]
@@ -324,12 +376,12 @@ def test_brainvision_data_software_filters_latin1_global_units():
 def test_brainvision_data():
     """Test reading raw Brain Vision files."""
     pytest.raises(IOError, read_raw_brainvision, vmrk_path)
-    pytest.raises(ValueError, read_raw_brainvision, vhdr_path, montage,
+    pytest.raises(ValueError, read_raw_brainvision, vhdr_path,
                   preload=True, scale="foo")
 
     raw_py = _test_raw_reader(
-        read_raw_brainvision, vhdr_fname=vhdr_path, montage=montage,
-        eog=eog, misc='auto', event_id=event_id)
+        read_raw_brainvision, vhdr_fname=vhdr_path, eog=eog, misc='auto'
+    )
 
     assert ('RawBrainVision' in repr(raw_py))
 
@@ -367,14 +419,12 @@ def test_brainvision_data():
 
     # test loading v2
     read_raw_brainvision(vhdr_v2_path, eog=eog, preload=True,
-                         event_id=event_id,
-                         trig_shift_by_type={'response': 1000},
                          verbose='error')
     # For the nanovolt unit test we use the same data file with a different
     # header file.
     raw_nV = _test_raw_reader(
-        read_raw_brainvision, vhdr_fname=vhdr_nV_path, montage=montage,
-        eog=eog, misc='auto', event_id=event_id)
+        read_raw_brainvision, vhdr_fname=vhdr_nV_path, eog=eog, misc='auto'
+    )
     assert_equal(raw_nV.info['chs'][0]['ch_name'], 'FP1')
     assert_equal(raw_nV.info['chs'][0]['kind'], FIFF.FIFFV_EEG_CH)
     data_nanovolt, _ = raw_nV[0]
@@ -422,197 +472,29 @@ def test_brainvision_vectorized_data():
     assert_array_almost_equal(raw._data[:, :2], first_two_samples_all_chs)
 
 
-def test_events():
-    """Test reading and modifying events."""
-    tempdir = _TempDir()
-    # Note: BrainVision event offsets are 1-based, mne offsets are 0-based.
-    # So in all tests below, the "onset" is 1 less than what's in the file
+def test_coodinates_extraction():
+    """Test reading of [Coordinates] section if present."""
+    # vhdr 2 has a Coordinates section
+    with pytest.warns(RuntimeWarning, match='coordinate information'):
+        raw = read_raw_brainvision(vhdr_v2_path)
 
-    # check that events are read and stim channel is synthesized correctly
-    raw = read_raw_brainvision_old(vhdr_path, eog=eog, event_id=event_id)
-    events = raw._get_brainvision_events()
-    events = events[events[:, 2] != event_id['Sync On']]
-    assert_array_equal(events, [[486, 0, 253],
-                                [496, 0, 255],
-                                [1769, 0, 254],
-                                [1779, 0, 255],
-                                [3252, 0, 254],
-                                [3262, 0, 255],
-                                [4935, 0, 253],
-                                [4945, 0, 255],
-                                [5999, 0, 255],
-                                [6619, 0, 254],
-                                [6629, 0, 255],
-                                [7699, 0, 1]])
+    # Basic check of extracted coordinates
+    assert raw.info['dig'] is not None
+    diglist = raw.info['dig']
+    coords = np.array([dig['r'] for dig in diglist])
+    EXPECTED_SHAPE = (
+        len(raw.ch_names) - 4,  # HL, HR, Vb, ReRef are not set in dig
+        3,
+    )
+    assert coords.shape == EXPECTED_SHAPE
 
-    # check that events are read and stim channel is synthesized correctly and
-    # response triggers are shifted using the deprecated response_trig_shift.
-    with pytest.deprecated_call():
-        raw = read_raw_brainvision_old(vhdr_path, eog=eog,
-                                       response_trig_shift=1000,
-                                       event_id=event_id)
-    events = raw._get_brainvision_events()
-    assert_array_equal(events, [[486, 0, 253],
-                                [496, 0, 255],
-                                [1769, 0, 254],
-                                [1779, 0, 255],
-                                [3252, 0, 254],
-                                [3262, 0, 255],
-                                [4935, 0, 253],
-                                [4945, 0, 255],
-                                [5999, 0, 1255],
-                                [6619, 0, 254],
-                                [6629, 0, 255],
-                                [7629, 0, 5],
-                                [7699, 0, 1]])
+    # Make sure the scaling seems right
+    # a coordinate more than 20cm away from origin is implausible
+    assert coords.max() < 0.2
 
-    # check that trig_shift_by_type works as well
-    raw = read_raw_brainvision_old(vhdr_path, eog=eog,
-                                   trig_shift_by_type={'response': 1000,
-                                                       'Optic': 2000},
-                                   event_id=event_id)
-    events = raw._get_brainvision_events()
-    assert_array_equal(events, [[486, 0, 253],
-                                [496, 0, 255],
-                                [1769, 0, 254],
-                                [1779, 0, 255],
-                                [3252, 0, 254],
-                                [3262, 0, 255],
-                                [4935, 0, 253],
-                                [4945, 0, 255],
-                                [5999, 0, 1255],
-                                [6619, 0, 254],
-                                [6629, 0, 255],
-                                [7629, 0, 5],
-                                [7699, 0, 2001]])
-
-    # Check that we warn if a trigger is dropped
-    with pytest.warns(RuntimeWarning, match='will be dropped'):
-        raw = read_raw_brainvision_old(vhdr_path)
-    # check that events are read and stim channel is synthesized correctly and
-    # response triggers are ignored.
-    raw = read_raw_brainvision_old(vhdr_path, eog=eog, event_id=event_id,
-                                   trig_shift_by_type={'response': None})
-    events = raw._get_brainvision_events()
-    events = events[events[:, 2] != event_id['Sync On']]
-    assert_array_equal(events, [[486, 0, 253],
-                                [496, 0, 255],
-                                [1769, 0, 254],
-                                [1779, 0, 255],
-                                [3252, 0, 254],
-                                [3262, 0, 255],
-                                [4935, 0, 253],
-                                [4945, 0, 255],
-                                [6619, 0, 254],
-                                [6629, 0, 255],
-                                [7699, 0, 1]])
-
-    # Error handling of trig_shift_by_type
-    pytest.raises(TypeError, read_raw_brainvision_old, vhdr_path, eog=eog,
-                  preload=True, trig_shift_by_type=1)
-    pytest.raises(TypeError, read_raw_brainvision_old, vhdr_path, eog=eog,
-                  preload=True, trig_shift_by_type={'response': 0.1})
-    pytest.raises(TypeError, read_raw_brainvision_old, vhdr_path, eog=eog,
-                  preload=True, trig_shift_by_type={'response': np.nan})
-    pytest.raises(ValueError, read_raw_brainvision_old, vhdr_path, eog=eog,
-                  preload=True, trig_shift_by_type={'response': 1000,
-                                                    'Response': 1001})
-    with pytest.warns(DeprecationWarning):
-        pytest.raises(ValueError, read_raw_brainvision_old, vhdr_path, eog=eog,
-                      preload=True, trig_shift_by_type={'response': 1000},
-                      response_trig_shift=1001)
-
-    # Check that events of type "Comment" are read if they contain square
-    # brackets (which usually signify a new section within a BrainVision file)
-    # If no event_id specified, skip the marker and continue as planned
-    with pytest.warns(RuntimeWarning, match='channel types to misc'):
-        raw = read_raw_brainvision_old(vhdr_v2_path)
-    events = raw._get_brainvision_events()
-    assert events.shape == (11, 3)  # shape of events without comment
-
-    # with event_id specified, get that comment and assert it's there
-    tmp_event_id = {'comment using [square] brackets': 999}
-    with pytest.warns(RuntimeWarning, match='channel types to misc'):
-        raw = read_raw_brainvision_old(vhdr_v2_path, event_id=tmp_event_id)
-    events = raw._get_brainvision_events()
-    assert 999 in events[:, -1]
-    assert events.shape == (12, 3)  # shape of events with comment
-
-    # check that events are read properly when event_id is specified for
-    # auxiliary events
-    raw = read_raw_brainvision_old(vhdr_path, eog=eog, preload=True,
-                                   trig_shift_by_type={'response': None},
-                                   event_id=event_id)
-    events = raw._get_brainvision_events()
-
-    expected_events = np.array([[486, 0, 253],
-                                [496, 0, 255],
-                                [1769, 0, 254],
-                                [1779, 0, 255],
-                                [3252, 0, 254],
-                                [3262, 0, 255],
-                                [4935, 0, 253],
-                                [4945, 0, 255],
-                                [6619, 0, 254],
-                                [6629, 0, 255],
-                                [7629, 0, 5],
-                                [7699, 0, 1]])
-
-    assert_array_equal(events, expected_events)
-
-    # Test that both trig_shift_by_type and event_id can be set
-    read_raw_brainvision(vhdr_path, eog=eog, preload=False,
-                         trig_shift_by_type={'response': 100},
-                         event_id=event_id)
-    mne_events = find_events(raw, stim_channel='STI 014')
-    assert_array_equal(events[:, [0, 2]], mne_events[:, [0, 2]])
-
-    # modify events and check that stim channel is updated
-    index = events[:, 2] == 255
-    events = events[index]
-    raw._set_brainvision_events(events)
-    mne_events = find_events(raw, stim_channel='STI 014')
-    assert_array_equal(events[:, [0, 2]], mne_events[:, [0, 2]])
-
-    # remove events
-    nchan = raw.info['nchan']
-    ch_name = raw.info['chs'][-2]['ch_name']
-    events = np.empty((0, 3))
-    raw._set_brainvision_events(events)
-    assert_equal(raw.info['nchan'], nchan)
-    assert_equal(len(raw._data), nchan)
-    assert_equal(raw.info['chs'][-2]['ch_name'], ch_name)
-    assert_equal(len(find_events(raw, 'STI 014')), 0)
-    assert_allclose(raw[-1][0], 0.)
-    fname = op.join(tempdir, 'evt_raw.fif')
-    raw.save(fname)
-
-    # add events back in
-    events = [[10, 1, 2]]
-    raw._set_brainvision_events(events)
-    assert_equal(raw.info['nchan'], nchan)
-    assert_equal(len(raw._data), nchan)
-    assert_equal(raw.info['chs'][-1]['ch_name'], 'STI 014')
-
-    # degenerate
-    with pytest.raises(TypeError, match='stim_channel must be an instance of'):
-        read_raw_brainvision(vhdr_path, stim_channel='foo')
-
-
-def test_brainvision_with_montage():
-    """Test reading embedded montage information."""
-    with pytest.warns(RuntimeWarning, match='Omitted 3 annot'):
-        raw = read_raw_brainvision(vhdr_v2_path, eog=eog, misc=['ReRef'])
-    for i, d in enumerate(raw.info['dig'], 1):
-        assert_equal(d['coord_frame'], FIFF.FIFFV_COORD_HEAD)
-        assert_equal(d['ident'], i)
-        assert_equal(d['kind'], FIFF.FIFFV_POINT_EEG)
-        assert_equal(len(d['r']), 3)
-
-    raw_none = read_raw_brainvision(vhdr_v2_path, verbose='error')
-    for r, n in zip(raw.info['chs'], raw_none.info['chs']):
-        if r['kind'] != n['kind']:
-            assert_array_equal(r['loc'], n['loc'])
+    # vhdr 1 does not have a Coordinates section
+    raw2 = read_raw_brainvision(vhdr_path)
+    assert raw2.info['dig'] is None
 
 
 @testing.requires_testing_data
@@ -628,36 +510,102 @@ def test_brainvision_neuroone_export():
 def test_read_vmrk_annotations():
     """Test load brainvision annotations."""
     sfreq = 1000.0
-    annotations = read_annotations(vmrk_path, sfreq=sfreq)
-    assert annotations.orig_time == 1384359243.794231
-    expected = np.array([0, 486., 496., 1769., 1779., 3252., 3262., 4935.,
-                         4945., 5999., 6619., 6629., 7629., 7699.]) / sfreq
-    description = ['New Segment/',
-                   'Stimulus/S253', 'Stimulus/S255', 'Stimulus/S254',
-                   'Stimulus/S255', 'Stimulus/S254', 'Stimulus/S255',
-                   'Stimulus/S253', 'Stimulus/S255', 'Response/R255',
-                   'Stimulus/S254', 'Stimulus/S255',
-                   'SyncStatus/Sync On', 'Optic/O  1']
-    assert_array_almost_equal(annotations.onset,
-                              expected, decimal=7)
-    assert_array_equal(annotations.description, description)
 
-    # Test automatic detection of sfreq from header file
-    annotations_auto = read_annotations(vmrk_path)
-    assert_array_equal(annotations.onset, annotations_auto.onset)
+    # Test vmrk file without annotations
+    # delete=False is for Windows compatibility
+    with open(vmrk_path) as myfile:
+        head = [next(myfile) for x in range(6)]
+    with NamedTemporaryFile(mode='w+', suffix='.vmrk', delete=False) as temp:
+        for item in head:
+            temp.write(item)
+        temp.seek(0)
+        read_annotations(temp.name, sfreq=sfreq)
+    try:
+        temp.close()
+        unlink(temp.name)
+    except IOError:
+        pass
 
 
-def test_read_raw_brainvision_warn_on_marker_drop():
-    """Test that loading test.vhdr warns a summary of each dropped type."""
-    EXPECTED_WARN_MSG = (r"1 event\(s\) will be dropped, such as "
-                         r"\[\'SyncStatus\/Sync On\'\]. Consider using the "
-                         r"event\_id parameter to parse events that do not "
-                         r"follow the BrainVision format. For more "
-                         r"information, see the docstring of "
-                         r"read_raw_brainvision.")
-    with pytest.warns(RuntimeWarning, match=EXPECTED_WARN_MSG) as recwarn:
-        read_raw_brainvision_old(vhdr_path, verbose=True)
-    assert len(recwarn) == 1
+@testing.requires_testing_data
+def test_read_vhdr_annotations_and_events():
+    """Test load brainvision annotations and parse them to events."""
+    sfreq = 1000.0
+    expected_orig_time = 1384359243.794231
+    expected_onset_latency = np.array(
+        [0, 486., 496., 1769., 1779., 3252., 3262., 4935., 4945., 5999., 6619.,
+         6629., 7629., 7699.]
+    )
+    expected_annot_description = [
+        'New Segment/', 'Stimulus/S253', 'Stimulus/S255', 'Event/254',
+        'Stimulus/S255', 'Event/254', 'Stimulus/S255', 'Stimulus/S253',
+        'Stimulus/S255', 'Response/R255', 'Event/254', 'Stimulus/S255',
+        'SyncStatus/Sync On', 'Optic/O  1'
+    ]
+    expected_events = np.stack([
+        expected_onset_latency,
+        np.zeros_like(expected_onset_latency),
+        [99999, 253, 255, 254, 255, 254, 255, 253, 255, 1255, 254, 255, 99998,
+         2001],
+    ]).astype('int64').T
+    expected_event_id = {'New Segment/': 99999, 'Stimulus/S253': 253,
+                         'Stimulus/S255': 255, 'Event/254': 254,
+                         'Response/R255': 1255, 'SyncStatus/Sync On': 99998,
+                         'Optic/O  1': 2001}
 
+    raw = read_raw_brainvision(vhdr_path, eog=eog)
+
+    # validate annotations
+    assert raw.annotations.orig_time == expected_orig_time
+    assert_allclose(raw.annotations.onset, expected_onset_latency / sfreq)
+    assert_array_equal(raw.annotations.description, expected_annot_description)
+
+    # validate event extraction
+    events, event_id = events_from_annotations(raw)
+    assert_array_equal(events, expected_events)
+    assert event_id == expected_event_id
+
+    # validate that None gives us a sorted list
+    expected_none_event_id = {desc: idx + 1 for idx, desc in enumerate(sorted(
+        event_id.keys()))}
+    events, event_id = events_from_annotations(raw, event_id=None)
+    assert event_id == expected_none_event_id
+
+    # Add some custom ones, plus a 2-digit one
+    s_10 = 'Stimulus/S 10'
+    raw.annotations.append([1, 2, 3], 10, ['ZZZ', s_10, 'YYY'])
+    expected_event_id.update(YYY=10001, ZZZ=10002)  # others starting at 10001
+    expected_event_id[s_10] = 10
+    _, event_id = events_from_annotations(raw)
+    assert event_id == expected_event_id
+
+    # Concatenating two shouldn't change the resulting event_id
+    # (BAD and EDGE should be ignored)
+    with pytest.warns(RuntimeWarning, match='expanding outside'):
+        raw_concat = concatenate_raws([raw.copy(), raw.copy()])
+    _, event_id = events_from_annotations(raw_concat)
+    assert event_id == expected_event_id
+
+
+@testing.requires_testing_data
+def test_automatic_vmrk_sfreq_recovery():
+    """Test proper sfreq inference by checking the onsets."""
+    assert_array_equal(read_annotations(vmrk_path, sfreq='auto'),
+                       read_annotations(vmrk_path, sfreq=1000.0))
+
+
+@testing.requires_testing_data
+def test_event_id_stability_when_save_and_fif_reload(tmpdir):
+    """Test load events from brainvision annotations when read_raw_fif."""
+    fname = op.join(str(tmpdir), 'bv-raw.fif')
+    raw = read_raw_brainvision(vhdr_path, eog=eog)
+    original_events, original_event_id = events_from_annotations(raw)
+
+    raw.save(fname)
+    raw = read_raw_fif(fname)
+    events, event_id = events_from_annotations(raw)
+
+    assert event_id == original_event_id
+    assert_array_equal(events, original_events)
 
 run_tests_if_main()
