@@ -1,26 +1,11 @@
-# Authors: Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
+# Authors: Alexandre Gramfort <alexandre.gramfort@inria.fr>
 #          Matti Hamalainen <msh@nmr.mgh.harvard.edu>
 #
 # License: BSD (3-clause)
 
+from ..utils._bunch import BunchConstNamed
 
-class Bunch(dict):
-    """Dictionnary-like object thatexposes its keys as attributes."""
-
-    def __init__(self, **kwargs):  # noqa: D102
-        dict.__init__(self, kwargs)
-        self.__dict__ = self
-
-
-class BunchConst(Bunch):
-    """Class to prevent us from re-defining constants (DRY)."""
-
-    def __setattr__(self, attr, val):  # noqa: D105
-        if attr != '__dict__' and hasattr(self, attr):
-            raise AttributeError('Attribute "%s" already set' % attr)
-        super(BunchConst, self).__setattr__(attr, val)
-
-FIFF = BunchConst()
+FIFF = BunchConstNamed()
 
 #
 # FIFF version number in use
@@ -51,9 +36,14 @@ FIFF.FIFFB_EVENTS             = 115
 FIFF.FIFFB_INDEX              = 116
 FIFF.FIFFB_DACQ_PARS          = 117
 FIFF.FIFFB_REF                = 118
-FIFF.FIFFB_SMSH_RAW_DATA      = 119
-FIFF.FIFFB_SMSH_ASPECT        = 120
+FIFF.FIFFB_IAS_RAW_DATA       = 119
+FIFF.FIFFB_IAS_ASPECT         = 120
 FIFF.FIFFB_HPI_SUBSYSTEM      = 121
+# FIFF.FIFFB_PHANTOM_SUBSYSTEM  = 122
+# FIFF.FIFFB_STATUS_SUBSYSTEM   = 123
+FIFF.FIFFB_DEVICE             = 124
+FIFF.FIFFB_HELIUM             = 125
+FIFF.FIFFB_CHANNEL_INFO       = 126
 
 FIFF.FIFFB_SPHERE             = 300   # Concentric sphere model related
 FIFF.FIFFB_BEM                = 310   # Boundary-element method
@@ -77,7 +67,7 @@ FIFF.FIFFB_SSS_INFO           = 502
 FIFF.FIFFB_SSS_CAL            = 503
 FIFF.FIFFB_SSS_ST_INFO        = 504
 FIFF.FIFFB_SSS_BASES          = 505
-FIFF.FIFFB_SMARTSHIELD        = 510
+FIFF.FIFFB_IAS                = 510
 #
 # Of general interest
 #
@@ -105,6 +95,16 @@ FIFF.FIFF_REF_FILE_NAME   = 118
 FIFF.FIFF_DACQ_PARS      = 150
 FIFF.FIFF_DACQ_STIM      = 151
 
+FIFF.FIFF_DEVICE_TYPE    = 152
+FIFF.FIFF_DEVICE_MODEL   = 153
+FIFF.FIFF_DEVICE_SERIAL  = 154
+FIFF.FIFF_DEVICE_SITE    = 155
+
+FIFF.FIFF_HE_LEVEL_RAW   = 156
+FIFF.FIFF_HELIUM_LEVEL   = 157
+FIFF.FIFF_ORIG_FILE_GUID = 158
+FIFF.FIFF_UTC_OFFSET     = 159
+
 FIFF.FIFF_NCHAN       = 200
 FIFF.FIFF_SFREQ       = 201
 FIFF.FIFF_DATA_PACK   = 202
@@ -281,11 +281,12 @@ FIFF.FIFF_REF_PATH           = 1101
 FIFF.FIFFV_ASPECT_AVERAGE       = 100  # Normal average of epochs
 FIFF.FIFFV_ASPECT_STD_ERR       = 101  # Std. error of mean
 FIFF.FIFFV_ASPECT_SINGLE        = 102  # Single epoch cut out from the continuous data
-FIFF.FIFFV_ASPECT_SUBAVERAGE    = 103
+FIFF.FIFFV_ASPECT_SUBAVERAGE    = 103  # Partial average (subaverage)
 FIFF.FIFFV_ASPECT_ALTAVERAGE    = 104  # Alternating subaverage
 FIFF.FIFFV_ASPECT_SAMPLE        = 105  # A sample cut out by graph
 FIFF.FIFFV_ASPECT_POWER_DENSITY = 106  # Power density spectrum
 FIFF.FIFFV_ASPECT_DIPOLE_WAVE   = 200  # Dipole amplitude curve
+
 #
 # BEM surface IDs
 #
@@ -322,7 +323,7 @@ FIFF.FIFFV_MNE_SURF_LEFT_HEMI     = 101
 FIFF.FIFFV_MNE_SURF_RIGHT_HEMI    = 102
 FIFF.FIFFV_MNE_SURF_MEG_HELMET    = 201               # Use this irrespective of the system
 #
-#   These relate to the Isotrak data
+#   These relate to the Isotrak data (enum(point))
 #
 FIFF.FIFFV_POINT_CARDINAL = 1
 FIFF.FIFFV_POINT_HPI      = 2
@@ -330,10 +331,13 @@ FIFF.FIFFV_POINT_EEG      = 3
 FIFF.FIFFV_POINT_ECG      = FIFF.FIFFV_POINT_EEG
 FIFF.FIFFV_POINT_EXTRA    = 4
 FIFF.FIFFV_POINT_HEAD     = 5  # Point on the surface of the head
-
+#
+# Cardinal point types (enum(cardinal_point))
+#
 FIFF.FIFFV_POINT_LPA = 1
 FIFF.FIFFV_POINT_NASION = 2
 FIFF.FIFFV_POINT_RPA = 3
+FIFF.FIFFV_POINT_INION = 4
 #
 #   SSP
 #
@@ -673,7 +677,7 @@ FIFF.FIFFV_MNE_COORD_KIT_HEAD    = FIFF.FIFFV_MNE_COORD_CTF_HEAD
 #   FWD Types
 #
 
-FWD = BunchConst()
+FWD = BunchConstNamed()
 
 FWD.COIL_UNKNOWN                 = 0
 FWD.COILC_UNKNOWN                = 0
@@ -810,8 +814,10 @@ FIFF.FIFFV_COIL_EEG_BIPOLAR           = 5  # Bipolar EEG lead
 FIFF.FIFFV_COIL_DIPOLE             = 200  # Time-varying dipole definition
 # The coil info contains dipole location (r0) and
 # direction (ex)
-FIFF.FIFFV_COIL_FNIRS_HBO               = 300  # fNIRS oxyhemoglobin
-FIFF.FIFFV_COIL_FNIRS_HBR               = 301  # fNIRS deoxyhemoglobin
+FIFF.FIFFV_COIL_FNIRS_HBO             = 300  # fNIRS oxyhemoglobin
+FIFF.FIFFV_COIL_FNIRS_HBR             = 301  # fNIRS deoxyhemoglobin
+FIFF.FIFFV_COIL_FNIRS_RAW             = 302  # fNIRS raw light intensity
+FIFF.FIFFV_COIL_FNIRS_OD              = 303  # fNIRS optical density
 
 FIFF.FIFFV_COIL_MCG_42             = 1000  # For testing the MCG software
 
@@ -822,10 +828,12 @@ FIFF.FIFFV_COIL_VV_PLANAR_W        = 3011  # VV prototype wirewound planar senso
 FIFF.FIFFV_COIL_VV_PLANAR_T1       = 3012  # Vectorview SQ20483N planar gradiometer
 FIFF.FIFFV_COIL_VV_PLANAR_T2       = 3013  # Vectorview SQ20483N-A planar gradiometer
 FIFF.FIFFV_COIL_VV_PLANAR_T3       = 3014  # Vectorview SQ20950N planar gradiometer
+FIFF.FIFFV_COIL_VV_PLANAR_T4       = 3015  # Vectorview planar gradiometer (MEG-MRI)
 FIFF.FIFFV_COIL_VV_MAG_W           = 3021  # VV prototype wirewound magnetometer
 FIFF.FIFFV_COIL_VV_MAG_T1          = 3022  # Vectorview SQ20483N magnetometer
 FIFF.FIFFV_COIL_VV_MAG_T2          = 3023  # Vectorview SQ20483-A magnetometer
 FIFF.FIFFV_COIL_VV_MAG_T3          = 3024  # Vectorview SQ20950N magnetometer
+FIFF.FIFFV_COIL_VV_MAG_T4          = 3025  # Vectorview magnetometer (MEG-MRI)
 
 FIFF.FIFFV_COIL_MAGNES_MAG         = 4001  # Magnes WH magnetometer
 FIFF.FIFFV_COIL_MAGNES_GRAD        = 4002  # Magnes WH gradiometer
@@ -868,14 +876,19 @@ FIFF.FIFFV_COIL_ARTEMIS123_GRAD         = 7501
 FIFF.FIFFV_COIL_ARTEMIS123_REF_MAG      = 7502
 FIFF.FIFFV_COIL_ARTEMIS123_REF_GRAD     = 7503
 #
+# QuSpin sensors
+#
+FIFF.FIFFV_COIL_QUSPIN_ZFOPM_MAG   = 8001
+FIFF.FIFFV_COIL_QUSPIN_ZFOPM_MAG2  = 8002
+#
 # KRISS sensors
 #
 FIFF.FIFFV_COIL_KRISS_GRAD         = 9001
 #
 # Compumedics adult/pediatric gradiometer
 #
-FIFF.FIFFV_COIL_COMPUMEDICS_ADULT_GRAD      = 10001
-FIFF.FIFFV_COIL_COMPUMEDICS_PEDIATRIC_GRAD  = 10002
+FIFF.FIFFV_COIL_COMPUMEDICS_ADULT_GRAD      = 9101
+FIFF.FIFFV_COIL_COMPUMEDICS_PEDIATRIC_GRAD  = 9102
 
 # MNE RealTime
 FIFF.FIFF_MNE_RT_COMMAND           = 3700  # realtime command
@@ -884,6 +897,7 @@ FIFF.FIFF_MNE_RT_CLIENT_ID         = 3701  # realtime client
 # MNE epochs bookkeeping
 FIFF.FIFF_MNE_EPOCHS_SELECTION     = 3800  # the epochs selection
 FIFF.FIFF_MNE_EPOCHS_DROP_LOG      = 3801  # the drop log
+FIFF.FIFF_MNE_EPOCHS_REJECT_FLAT   = 3802  # rejection and flat params
 
 # MNE annotations
 FIFF.FIFFB_MNE_ANNOTATIONS         = 3810  # annotations block
