@@ -4,4 +4,4 @@
 #
 # License: BSD (3-clause)
 
-from .edf import read_raw_edf, find_edf_events
+from .edf import read_raw_edf, read_raw_bdf, read_raw_gdf
