@@ -2,4 +2,4 @@
 
 # Author: Mainak Jas <mainak.jas@telecom-paristech.fr>
 
-from .eeglab import read_raw_eeglab, read_epochs_eeglab, read_events_eeglab
+from .eeglab import read_raw_eeglab, read_epochs_eeglab
