@@ -4,13 +4,13 @@
 
 import inspect
 import os
-from mne.externals.six.moves import cPickle as pickle
+import pickle
 
 import pytest
 from numpy.testing import assert_array_equal
 
 from mne.io.kit import read_mrk
-from mne.io.meas_info import _write_dig_points
+from mne._digitization._utils import _write_dig_points
 from mne.utils import _TempDir
 
 
