@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Authors: Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
+# Authors: Alexandre Gramfort <alexandre.gramfort@inria.fr>
 #          Matti Hamalainen <msh@nmr.mgh.harvard.edu>
 #          Teon Brooks <teon.brooks@gmail.com>
 #          Stefan Appelhoff <stefan.appelhoff@mailbox.org>
@@ -9,9 +9,8 @@
 from collections import Counter
 from copy import deepcopy
 import datetime
+from io import BytesIO
 import operator
-import os.path as op
-import re
 
 import numpy as np
 from scipy import linalg
@@ -20,7 +19,7 @@ from .pick import channel_type
 from .constants import FIFF
 from .open import fiff_open
 from .tree import dir_tree_find
-from .tag import read_tag, find_tag
+from .tag import read_tag, find_tag, _coord_dict
 from .proj import _read_proj, _write_proj, _uniquify_projs, _normalize_proj
 from .ctf_comp import read_ctf_comp, write_ctf_comp
 from .write import (start_file, end_file, start_block, end_block,
@@ -28,13 +27,17 @@ from .write import (start_file, end_file, start_block, end_block,
                     write_coord_trans, write_ch_info, write_name_list,
                     write_julian, write_float_matrix, write_id, DATE_NONE)
 from .proc_history import _read_proc_history, _write_proc_history
-from ..transforms import _to_const
-from ..transforms import invert_transform
+from ..transforms import invert_transform, Transform
 from ..utils import logger, verbose, warn, object_diff, _validate_type
-from .. import __version__
-from ..externals.six import b, BytesIO, string_types, text_type
+from .._digitization.base import _format_dig_points
 from .compensator import get_current_comp
 
+# XXX: most probably the functions needing this, should go somewhere else
+from .._digitization.base import _dig_kind_proper, _dig_kind_rev
+from .._digitization.base import _dig_kind_ints
+from .._digitization._utils import _read_dig_fif
+
+b = bytes  # alias
 
 _kind_dict = dict(
     eeg=(FIFF.FIFFV_EEG_CH, FIFF.FIFFV_COIL_EEG, FIFF.FIFF_UNIT_V),
@@ -58,15 +61,16 @@ _kind_dict = dict(
 def _get_valid_units():
     """Get valid units according to the International System of Units (SI).
 
-    The International System of Units (SI, [1]) is the default system for
+    The International System of Units (SI, [1]_) is the default system for
     describing units in the Brain Imaging Data Structure (BIDS). For more
-    information, see the BIDS specification [2] and the appendix "Units"
+    information, see the BIDS specification [2]_ and the appendix "Units"
     therein.
 
     References
     ----------
     [1] .. https://en.wikipedia.org/wiki/International_System_of_Units
-    [2] .. http://bids.neuroimaging.io/bids_spec.pdf
+    [2] .. https://bids-specification.readthedocs.io/en/stable/
+
     """
     valid_prefix_names = ['yocto', 'zepto', 'atto', 'femto', 'pico', 'nano',
                           'micro', 'milli', 'centi', 'deci', 'deca', 'hecto',
@@ -124,6 +128,35 @@ def _stamp_to_dt(stamp):
             datetime.timedelta(0, 0, stamp[1]))  # day, sec, μs
 
 
+def _unique_channel_names(ch_names):
+    """Ensure unique channel names."""
+    FIFF_CH_NAME_MAX_LENGTH = 15
+    unique_ids = np.unique(ch_names, return_index=True)[1]
+    if len(unique_ids) != len(ch_names):
+        dups = {ch_names[x]
+                for x in np.setdiff1d(range(len(ch_names)), unique_ids)}
+        warn('Channel names are not unique, found duplicates for: '
+             '%s. Applying running numbers for duplicates.' % dups)
+        for ch_stem in dups:
+            overlaps = np.where(np.array(ch_names) == ch_stem)[0]
+            # We need an extra character since we append '-'.
+            # np.ceil(...) is the maximum number of appended digits.
+            n_keep = (FIFF_CH_NAME_MAX_LENGTH - 1 -
+                      int(np.ceil(np.log10(len(overlaps)))))
+            n_keep = min(len(ch_stem), n_keep)
+            ch_stem = ch_stem[:n_keep]
+            for idx, ch_idx in enumerate(overlaps):
+                ch_name = ch_stem + '-%s' % idx
+                if ch_name not in ch_names:
+                    ch_names[ch_idx] = ch_name
+                else:
+                    raise ValueError('Adding a running number for a '
+                                     'duplicate resulted in another '
+                                     'duplicate name %s' % ch_name)
+
+    return ch_names
+
+
 # XXX Eventually this should be de-duplicated with the MNE-MATLAB stuff...
 class Info(dict):
     """Measurement information.
@@ -207,6 +240,10 @@ class Info(dict):
         1970-01-01 00:00:00) denoting the date and time at which the
         measurement was taken. The second element is the additional number of
         microseconds.
+    utc_offset : str
+        "UTC offset of related meas_date (sHH:MM).
+
+        .. versionadded:: 0.19
     meas_id : dict | None
         The ID assigned to this measurement by the acquisition system or
         during file conversion. Follows the same format as ``file_id``.
@@ -227,6 +264,14 @@ class Info(dict):
     subject_info : dict | None
         Information about the subject.
         See Notes for details.
+    device_info : dict | None
+        Information about the acquisition device. See Notes for details.
+
+        .. versionadded:: 0.19
+    helium_info : dict | None
+        Information about the device helium. See Notes for details.
+
+        .. versionadded:: 0.19
 
     See Also
     --------
@@ -268,7 +313,7 @@ class Info(dict):
         unit : int
             The unit to use, e.g. ``FIFF_UNIT_T_M``.
         unit_mul : int
-            Unit multipliers, most commontly ``FIFF_UNITM_NONE``.
+            Unit multipliers, most commonly ``FIFF_UNITM_NONE``.
 
     * ``comps`` list of dict:
 
@@ -284,16 +329,19 @@ class Info(dict):
         save_calibrated : bool
             Were the compensation data saved in calibrated form.
 
-    * ``dig`` dict:
+    * ``dig`` list of dict:
 
         kind : int
-            Digitization kind, e.g. ``FIFFV_POINT_EXTRA``.
+            The kind of channel,
+            e.g. ``FIFFV_POINT_EEG``, ``FIFFV_POINT_CARDINAL``.
+        r : array, shape (3,)
+            3D position in m. and coord_frame.
         ident : int
-            Identifier.
-        r : ndarary, shape (3,)
-            Position.
+            Number specifying the identity of the point.
+            e.g. ``FIFFV_POINT_NASION`` if kind is ``FIFFV_POINT_CARDINAL``, or
+            42 if kind is ``FIFFV_POINT_EEG``.
         coord_frame : int
-            Coordinate frame, e.g. ``FIFFV_COORD_HEAD``.
+            The coordinate frame used, e.g. ``FIFFV_COORD_HEAD``.
 
     * ``events`` list of dict:
 
@@ -421,6 +469,28 @@ class Info(dict):
         hand : int
             Handedness (1=right, 2=left).
 
+    * ``device_info`` dict:
+
+        type : str
+            Device type.
+        model : str
+            Device model.
+        serial : str
+            Device serial.
+        site : str
+            Device site.
+
+    * ``helium_info`` dict:
+
+        he_level_raw : float
+            Helium level (%) before position correction.
+        helium_level : float
+            Helium level (%) after position correction.
+        orig_file_guid : str
+            Original file GUID.
+        meas_date : tuple of int
+            The helium level meas date.
+
     """
 
     def copy(self):
@@ -475,6 +545,13 @@ class Info(dict):
             elif k == 'kit_system_id' and v is not None:
                 from .kit.constants import KIT_SYSNAMES
                 entr = '%i (%s)' % (v, KIT_SYSNAMES.get(v, 'unknown'))
+            elif k == 'dig' and v is not None:
+                counts = Counter(d['kind'] for d in v)
+                counts = ['%d %s' % (counts[ii],
+                                     _dig_kind_proper[_dig_kind_rev[ii]])
+                          for ii in _dig_kind_ints if ii in counts]
+                counts = (' (%s)' % (', '.join(counts))) if len(counts) else ''
+                entr = '%d items%s' % (len(v), counts)
             else:
                 this_len = (len(v) if hasattr(v, '__len__') else
                             ('%s' % v if v is not None else None))
@@ -529,22 +606,9 @@ class Info(dict):
         self._check_ch_name_length()
 
         # make sure channel names are unique
-        unique_ids = np.unique(self['ch_names'], return_index=True)[1]
-        if len(unique_ids) != self['nchan']:
-            dups = set(self['ch_names'][x]
-                       for x in np.setdiff1d(range(self['nchan']), unique_ids))
-            warn('Channel names are not unique, found duplicates for: '
-                 '%s. Applying running numbers for duplicates.' % dups)
-            for ch_stem in dups:
-                overlaps = np.where(np.array(self['ch_names']) == ch_stem)[0]
-                n_keep = min(len(ch_stem),
-                             14 - int(np.ceil(np.log10(len(overlaps)))))
-                ch_stem = ch_stem[:n_keep]
-                for idx, ch_idx in enumerate(overlaps):
-                    ch_name = ch_stem + '-%s' % idx
-                    assert ch_name not in self['ch_names']
-                    self['ch_names'][ch_idx] = ch_name
-                    self['chs'][ch_idx]['ch_name'] = ch_name
+        self['ch_names'] = _unique_channel_names(self['ch_names'])
+        for idx, ch_name in enumerate(self['ch_names']):
+            self['chs'][idx]['ch_name'] = ch_name
 
         if 'filename' in self:
             warn('the "filename" key is misleading '
@@ -573,7 +637,9 @@ def _simplify_info(info):
     chs = [{key: ch[key]
             for key in ('ch_name', 'kind', 'unit', 'coil_type', 'loc')}
            for ch in info['chs']]
-    sub_info = Info(chs=chs, bads=info['bads'], comps=info['comps'])
+    sub_info = Info(chs=chs, bads=info['bads'], comps=info['comps'],
+                    projs=info['projs'],
+                    custom_ref_applied=info['custom_ref_applied'])
     sub_info._update_redundant()
     return sub_info
 
@@ -586,9 +652,7 @@ def read_fiducials(fname, verbose=None):
     ----------
     fname : str
         The filename to read.
-    verbose : bool, str, int, or None
-        If not None, override default verbose level (see :func:`mne.verbose`
-        and :ref:`Logging documentation <tut_logging>` for more).
+    %(verbose)s
 
     Returns
     -------
@@ -636,11 +700,10 @@ def write_fiducials(fname, pts, coord_frame=FIFF.FIFFV_COORD_UNKNOWN,
     coord_frame : int
         The coordinate frame of the points (one of
         mne.io.constants.FIFF.FIFFV_COORD_...).
-    verbose : bool, str, int, or None
-        If not None, override default verbose level (see :func:`mne.verbose`
-        and :ref:`Logging documentation <tut_logging>` for more).
+    %(verbose)s
     """
-    write_dig(fname, pts, coord_frame)
+    from .._digitization._utils import write_dig as ff
+    ff(fname, pts, coord_frame)
 
 
 def write_dig(fname, pts, coord_frame=None):
@@ -658,215 +721,8 @@ def write_dig(fname, pts, coord_frame=None):
         here. Can be None (default) if the points could have varying
         coordinate frames.
     """
-    if coord_frame is not None:
-        coord_frame = _to_const(coord_frame)
-        pts_frames = set((pt.get('coord_frame', coord_frame) for pt in pts))
-        bad_frames = pts_frames - set((coord_frame,))
-        if len(bad_frames) > 0:
-            raise ValueError(
-                'Points have coord_frame entries that are incompatible with '
-                'coord_frame=%i: %s.' % (coord_frame, str(tuple(bad_frames))))
-
-    with start_file(fname) as fid:
-        write_dig_points(fid, pts, block=True, coord_frame=coord_frame)
-        end_file(fid)
-
-
-def _read_dig_fif(fid, meas_info):
-    """Read digitizer data from a FIFF file."""
-    isotrak = dir_tree_find(meas_info, FIFF.FIFFB_ISOTRAK)
-    dig = None
-    if len(isotrak) == 0:
-        logger.info('Isotrak not found')
-    elif len(isotrak) > 1:
-        warn('Multiple Isotrak found')
-    else:
-        isotrak = isotrak[0]
-        dig = []
-        for k in range(isotrak['nent']):
-            kind = isotrak['directory'][k].kind
-            pos = isotrak['directory'][k].pos
-            if kind == FIFF.FIFF_DIG_POINT:
-                tag = read_tag(fid, pos)
-                dig.append(tag.data)
-                dig[-1]['coord_frame'] = FIFF.FIFFV_COORD_HEAD
-    return dig
-
-
-def _read_dig_points(fname, comments='%', unit='auto'):
-    """Read digitizer data from a text file.
-
-    If fname ends in .hsp or .esp, the function assumes digitizer files in [m],
-    otherwise it assumes space-delimited text files in [mm].
-
-    Parameters
-    ----------
-    fname : str
-        The filepath of space delimited file with points, or a .mat file
-        (Polhemus FastTrak format).
-    comments : str
-        The character used to indicate the start of a comment;
-        Default: '%'.
-    unit : 'auto' | 'm' | 'cm' | 'mm'
-        Unit of the digitizer files (hsp and elp). If not 'm', coordinates will
-        be rescaled to 'm'. Default is 'auto', which assumes 'm' for *.hsp and
-        *.elp files and 'mm' for *.txt files, corresponding to the known
-        Polhemus export formats.
-
-    Returns
-    -------
-    dig_points : np.ndarray, shape (n_points, 3)
-        Array of dig points in [m].
-    """
-    if unit not in ('auto', 'm', 'mm', 'cm'):
-        raise ValueError('unit must be one of "auto", "m", "mm", or "cm"')
-
-    _, ext = op.splitext(fname)
-    if ext == '.elp' or ext == '.hsp':
-        with open(fname) as fid:
-            file_str = fid.read()
-        value_pattern = r"\-?\d+\.?\d*e?\-?\d*"
-        coord_pattern = r"({0})\s+({0})\s+({0})\s*$".format(value_pattern)
-        if ext == '.hsp':
-            coord_pattern = '^' + coord_pattern
-        points_str = [m.groups() for m in re.finditer(coord_pattern, file_str,
-                                                      re.MULTILINE)]
-        dig_points = np.array(points_str, dtype=float)
-    elif ext == '.mat':  # like FastScan II
-        from scipy.io import loadmat
-        dig_points = loadmat(fname)['Points'].T
-    else:
-        dig_points = np.loadtxt(fname, comments=comments, ndmin=2)
-        if unit == 'auto':
-            unit = 'mm'
-        if dig_points.shape[1] > 3:
-            warn('Found %d columns instead of 3, using first 3 for XYZ '
-                 'coordinates' % (dig_points.shape[1],))
-            dig_points = dig_points[:, :3]
-
-    if dig_points.shape[-1] != 3:
-        err = 'Data must be (n, 3) instead of %s' % (dig_points.shape,)
-        raise ValueError(err)
-
-    if unit == 'mm':
-        dig_points /= 1000.
-    elif unit == 'cm':
-        dig_points /= 100.
-
-    return dig_points
-
-
-def _write_dig_points(fname, dig_points):
-    """Write points to text file.
-
-    Parameters
-    ----------
-    fname : str
-        Path to the file to write. The kind of file to write is determined
-        based on the extension: '.txt' for tab separated text file.
-    dig_points : numpy.ndarray, shape (n_points, 3)
-        Points.
-    """
-    _, ext = op.splitext(fname)
-    dig_points = np.asarray(dig_points)
-    if (dig_points.ndim != 2) or (dig_points.shape[1] != 3):
-        err = ("Points must be of shape (n_points, 3), "
-               "not %s" % (dig_points.shape,))
-        raise ValueError(err)
-
-    if ext == '.txt':
-        with open(fname, 'wb') as fid:
-            version = __version__
-            now = datetime.datetime.now().strftime("%I:%M%p on %B %d, %Y")
-            fid.write(b("% Ascii 3D points file created by mne-python version "
-                        "{version} at {now}\n".format(version=version,
-                                                      now=now)))
-            fid.write(b("% {N} 3D points, "
-                        "x y z per line\n".format(N=len(dig_points))))
-            np.savetxt(fid, dig_points, delimiter='\t', newline='\n')
-    else:
-        msg = "Unrecognized extension: %r. Need '.txt'." % ext
-        raise ValueError(msg)
-
-
-def _make_dig_points(nasion=None, lpa=None, rpa=None, hpi=None,
-                     extra_points=None, dig_ch_pos=None):
-    """Construct digitizer info for the info.
-
-    Parameters
-    ----------
-    nasion : array-like | numpy.ndarray, shape (3,) | None
-        Point designated as the nasion point.
-    lpa : array-like |  numpy.ndarray, shape (3,) | None
-        Point designated as the left auricular point.
-    rpa : array-like |  numpy.ndarray, shape (3,) | None
-        Point designated as the right auricular point.
-    hpi : array-like | numpy.ndarray, shape (n_points, 3) | None
-        Points designated as head position indicator points.
-    extra_points : array-like | numpy.ndarray, shape (n_points, 3)
-        Points designed as the headshape points.
-    dig_ch_pos : dict
-        Dict of EEG channel positions.
-
-    Returns
-    -------
-    dig : list
-        List of digitizer points to be added to the info['dig'].
-    """
-    dig = []
-    if lpa is not None:
-        lpa = np.asarray(lpa)
-        if lpa.shape != (3,):
-            raise ValueError('LPA should have the shape (3,) instead of %s'
-                             % (lpa.shape,))
-        dig.append({'r': lpa, 'ident': FIFF.FIFFV_POINT_LPA,
-                    'kind': FIFF.FIFFV_POINT_CARDINAL,
-                    'coord_frame': FIFF.FIFFV_COORD_HEAD})
-    if nasion is not None:
-        nasion = np.asarray(nasion)
-        if nasion.shape != (3,):
-            raise ValueError('Nasion should have the shape (3,) instead of %s'
-                             % (nasion.shape,))
-        dig.append({'r': nasion, 'ident': FIFF.FIFFV_POINT_NASION,
-                    'kind': FIFF.FIFFV_POINT_CARDINAL,
-                    'coord_frame': FIFF.FIFFV_COORD_HEAD})
-    if rpa is not None:
-        rpa = np.asarray(rpa)
-        if rpa.shape != (3,):
-            raise ValueError('RPA should have the shape (3,) instead of %s'
-                             % (rpa.shape,))
-        dig.append({'r': rpa, 'ident': FIFF.FIFFV_POINT_RPA,
-                    'kind': FIFF.FIFFV_POINT_CARDINAL,
-                    'coord_frame': FIFF.FIFFV_COORD_HEAD})
-    if hpi is not None:
-        hpi = np.asarray(hpi)
-        if hpi.ndim != 2 or hpi.shape[1] != 3:
-            raise ValueError('HPI should have the shape (n_points, 3) instead '
-                             'of %s' % (hpi.shape,))
-        for idx, point in enumerate(hpi):
-            dig.append({'r': point, 'ident': idx + 1,
-                        'kind': FIFF.FIFFV_POINT_HPI,
-                        'coord_frame': FIFF.FIFFV_COORD_HEAD})
-    if extra_points is not None:
-        extra_points = np.asarray(extra_points)
-        if extra_points.shape[1] != 3:
-            raise ValueError('Points should have the shape (n_points, 3) '
-                             'instead of %s' % (extra_points.shape,))
-        for idx, point in enumerate(extra_points):
-            dig.append({'r': point, 'ident': idx + 1,
-                        'kind': FIFF.FIFFV_POINT_EXTRA,
-                        'coord_frame': FIFF.FIFFV_COORD_HEAD})
-    if dig_ch_pos is not None:
-        keys = sorted(dig_ch_pos.keys())
-        try:  # use the last 3 as int if possible (e.g., EEG001->1)
-            idents = [int(key[-3:]) for key in keys]
-        except ValueError:  # and if any conversion fails, simply use arange
-            idents = np.arange(1, len(keys) + 1)
-        for key, ident in zip(keys, idents):
-            dig.append({'r': dig_ch_pos[key], 'ident': ident,
-                        'kind': FIFF.FIFFV_POINT_EEG,
-                        'coord_frame': FIFF.FIFFV_COORD_HEAD})
-    return dig
+    from .._digitization._utils import write_dig as ff
+    return ff(fname, pts, coord_frame=None)
 
 
 @verbose
@@ -877,9 +733,7 @@ def read_info(fname, verbose=None):
     ----------
     fname : str
         File name.
-    verbose : bool, str, int, or None
-        If not None, override default verbose level (see :func:`mne.verbose`
-        and :ref:`Logging documentation <tut_logging>` for more).
+    %(verbose)s
 
     Returns
     -------
@@ -933,9 +787,7 @@ def read_meas_info(fid, tree, clean_bads=False, verbose=None):
         If True, clean info['bads'] before running consistency check.
         Should only be needed for old files where we did not check bads
         before saving.
-    verbose : bool, str, int, or None
-        If not None, override default verbose level (see :func:`mne.verbose`
-        and :ref:`Logging documentation <tut_logging>` for more).
+    %(verbose)s
 
     Returns
     -------
@@ -964,6 +816,7 @@ def read_meas_info(fid, tree, clean_bads=False, verbose=None):
     ctf_head_t = None
     dev_ctf_t = None
     meas_date = None
+    utc_offset = None
     highpass = None
     lowpass = None
     nchan = None
@@ -1003,6 +856,9 @@ def read_meas_info(fid, tree, clean_bads=False, verbose=None):
             meas_date = tuple(tag.data)
             if len(meas_date) == 1:  # can happen from old C conversions
                 meas_date = (meas_date[0], 0)
+        elif kind == FIFF.FIFF_UTC_OFFSET:
+            tag = read_tag(fid, pos)
+            utc_offset = str(tag.data)
         elif kind == FIFF.FIFF_COORD_TRANS:
             tag = read_tag(fid, pos)
             cand = tag.data
@@ -1170,7 +1026,7 @@ def read_meas_info(fid, tree, clean_bads=False, verbose=None):
             kind = hpi_meas['directory'][k].kind
             pos = hpi_meas['directory'][k].pos
             if kind == FIFF.FIFF_CREATOR:
-                hm['creator'] = text_type(read_tag(fid, pos).data)
+                hm['creator'] = str(read_tag(fid, pos).data)
             elif kind == FIFF.FIFF_SFREQ:
                 hm['sfreq'] = float(read_tag(fid, pos).data)
             elif kind == FIFF.FIFF_NCHAN:
@@ -1204,6 +1060,7 @@ def read_meas_info(fid, tree, clean_bads=False, verbose=None):
         hm['hpi_coils'] = hcs
         hms.append(hm)
     info['hpi_meas'] = hms
+    del hms
 
     subject_info = dir_tree_find(meas_info, FIFF.FIFFB_SUBJECT)
     si = None
@@ -1218,16 +1075,16 @@ def read_meas_info(fid, tree, clean_bads=False, verbose=None):
                 si['id'] = int(tag.data)
             elif kind == FIFF.FIFF_SUBJ_HIS_ID:
                 tag = read_tag(fid, pos)
-                si['his_id'] = text_type(tag.data)
+                si['his_id'] = str(tag.data)
             elif kind == FIFF.FIFF_SUBJ_LAST_NAME:
                 tag = read_tag(fid, pos)
-                si['last_name'] = text_type(tag.data)
+                si['last_name'] = str(tag.data)
             elif kind == FIFF.FIFF_SUBJ_FIRST_NAME:
                 tag = read_tag(fid, pos)
-                si['first_name'] = text_type(tag.data)
+                si['first_name'] = str(tag.data)
             elif kind == FIFF.FIFF_SUBJ_MIDDLE_NAME:
                 tag = read_tag(fid, pos)
-                si['middle_name'] = text_type(tag.data)
+                si['middle_name'] = str(tag.data)
             elif kind == FIFF.FIFF_SUBJ_BIRTH_DAY:
                 tag = read_tag(fid, pos)
                 si['birthday'] = tag.data
@@ -1244,6 +1101,53 @@ def read_meas_info(fid, tree, clean_bads=False, verbose=None):
                 tag = read_tag(fid, pos)
                 si['height'] = tag.data
     info['subject_info'] = si
+    del si
+
+    device_info = dir_tree_find(meas_info, FIFF.FIFFB_DEVICE)
+    di = None
+    if len(device_info) == 1:
+        device_info = device_info[0]
+        di = dict()
+        for k in range(device_info['nent']):
+            kind = device_info['directory'][k].kind
+            pos = device_info['directory'][k].pos
+            if kind == FIFF.FIFF_DEVICE_TYPE:
+                tag = read_tag(fid, pos)
+                di['type'] = str(tag.data)
+            elif kind == FIFF.FIFF_DEVICE_MODEL:
+                tag = read_tag(fid, pos)
+                di['model'] = str(tag.data)
+            elif kind == FIFF.FIFF_DEVICE_SERIAL:
+                tag = read_tag(fid, pos)
+                di['serial'] = str(tag.data)
+            elif kind == FIFF.FIFF_DEVICE_SITE:
+                tag = read_tag(fid, pos)
+                di['site'] = str(tag.data)
+    info['device_info'] = di
+    del di
+
+    helium_info = dir_tree_find(meas_info, FIFF.FIFFB_HELIUM)
+    hi = None
+    if len(helium_info) == 1:
+        helium_info = helium_info[0]
+        hi = dict()
+        for k in range(helium_info['nent']):
+            kind = helium_info['directory'][k].kind
+            pos = helium_info['directory'][k].pos
+            if kind == FIFF.FIFF_HE_LEVEL_RAW:
+                tag = read_tag(fid, pos)
+                hi['he_level_raw'] = float(tag.data)
+            elif kind == FIFF.FIFF_HELIUM_LEVEL:
+                tag = read_tag(fid, pos)
+                hi['helium_level'] = float(tag.data)
+            elif kind == FIFF.FIFF_ORIG_FILE_GUID:
+                tag = read_tag(fid, pos)
+                hi['orig_file_guid'] = str(tag.data)
+            elif kind == FIFF.FIFF_MEAS_DATE:
+                tag = read_tag(fid, pos)
+                hi['meas_date'] = tuple(int(t) for t in tag.data)
+    info['helium_info'] = hi
+    del hi
 
     hpi_subsystem = dir_tree_find(meas_info, FIFF.FIFFB_HPI_SUBSYSTEM)
     hs = None
@@ -1258,7 +1162,7 @@ def read_meas_info(fid, tree, clean_bads=False, verbose=None):
                 hs['ncoil'] = int(tag.data)
             elif kind == FIFF.FIFF_EVENT_CHANNEL:
                 tag = read_tag(fid, pos)
-                hs['event_channel'] = text_type(tag.data)
+                hs['event_channel'] = str(tag.data)
             hpi_coils = dir_tree_find(hpi_subsystem, FIFF.FIFFB_HPI_COIL)
             hc = []
             for coil in hpi_coils:
@@ -1299,6 +1203,7 @@ def read_meas_info(fid, tree, clean_bads=False, verbose=None):
     if np.array_equal(meas_date, DATE_NONE):
         meas_date = None
     info['meas_date'] = meas_date
+    info['utc_offset'] = utc_offset
 
     info['sfreq'] = sfreq
     info['highpass'] = highpass if highpass is not None else 0.
@@ -1323,7 +1228,7 @@ def read_meas_info(fid, tree, clean_bads=False, verbose=None):
         info['dev_ctf_t'] = Transform('meg', 'ctf_head', dev_ctf_trans)
 
     #   All kinds of auxliary stuff
-    info['dig'] = dig
+    info['dig'] = _format_dig_points(dig)
     info['bads'] = bads
     info._update_redundant()
     if clean_bads:
@@ -1481,6 +1386,8 @@ def write_meas_info(fid, info, data_type=None, reset_range=True):
         write_string(fid, FIFF.FIFF_PROJ_NAME, info['proj_name'])
     if info.get('meas_date') is not None:
         write_int(fid, FIFF.FIFF_MEAS_DATE, info['meas_date'])
+    if info.get('utc_offset') is not None:
+        write_string(fid, FIFF.FIFF_UTC_OFFSET, info['utc_offset'])
     write_int(fid, FIFF.FIFF_NCHAN, info['nchan'])
     write_float(fid, FIFF.FIFF_SFREQ, info['sfreq'])
     if info['lowpass'] is not None:
@@ -1533,6 +1440,31 @@ def write_meas_info(fid, info, data_type=None, reset_range=True):
         if si.get('height') is not None:
             write_float(fid, FIFF.FIFF_SUBJ_HEIGHT, si['height'])
         end_block(fid, FIFF.FIFFB_SUBJECT)
+        del si
+
+    if info.get('device_info') is not None:
+        start_block(fid, FIFF.FIFFB_DEVICE)
+        di = info['device_info']
+        write_string(fid, FIFF.FIFF_DEVICE_TYPE, di['type'])
+        for key in ('model', 'serial', 'site'):
+            if di.get(key) is not None:
+                write_string(fid, getattr(FIFF, 'FIFF_DEVICE_' + key.upper()),
+                             di[key])
+        end_block(fid, FIFF.FIFFB_DEVICE)
+        del di
+
+    if info.get('helium_info') is not None:
+        start_block(fid, FIFF.FIFFB_HELIUM)
+        hi = info['helium_info']
+        if hi.get('he_level_raw') is not None:
+            write_float(fid, FIFF.FIFF_HE_LEVEL_RAW, hi['he_level_raw'])
+        if hi.get('helium_level') is not None:
+            write_float(fid, FIFF.FIFF_HELIUM_LEVEL, hi['helium_level'])
+        if hi.get('orig_file_guid') is not None:
+            write_string(fid, FIFF.FIFF_ORIG_FILE_GUID, hi['orig_file_guid'])
+        write_int(fid, FIFF.FIFF_MEAS_DATE, hi['meas_date'])
+        end_block(fid, FIFF.FIFFB_HELIUM)
+        del hi
 
     if info.get('hpi_subsystem') is not None:
         hs = info['hpi_subsystem']
@@ -1549,6 +1481,7 @@ def write_meas_info(fid, info, data_type=None, reset_range=True):
                               coil['event_bits'])
                 end_block(fid, FIFF.FIFFB_HPI_COIL)
         end_block(fid, FIFF.FIFFB_HPI_SUBSYSTEM)
+        del hs
 
     #   CTF compensation info
     write_ctf_comp(fid, info['comps'])
@@ -1666,7 +1599,7 @@ def _merge_info_values(infos, key, verbose=None):
             logger.info('Found multiple StringIO instances. '
                         'Setting value to `None`')
             return None
-        elif isinstance(list(unique_values)[0], string_types):
+        elif isinstance(list(unique_values)[0], str):
             logger.info('Found multiple filenames. '
                         'Setting value to `None`')
             return None
@@ -1695,9 +1628,7 @@ def _merge_info(infos, force_update_to_first=False, verbose=None):
         If True, force the fields for objects in `info` will be updated
         to match those in the first item. Use at your own risk, as this
         may overwrite important metadata.
-    verbose : bool, str, int, or NonIe
-        If not None, override default verbose level (see :func:`mne.verbose`
-        and :ref:`Logging documentation <tut_logging>` for more).
+    %(verbose)s
 
     Returns
     -------
@@ -1714,8 +1645,8 @@ def _merge_info(infos, force_update_to_first=False, verbose=None):
     for this_info in infos:
         info['chs'].extend(this_info['chs'])
     info._update_redundant()
-    duplicates = set([ch for ch in info['ch_names']
-                      if info['ch_names'].count(ch) > 1])
+    duplicates = {ch for ch in info['ch_names']
+                  if info['ch_names'].count(ch) > 1}
     if len(duplicates) > 0:
         msg = ("The following channels are present in more than one input "
                "measurement info objects: %s" % list(duplicates))
@@ -1764,8 +1695,8 @@ def _merge_info(infos, force_update_to_first=False, verbose=None):
     # other fields
     other_fields = ['acq_pars', 'acq_stim', 'bads',
                     'comps', 'custom_ref_applied', 'description',
-                    'experimenter', 'file_id', 'highpass',
-                    'hpi_subsystem', 'events',
+                    'experimenter', 'file_id', 'highpass', 'utc_offset',
+                    'hpi_subsystem', 'events', 'device_info', 'helium_info',
                     'line_freq', 'lowpass', 'meas_id',
                     'proj_id', 'proj_name', 'projs', 'sfreq', 'gantry_angle',
                     'subject_info', 'sfreq', 'xplotter_layout', 'proc_history']
@@ -1793,16 +1724,8 @@ def create_info(ch_names, sfreq, ch_types=None, montage=None, verbose=None):
         Currently supported fields are 'ecg', 'bio', 'stim', 'eog', 'misc',
         'seeg', 'ecog', 'mag', 'eeg', 'ref_meg', 'grad', 'emg', 'hbr' or 'hbo'.
         If str, then all channels are assumed to be of the same type.
-    montage : None | str | Montage | DigMontage | list
-        A montage containing channel positions. If str or Montage is
-        specified, the channel info will be updated with the channel
-        positions. Default is None. If DigMontage is specified, the
-        digitizer information will be updated. A list of unique montages,
-        can be specified and applied to the info. See also the documentation of
-        :func:`mne.channels.read_montage` for more information.
-    verbose : bool, str, int, or None
-        If not None, override default verbose level (see :func:`mne.verbose`
-        and :ref:`Logging documentation <tut_logging>` for more).
+    %(montage)s
+    %(verbose)s
 
     Returns
     -------
@@ -1841,11 +1764,13 @@ def create_info(ch_names, sfreq, ch_types=None, montage=None, verbose=None):
     nchan = len(ch_names)
     if ch_types is None:
         ch_types = ['misc'] * nchan
-    if isinstance(ch_types, string_types):
+    if isinstance(ch_types, str):
         ch_types = [ch_types] * nchan
-    if len(ch_types) != nchan:
+    ch_types = np.atleast_1d(np.array(ch_types, np.str))
+    if ch_types.ndim != 1 or len(ch_types) != nchan:
         raise ValueError('ch_types and ch_names must be the same length '
-                         '(%s != %s)' % (len(ch_types), nchan))
+                         '(%s != %s) for ch_types=%s'
+                         % (len(ch_types), nchan, ch_types))
     info = _empty_info(sfreq)
     for ci, (name, kind) in enumerate(zip(ch_names, ch_types)):
         _validate_type(name, 'str', "each entry in ch_names")
@@ -1854,22 +1779,24 @@ def create_info(ch_names, sfreq, ch_types=None, montage=None, verbose=None):
             raise KeyError('kind must be one of %s, not %s'
                            % (list(_kind_dict.keys()), kind))
         kind = _kind_dict[kind]
+        # mirror what tag.py does here
+        coord_frame = _coord_dict.get(kind[0], FIFF.FIFFV_COORD_UNKNOWN)
         chan_info = dict(loc=np.full(12, np.nan), unit_mul=0, range=1., cal=1.,
                          kind=kind[0], coil_type=kind[1],
-                         unit=kind[2], coord_frame=FIFF.FIFFV_COORD_UNKNOWN,
-                         ch_name=name, scanno=ci + 1, logno=ci + 1)
+                         unit=kind[2], coord_frame=coord_frame,
+                         ch_name=str(name), scanno=ci + 1, logno=ci + 1)
         info['chs'].append(chan_info)
     info._update_redundant()
     if montage is not None:
-        from ..channels.montage import (Montage, DigMontage, _set_montage,
-                                        read_montage)
-        if not isinstance(montage, list):
+        from ..channels.montage import (Montage, DigMontage, _set_montage)
+        if isinstance(montage, list):
+            warn('Passing a list as a montage is deprecated and will be '
+                 ' removed in v0.20. Please use DigMontage and + operator.',
+                 DeprecationWarning)
+        else:
             montage = [montage]
         for montage_ in montage:
-            if isinstance(montage_, (Montage, DigMontage)):
-                _set_montage(info, montage_)
-            elif isinstance(montage_, string_types):
-                montage_ = read_montage(montage_)
+            if isinstance(montage_, (str, Montage, DigMontage)):
                 _set_montage(info, montage_)
             else:
                 raise TypeError('Montage must be an instance of Montage, '
@@ -1882,8 +1809,8 @@ def create_info(ch_names, sfreq, ch_types=None, montage=None, verbose=None):
 RAW_INFO_FIELDS = (
     'acq_pars', 'acq_stim', 'bads', 'ch_names', 'chs',
     'comps', 'ctf_head_t', 'custom_ref_applied', 'description', 'dev_ctf_t',
-    'dev_head_t', 'dig', 'experimenter', 'events',
-    'file_id', 'highpass', 'hpi_meas', 'hpi_results',
+    'dev_head_t', 'dig', 'experimenter', 'events', 'utc_offset', 'device_info',
+    'file_id', 'highpass', 'hpi_meas', 'hpi_results', 'helium_info',
     'hpi_subsystem', 'kit_system_id', 'line_freq', 'lowpass', 'meas_date',
     'meas_id', 'nchan', 'proj_id', 'proj_name', 'projs', 'sfreq',
     'subject_info', 'xplotter_layout', 'proc_history', 'gantry_angle',
@@ -1892,11 +1819,10 @@ RAW_INFO_FIELDS = (
 
 def _empty_info(sfreq):
     """Create an empty info dictionary."""
-    from ..transforms import Transform
     _none_keys = (
         'acq_pars', 'acq_stim', 'ctf_head_t', 'description',
-        'dev_ctf_t', 'dig', 'experimenter',
-        'file_id', 'highpass', 'hpi_subsystem', 'kit_system_id',
+        'dev_ctf_t', 'dig', 'experimenter', 'utc_offset', 'device_info',
+        'file_id', 'highpass', 'hpi_subsystem', 'kit_system_id', 'helium_info',
         'line_freq', 'lowpass', 'meas_date', 'meas_id', 'proj_id', 'proj_name',
         'subject_info', 'xplotter_layout', 'gantry_angle',
     )
@@ -1908,10 +1834,10 @@ def _empty_info(sfreq):
     for k in _list_keys:
         info[k] = list()
     info['custom_ref_applied'] = False
-    info['dev_head_t'] = Transform('meg', 'head')
     info['highpass'] = 0.
     info['sfreq'] = float(sfreq)
     info['lowpass'] = info['sfreq'] / 2.
+    info['dev_head_t'] = Transform('meg', 'head')
     info._update_redundant()
     info._check_consistency()
     return info
