@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Authors: Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
+# Authors: Alexandre Gramfort <alexandre.gramfort@inria.fr>
 #          Matti Hamalainen <msh@nmr.mgh.harvard.edu>
 #          Martin Luessi <mluessi@nmr.mgh.harvard.edu>
 #
@@ -11,8 +11,8 @@ import re
 import numpy as np
 
 from .constants import FIFF
-from ..utils import logger, verbose, _validate_type
-from ..externals.six import string_types
+from ..utils import (logger, verbose, _validate_type, fill_doc, _ensure_int,
+                     _check_option)
 
 
 def get_channel_types():
@@ -55,37 +55,81 @@ def get_channel_types():
                          coil_type=FIFF.FIFFV_COIL_FNIRS_HBR))
 
 
+_first_rule = {
+    FIFF.FIFFV_MEG_CH: 'meg',
+    FIFF.FIFFV_REF_MEG_CH: 'ref_meg',
+    FIFF.FIFFV_EEG_CH: 'eeg',
+    FIFF.FIFFV_STIM_CH: 'stim',
+    FIFF.FIFFV_EOG_CH: 'eog',
+    FIFF.FIFFV_EMG_CH: 'emg',
+    FIFF.FIFFV_ECG_CH: 'ecg',
+    FIFF.FIFFV_RESP_CH: 'resp',
+    FIFF.FIFFV_MISC_CH: 'misc',
+    FIFF.FIFFV_EXCI_CH: 'exci',
+    FIFF.FIFFV_IAS_CH: 'ias',
+    FIFF.FIFFV_SYST_CH: 'syst',
+    FIFF.FIFFV_SEEG_CH: 'seeg',
+    FIFF.FIFFV_BIO_CH: 'bio',
+    FIFF.FIFFV_QUAT_0: 'chpi',
+    FIFF.FIFFV_QUAT_1: 'chpi',
+    FIFF.FIFFV_QUAT_2: 'chpi',
+    FIFF.FIFFV_QUAT_3: 'chpi',
+    FIFF.FIFFV_QUAT_4: 'chpi',
+    FIFF.FIFFV_QUAT_5: 'chpi',
+    FIFF.FIFFV_QUAT_6: 'chpi',
+    FIFF.FIFFV_HPI_G: 'chpi',
+    FIFF.FIFFV_HPI_ERR: 'chpi',
+    FIFF.FIFFV_HPI_MOV: 'chpi',
+    FIFF.FIFFV_DIPOLE_WAVE: 'dipole',
+    FIFF.FIFFV_GOODNESS_FIT: 'gof',
+    FIFF.FIFFV_ECOG_CH: 'ecog',
+    FIFF.FIFFV_FNIRS_CH: 'fnirs',
+}
+# How to reduce our categories in channel_type (originally)
+_second_rules = {
+    'meg': ('unit', {FIFF.FIFF_UNIT_T_M: 'grad',
+                     FIFF.FIFF_UNIT_T: 'mag'}),
+    'fnirs': ('coil_type', {FIFF.FIFFV_COIL_FNIRS_HBO: 'hbo',
+                            FIFF.FIFFV_COIL_FNIRS_HBR: 'hbr'}),
+}
+
+
 def channel_type(info, idx):
     """Get channel type.
 
     Parameters
     ----------
-    info : dict
-        Measurement info
+    info : instance of Info
+        A measurement info object.
     idx : int
-        Index of channel
+        Index of channel.
 
     Returns
     -------
-    type : 'grad' | 'mag' | 'eeg' | 'stim' | 'eog' | 'emg' | 'ecg'
-           'ref_meg' | 'resp' | 'exci' | 'ias' | 'syst' | 'misc'
-           'seeg' | 'bio' | 'chpi' | 'dipole' | 'gof' | 'ecog' | 'hbo' | 'hbr'
-        Type of channel
+    type : str
+        Type of channel. Will be one of::
+
+            {'grad', 'mag', 'eeg', 'stim', 'eog', 'emg', 'ecg', 'ref_meg',
+             'resp', 'exci', 'ias', 'syst', 'misc', 'seeg', 'bio', 'chpi',
+             'dipole', 'gof', 'ecog', 'hbo', 'hbr'}
+
     """
+    # This is faster than the original _channel_type_old now in test_pick.py
+    # because it uses (at most!) two dict lookups plus one conditional
+    # to get the channel type string.
     ch = info['chs'][idx]
-
-    # iterate through all defined channel types until we find a match with ch
-    for t, rules in get_channel_types().items():
-        for key, vals in rules.items():  # all keys must match the values
-            if ch.get(key, None) not in np.array(vals):
-                break  # not channel type t, go to next iteration
-        else:
-            return t
-
-    raise ValueError('Unknown channel type for {}'.format(ch["ch_name"]))
+    try:
+        first_kind = _first_rule[ch['kind']]
+    except KeyError:
+        raise ValueError('Unknown channel type (%s) for channel "%s"'
+                         % (ch['kind'], ch["ch_name"]))
+    if first_kind in _second_rules:
+        key, second_rule = _second_rules[first_kind]
+        first_kind = second_rule[ch[key]]
+    return first_kind
 
 
-def pick_channels(ch_names, include, exclude=[]):
+def pick_channels(ch_names, include, exclude=[], ordered=False):
     """Pick channels by names.
 
     Returns the indices of the good channels in ch_names.
@@ -103,6 +147,12 @@ def pick_channels(ch_names, include, exclude=[]):
     exclude : list of string
         List of channels to exclude (if empty do not exclude any channel).
         Defaults to [].
+    ordered : bool
+        If true (default False), treat ``include`` as an ordered list
+        rather than a set, and any channels from ``include`` are missing
+        in ``ch_names`` an error will be raised.
+
+        .. versionadded:: 0.18
 
     See Also
     --------
@@ -117,15 +167,32 @@ def pick_channels(ch_names, include, exclude=[]):
         raise RuntimeError('ch_names is not a unique list, picking is unsafe')
     _check_excludes_includes(include)
     _check_excludes_includes(exclude)
-    if not isinstance(include, set):
-        include = set(include)
-    if not isinstance(exclude, set):
-        exclude = set(exclude)
-
-    sel = []
-    for k, name in enumerate(ch_names):
-        if (len(include) == 0 or name in include) and name not in exclude:
-            sel.append(k)
+    if not ordered:
+        if not isinstance(include, set):
+            include = set(include)
+        if not isinstance(exclude, set):
+            exclude = set(exclude)
+        sel = []
+        for k, name in enumerate(ch_names):
+            if (len(include) == 0 or name in include) and name not in exclude:
+                sel.append(k)
+    else:
+        if not isinstance(include, list):
+            include = list(include)
+        if len(include) == 0:
+            include = list(ch_names)
+        if not isinstance(exclude, list):
+            exclude = list(exclude)
+        sel, missing = list(), list()
+        for name in include:
+            if name in ch_names:
+                if name not in exclude:
+                    sel.append(ch_names.index(name))
+            else:
+                missing.append(name)
+        if len(missing):
+            raise ValueError('Missing channels from ch_names required by '
+                             'include:\n%s' % (missing,))
     return np.array(sel, int)
 
 
@@ -192,7 +259,7 @@ def _triage_fnirs_pick(ch, fnirs):
 
 def _check_meg_type(meg, allow_auto=False):
     """Ensure a valid meg type."""
-    if isinstance(meg, string_types):
+    if isinstance(meg, str):
         allowed_types = ['grad', 'mag', 'planar1', 'planar2']
         allowed_types += ['auto'] if allow_auto else []
         if meg not in allowed_types:
@@ -200,6 +267,20 @@ def _check_meg_type(meg, allow_auto=False):
                              % (allowed_types, meg))
 
 
+def _check_info_exclude(info, exclude):
+    _validate_type(info, "info")
+    info._check_consistency()
+    if exclude is None:
+        raise ValueError('exclude must be a list of strings or "bads"')
+    elif exclude == 'bads':
+        exclude = info.get('bads', [])
+    elif not isinstance(exclude, (list, tuple)):
+        raise ValueError('exclude must either be "bads" or a list of strings.'
+                         ' If only one channel is to be excluded, use '
+                         '[ch_name] instead of passing ch_name.')
+    return exclude
+
+
 def pick_types(info, meg=True, eeg=False, stim=False, eog=False, ecg=False,
                emg=False, ref_meg='auto', misc=False, resp=False, chpi=False,
                exci=False, ias=False, syst=False, seeg=False, dipole=False,
@@ -226,10 +307,11 @@ def pick_types(info, meg=True, eeg=False, stim=False, eog=False, ecg=False,
         If True include ECG channels.
     emg : bool
         If True include EMG channels.
-    ref_meg: bool | str
-        If True include CTF / 4D reference channels. If 'auto', the reference
-        channels are only included if compensations are present. Can also be
-        the string options from `meg`.
+    ref_meg : bool | str
+        If True include CTF / 4D reference channels. If 'auto', the
+        reference channels included if compensations are present
+        and ``meg`` is not False. Can also be the string options allowed
+        for the ``meg`` parameter.
     misc : bool
         If True include miscellaneous analog channels.
     resp : bool
@@ -273,25 +355,15 @@ def pick_types(info, meg=True, eeg=False, stim=False, eog=False, ecg=False,
     """
     # NOTE: Changes to this function's signature should also be changed in
     # PickChannelsMixin
-    _validate_type(info, "info")
-    info._check_consistency()
+    exclude = _check_info_exclude(info, exclude)
     nchan = info['nchan']
     pick = np.zeros(nchan, dtype=np.bool)
 
-    if exclude is None:
-        raise ValueError('exclude must be a list of strings or "bads"')
-    elif exclude == 'bads':
-        exclude = info.get('bads', [])
-    elif not isinstance(exclude, (list, tuple)):
-        raise ValueError('exclude must either be "bads" or a list of strings.'
-                         ' If only one channel is to be excluded, use '
-                         '[ch_name] instead of passing ch_name.')
-
     _check_meg_type(ref_meg, allow_auto=True)
     _check_meg_type(meg)
-    if isinstance(ref_meg, string_types) and ref_meg == 'auto':
+    if isinstance(ref_meg, str) and ref_meg == 'auto':
         ref_meg = ('comps' in info and info['comps'] is not None and
-                   len(info['comps']) > 0)
+                   len(info['comps']) > 0 and meg is not False)
 
     for param in (eeg, stim, eog, ecg, emg, misc, resp, chpi, exci,
                   ias, syst, seeg, dipole, gof, bio, ecog):
@@ -301,50 +373,29 @@ def pick_types(info, meg=True, eeg=False, stim=False, eog=False, ecg=False,
                  'not {0}.')
             raise ValueError(w.format(type(param)))
 
+    param_dict = dict(eeg=eeg, stim=stim, eog=eog, ecg=ecg, emg=emg,
+                      misc=misc, resp=resp, chpi=chpi, exci=exci,
+                      ias=ias, syst=syst, seeg=seeg, dipole=dipole,
+                      gof=gof, bio=bio, ecog=ecog)
+    # avoid triage if possible
+    if isinstance(meg, bool):
+        for key in ('grad', 'mag'):
+            param_dict[key] = meg
+    if isinstance(fnirs, bool):
+        for key in ('hbo', 'hbr'):
+            param_dict[key] = fnirs
     for k in range(nchan):
-        kind = info['chs'][k]['kind']
-        # XXX eventually we should de-duplicate this with channel_type!
-        if kind == FIFF.FIFFV_MEG_CH and meg:
-            pick[k] = _triage_meg_pick(info['chs'][k], meg)
-        elif kind == FIFF.FIFFV_EEG_CH and eeg:
-            pick[k] = True
-        elif kind == FIFF.FIFFV_STIM_CH and stim:
-            pick[k] = True
-        elif kind == FIFF.FIFFV_EOG_CH and eog:
-            pick[k] = True
-        elif kind == FIFF.FIFFV_ECG_CH and ecg:
-            pick[k] = True
-        elif kind == FIFF.FIFFV_EMG_CH and emg:
-            pick[k] = True
-        elif kind == FIFF.FIFFV_MISC_CH and misc:
-            pick[k] = True
-        elif kind == FIFF.FIFFV_REF_MEG_CH and ref_meg:
-            pick[k] = _triage_meg_pick(info['chs'][k], ref_meg)
-        elif kind == FIFF.FIFFV_RESP_CH and resp:
-            pick[k] = True
-        elif kind == FIFF.FIFFV_SYST_CH and syst:
-            pick[k] = True
-        elif kind == FIFF.FIFFV_SEEG_CH and seeg:
-            pick[k] = True
-        elif kind == FIFF.FIFFV_IAS_CH and ias:
-            pick[k] = True
-        elif kind == FIFF.FIFFV_EXCI_CH and exci:
-            pick[k] = True
-        elif kind in [FIFF.FIFFV_QUAT_0, FIFF.FIFFV_QUAT_1, FIFF.FIFFV_QUAT_2,
-                      FIFF.FIFFV_QUAT_3, FIFF.FIFFV_QUAT_4, FIFF.FIFFV_QUAT_5,
-                      FIFF.FIFFV_QUAT_6, FIFF.FIFFV_HPI_G, FIFF.FIFFV_HPI_ERR,
-                      FIFF.FIFFV_HPI_MOV] and chpi:
-            pick[k] = True
-        elif kind == FIFF.FIFFV_DIPOLE_WAVE and dipole:
-            pick[k] = True
-        elif kind == FIFF.FIFFV_GOODNESS_FIT and gof:
-            pick[k] = True
-        elif kind == FIFF.FIFFV_BIO_CH and bio:
-            pick[k] = True
-        elif kind == FIFF.FIFFV_ECOG_CH and ecog:
-            pick[k] = True
-        elif kind == FIFF.FIFFV_FNIRS_CH:
-            pick[k] = _triage_fnirs_pick(info['chs'][k], fnirs)
+        ch_type = channel_type(info, k)
+        try:
+            pick[k] = param_dict[ch_type]
+        except KeyError:  # not so simple
+            assert ch_type in ('grad', 'mag', 'hbo', 'hbr', 'ref_meg')
+            if ch_type in ('grad', 'mag'):
+                pick[k] = _triage_meg_pick(info['chs'][k], meg)
+            elif ch_type == 'ref_meg':
+                pick[k] = _triage_meg_pick(info['chs'][k], ref_meg)
+            else:  # ch_type in ('hbo', 'hbr')
+                pick[k] = _triage_fnirs_pick(info['chs'][k], fnirs)
 
     # restrict channels to selection if provided
     if selection is not None:
@@ -380,9 +431,7 @@ def pick_info(info, sel=(), copy=True, verbose=None):
         are included.
     copy : bool
         If copy is False, info is modified inplace.
-    verbose : bool, str, int, or None
-        If not None, override default verbose level (see :func:`mne.verbose`
-        and :ref:`Logging documentation <tut_logging>` for more).
+    %(verbose)s
 
     Returns
     -------
@@ -429,7 +478,6 @@ def pick_info(info, sel=(), copy=True, verbose=None):
             c['data']['data'] = c['data']['data'][row_idx]
         info['comps'] = comps
     info._check_consistency()
-    info._check_consistency()
     return info
 
 
@@ -489,7 +537,8 @@ def pick_channels_evoked(orig, include=[], exclude='bads'):
 
 
 @verbose
-def pick_channels_forward(orig, include=[], exclude=[], verbose=None):
+def pick_channels_forward(orig, include=[], exclude=[], ordered=False,
+                          copy=True, verbose=None):
     """Pick channels from forward operator.
 
     Parameters
@@ -502,9 +551,16 @@ def pick_channels_forward(orig, include=[], exclude=[], verbose=None):
     exclude : list of string | 'bads'
         Channels to exclude (if empty, do not exclude any). Defaults to [].
         If 'bads', then exclude bad channels in orig.
-    verbose : bool, str, int, or None
-        If not None, override default verbose level (see :func:`mne.verbose`
-        and :ref:`Logging documentation <tut_logging>` for more).
+    ordered : bool
+        If true (default False), treat ``include`` as an ordered list
+        rather than a set.
+
+        .. versionadded:: 0.18
+    copy : bool
+        If True (default), make a copy.
+
+        .. versionadded:: 0.19
+    %(verbose)s
 
     Returns
     -------
@@ -514,18 +570,18 @@ def pick_channels_forward(orig, include=[], exclude=[], verbose=None):
     """
     orig['info']._check_consistency()
     if len(include) == 0 and len(exclude) == 0:
-        return orig
+        return orig.copy() if copy else orig
     exclude = _check_excludes_includes(exclude,
                                        info=orig['info'], allow_bads=True)
 
     # Allow for possibility of channel ordering in forward solution being
     # different from that of the M/EEG file it is based on.
     sel_sol = pick_channels(orig['sol']['row_names'], include=include,
-                            exclude=exclude)
+                            exclude=exclude, ordered=ordered)
     sel_info = pick_channels(orig['info']['ch_names'], include=include,
-                             exclude=exclude)
+                             exclude=exclude, ordered=ordered)
 
-    fwd = deepcopy(orig)
+    fwd = deepcopy(orig) if copy else orig
 
     # Check that forward solution and original data file agree on #channels
     if len(sel_sol) != len(sel_info):
@@ -605,27 +661,27 @@ def pick_types_forward(orig, meg=True, eeg=False, ref_meg=True, seeg=False,
     return pick_channels_forward(orig, include_ch_names)
 
 
+@fill_doc
 def channel_indices_by_type(info, picks=None):
     """Get indices of channels by type.
 
     Parameters
     ----------
-    info : instance of mne.measuerment_info.Info
-        The info.
-    picks : None | list of int
-        The indices of channels from which to get the type
+    info : instance of Info
+        A measurement info object.
+    %(picks_all)s
 
     Returns
     -------
     idx_by_type : dict
-        The dictionary that maps each channel type to the list of
-        channel indidces.
+        A dictionary that maps each channel type to a (possibly empty) list of
+        channel indices.
     """
-    idx_by_type = dict((key, list()) for key in _PICK_TYPES_KEYS if
-                       key not in ('meg', 'fnirs'))
+    idx_by_type = {key: list() for key in _PICK_TYPES_KEYS if
+                   key not in ('meg', 'fnirs')}
     idx_by_type.update(mag=list(), grad=list(), hbo=list(), hbr=list())
-    if picks is None:
-        picks = range(len(info["chs"]))
+    picks = _picks_to_idx(info, picks,
+                          none='all', exclude=(), allow_empty=True)
     for k in picks:
         ch_type = channel_type(info, k)
         for key in idx_by_type.keys():
@@ -672,6 +728,35 @@ def _mag_grad_dependent(info):
                for ph in info.get('proc_history', []))
 
 
+def _contains_ch_type(info, ch_type):
+    """Check whether a certain channel type is in an info object.
+
+    Parameters
+    ----------
+    info : instance of Info
+        The measurement information.
+    ch_type : str
+        the channel type to be checked for
+
+    Returns
+    -------
+    has_ch_type : bool
+        Whether the channel type is present or not.
+    """
+    _validate_type(ch_type, 'str', "ch_type")
+
+    meg_extras = ['mag', 'grad', 'planar1', 'planar2']
+    fnirs_extras = ['hbo', 'hbr']
+    valid_channel_types = sorted([key for key in _PICK_TYPES_KEYS
+                                  if key != 'meg'] + meg_extras + fnirs_extras)
+    _check_option('ch_type', ch_type, valid_channel_types)
+    if info is None:
+        raise ValueError('Cannot check for channels of type "%s" because info '
+                         'is None' % (ch_type,))
+    return any(ch_type == channel_type(info, ii)
+               for ii in range(info['nchan']))
+
+
 def _picks_by_type(info, meg_combined=False, ref_meg=False, exclude='bads'):
     """Get data channel indices as separate list of tuples.
 
@@ -693,35 +778,38 @@ def _picks_by_type(info, meg_combined=False, ref_meg=False, exclude='bads'):
     picks_list : list of tuples
         The list of tuples of picks and the type string.
     """
-    from ..channels.channels import _contains_ch_type
+    _validate_type(ref_meg, bool, 'ref_meg')
+    exclude = _check_info_exclude(info, exclude)
     if meg_combined == 'auto':
         meg_combined = _mag_grad_dependent(info)
     picks_list = []
-    has = [_contains_ch_type(info, k) for k in _DATA_CH_TYPES_SPLIT]
-    has = dict(zip(_DATA_CH_TYPES_SPLIT, has))
-    if has['mag'] and (meg_combined is not True or not has['grad']):
-        picks_list.append(
-            ('mag', pick_types(info, meg='mag', eeg=False, stim=False,
-             ref_meg=ref_meg, exclude=exclude))
+    picks_list = {ch_type: list() for ch_type in _DATA_CH_TYPES_SPLIT}
+    for k in range(info['nchan']):
+        if info['chs'][k]['ch_name'] not in exclude:
+            this_type = channel_type(info, k)
+            try:
+                picks_list[this_type].append(k)
+            except KeyError:
+                # This annoyance is due to differences in pick_types
+                # and channel_type behavior
+                if this_type == 'ref_meg':
+                    ch = info['chs'][k]
+                    if _triage_meg_pick(ch, ref_meg):
+                        if ch['unit'] == FIFF.FIFF_UNIT_T:
+                            picks_list['mag'].append(k)
+                        elif ch['unit'] == FIFF.FIFF_UNIT_T_M:
+                            picks_list['grad'].append(k)
+                else:
+                    pass  # not a data channel type
+    picks_list = [(ch_type, np.array(picks_list[ch_type], int))
+                  for ch_type in _DATA_CH_TYPES_SPLIT]
+    assert _DATA_CH_TYPES_SPLIT[:2] == ('mag', 'grad')
+    if meg_combined and len(picks_list[0][1]) and len(picks_list[1][1]):
+        picks_list.insert(
+            0, ('meg', np.unique(np.concatenate([picks_list.pop(0)[1],
+                                                 picks_list.pop(0)[1]])))
         )
-    if has['grad'] and (meg_combined is not True or not has['mag']):
-        picks_list.append(
-            ('grad', pick_types(info, meg='grad', eeg=False, stim=False,
-             ref_meg=ref_meg, exclude=exclude))
-        )
-    if has['mag'] and has['grad'] and meg_combined is True:
-        picks_list.append(
-            ('meg', pick_types(info, meg=True, eeg=False, stim=False,
-             ref_meg=ref_meg, exclude=exclude))
-        )
-    for ch_type in _DATA_CH_TYPES_SPLIT:
-        if ch_type in ['grad', 'mag']:  # exclude just MEG channels
-            continue
-        if has[ch_type]:
-            picks_list.append(
-                (ch_type, pick_types(info, meg=False, stim=False,
-                 ref_meg=ref_meg, exclude=exclude, **{ch_type: True}))
-            )
+    picks_list = [p for p in picks_list if len(p[1])]
     return picks_list
 
 
@@ -753,7 +841,7 @@ def _check_excludes_includes(chs, info=None, allow_bads=False):
         else:
             raise ValueError(
                 'include/exclude must be list, tuple, ndarray, or "bads". ' +
-                'You provided type {0}'.format(type(chs)))
+                'You provided type {}'.format(type(chs)))
     return chs
 
 
@@ -762,12 +850,19 @@ _PICK_TYPES_DATA_DICT = dict(
     misc=False, resp=False, chpi=False, exci=False, ias=False, syst=False,
     seeg=True, dipole=False, gof=False, bio=False, ecog=True, fnirs=True)
 _PICK_TYPES_KEYS = tuple(list(_PICK_TYPES_DATA_DICT.keys()) + ['ref_meg'])
-_DATA_CH_TYPES_SPLIT = ['mag', 'grad', 'eeg', 'seeg', 'ecog', 'hbo', 'hbr']
+_DATA_CH_TYPES_SPLIT = ('mag', 'grad', 'eeg', 'seeg', 'ecog', 'hbo', 'hbr')
+_DATA_CH_TYPES_ORDER_DEFAULT = ('mag', 'grad', 'eeg', 'eog', 'ecg', 'emg',
+                                'ref_meg', 'misc', 'stim', 'resp',
+                                'chpi', 'exci', 'ias', 'syst', 'seeg', 'bio',
+                                'ecog', 'hbo', 'hbr', 'whitened')
 
 # Valid data types, ordered for consistency, used in viz/evoked.
-_VALID_CHANNEL_TYPES = ['eeg', 'grad', 'mag', 'seeg', 'eog', 'ecg', 'emg',
+_VALID_CHANNEL_TYPES = ('eeg', 'grad', 'mag', 'seeg', 'eog', 'ecg', 'emg',
                         'dipole', 'gof', 'bio', 'ecog', 'hbo', 'hbr',
-                        'misc']
+                        'misc')
+
+_MEG_CH_TYPES_SPLIT = ('mag', 'grad', 'planar1', 'planar2')
+_FNIRS_CH_TYPES_SPLIT = ('hbo', 'hbr')
 
 
 def _pick_data_channels(info, exclude='bads', with_ref_meg=True):
@@ -785,12 +880,178 @@ def _pick_aux_channels(info, exclude='bads'):
                       ref_meg=False, exclude=exclude)
 
 
-def _pick_data_or_ica(info):
+def _pick_data_or_ica(info, exclude=()):
     """Pick only data or ICA channels."""
     if any(ch_name.startswith('ICA') for ch_name in info['ch_names']):
-        picks = pick_types(info, exclude=[], misc=True)
+        picks = pick_types(info, exclude=exclude, misc=True)
     else:
-        picks = _pick_data_channels(info, exclude=[], with_ref_meg=True)
+        picks = _pick_data_channels(info, exclude=exclude, with_ref_meg=True)
+    return picks
+
+
+def _picks_to_idx(info, picks, none='data', exclude='bads', allow_empty=False,
+                  with_ref_meg=True, return_kind=False):
+    """Convert and check pick validity."""
+    from .meas_info import Info
+    picked_ch_type_or_generic = False
+    #
+    # None -> all, data, or data_or_ica (ndarray of int)
+    #
+    orig_picks = picks
+    orig_repr = repr(orig_picks)
+    if isinstance(info, Info):
+        n_chan = info['nchan']
+    else:
+        info = _ensure_int(info, 'info', 'an int or Info')
+        n_chan = info
+    assert n_chan >= 0
+
+    if picks is None:
+        if isinstance(info, int):  # special wrapper for no real info
+            picks = np.arange(n_chan)
+            orig_repr += ', treated as range(%d)' % (n_chan,)
+        else:
+            picks = none  # let _picks_str_to_idx handle it
+            orig_repr += ', treated as "%s"' % (none,)
+
+    #
+    # slice
+    #
+    if isinstance(picks, slice):
+        picks = np.arange(n_chan)[picks]
+
+    #
+    # -> ndarray of int (and make a copy)
+    #
+    picks = np.atleast_1d(picks)  # this works even for picks == 'something'
+    picks = np.array([], dtype=int) if len(picks) == 0 else picks
+    if picks.ndim != 1:
+        raise ValueError('picks must be 1D, got %sD' % (picks.ndim,))
+    if picks.dtype.char in ('S', 'U'):
+        picks = _picks_str_to_idx(info, picks, exclude, with_ref_meg,
+                                  return_kind, orig_repr, allow_empty)
+        if return_kind:
+            picked_ch_type_or_generic = picks[1]
+            picks = picks[0]
+    if picks.dtype.kind not in ['i', 'u']:
+        raise TypeError('picks must be a list of int or list of str, got '
+                        'a data type of %s' % (picks.dtype,))
+    picks = picks.astype(int)
+
+    #
+    # ensure we have (optionally non-empty) ndarray of valid int
+    #
+    if len(picks) == 0 and not allow_empty:
+        raise ValueError('No appropriate channels found for the given picks '
+                         '(%r)' % (orig_picks,))
+    if (picks < -n_chan).any():
+        raise ValueError('All picks must be >= %d, got %r'
+                         % (-n_chan, orig_picks))
+    if (picks >= n_chan).any():
+        raise ValueError('All picks must be < n_channels (%d), got %r'
+                         % (n_chan, orig_picks))
+    picks %= n_chan  # ensure positive
+    if return_kind:
+        return picks, picked_ch_type_or_generic
+    return picks
+
+
+def _picks_str_to_idx(info, picks, exclude, with_ref_meg, return_kind,
+                      orig_repr, allow_empty):
+    """Turn a list of str into ndarray of int."""
+    # special case for _picks_to_idx w/no info: shouldn't really happen
+    if isinstance(info, int):
+        raise ValueError('picks as str can only be used when measurement '
+                         'info is available')
+
+    #
+    # first: check our special cases
+    #
+
+    picks_generic = list()
+    if len(picks) == 1:
+        if picks[0] in ('all', 'data', 'data_or_ica'):
+            if picks[0] == 'all':
+                use_exclude = info['bads'] if exclude == 'bads' else exclude
+                picks_generic = pick_channels(
+                    info['ch_names'], info['ch_names'], exclude=use_exclude)
+            elif picks[0] == 'data':
+                picks_generic = _pick_data_channels(info, exclude=exclude,
+                                                    with_ref_meg=with_ref_meg)
+            elif picks[0] == 'data_or_ica':
+                picks_generic = _pick_data_or_ica(info, exclude=exclude)
+            if len(picks_generic) == 0 and orig_repr.startswith('None, ') and \
+                    not allow_empty:
+                raise ValueError('picks (%s) yielded no channels, consider '
+                                 'passing picks explicitly' % (orig_repr,))
+
+    #
+    # second: match all to channel names
+    #
+
+    bad_name = None
+    picks_name = list()
+    for pick in picks:
+        try:
+            picks_name.append(info['ch_names'].index(pick))
+        except ValueError:
+            bad_name = pick
+            break
+
+    #
+    # third: match all to types
+    #
+    bad_type = None
+    picks_type = list()
+    kwargs = dict(meg=False)
+    meg, fnirs = set(), set()
+    for pick in picks:
+        if pick in _PICK_TYPES_KEYS:
+            kwargs[pick] = True
+        elif pick in _MEG_CH_TYPES_SPLIT:
+            meg |= {pick}
+        elif pick in _FNIRS_CH_TYPES_SPLIT:
+            fnirs |= {pick}
+        else:
+            bad_type = pick
+            break
+    else:
+        # triage MEG and FNIRS, which are complicated due to non-bool entries
+        extra_picks = set()
+        if len(meg) > 0 and not kwargs.get('meg', False):
+            # easiest just to iterate
+            for use_meg in meg:
+                extra_picks |= set(pick_types(
+                    info, meg=use_meg, ref_meg=False, exclude=exclude))
+        if len(fnirs) > 0 and not kwargs.get('fnirs', False):
+            # if it has two entries, it's both, otherwise it's just one
+            kwargs['fnirs'] = True if len(fnirs) == 2 else list(fnirs)[0]
+        picks_type = pick_types(info, exclude=exclude, **kwargs)
+        if len(extra_picks) > 0:
+            picks_type = sorted(set(picks_type) | set(extra_picks))
+
+    #
+    # finally: ensure we have exactly one usable list
+    #
+    all_picks = (picks_generic, picks_name, picks_type)
+    any_found = [len(p) > 0 for p in all_picks]
+    if sum(any_found) == 0:
+        if not allow_empty:
+            raise ValueError(
+                'picks (%s) could not be interpreted as '
+                'channel names (no channel "%s"), channel types (no '
+                'type "%s"), or a generic type (just "all" or "data")'
+                % (orig_repr, bad_name, bad_type))
+        picks = np.array([], int)
+    elif sum(any_found) > 1:
+        raise RuntimeError('Some channel names are ambiguously equivalent to '
+                           'channel types, cannot use string-based '
+                           'picks for these')
+    else:
+        picks = np.array(all_picks[np.where(any_found)[0][0]])
+    if return_kind:
+        picked_ch_type_or_generic = not len(picks_name)
+        return picks, picked_ch_type_or_generic
     return picks
 
 
@@ -798,11 +1059,8 @@ def _pick_inst(inst, picks, exclude, copy=True):
     """Return an instance with picked and excluded channels."""
     if copy is True:
         inst = inst.copy()
-    if picks is not None:
-        pick_names = [inst.info['ch_names'][pick] for pick in picks]
-    else:  # only pick channels that are plotted
-        picks = _pick_data_channels(inst.info, exclude=[])
-        pick_names = [inst.info['ch_names'][pick] for pick in picks]
+    picks = _picks_to_idx(inst.info, picks, exclude=[])
+    pick_names = [inst.info['ch_names'][pick] for pick in picks]
     inst.pick_channels(pick_names)
 
     if exclude == 'bads':
