@@ -20,6 +20,11 @@ def bad_2(x):
     return x[:-1]  # bad shape
 
 
+def bad_3(x):
+    """Fail."""
+    return x[0, :]
+
+
 def printer(x):
     """Print."""
     logger.info('exec')
@@ -35,10 +40,21 @@ def test_apply_function_verbose():
     raw = RawArray(np.zeros((n_chan, n_times)),
                    create_info(ch_names, 1., 'mag'))
     # test return types in both code paths (parallel / 1 job)
-    pytest.raises(TypeError, raw.apply_function, bad_1)
-    pytest.raises(ValueError, raw.apply_function, bad_2)
-    pytest.raises(TypeError, raw.apply_function, bad_1, n_jobs=2)
-    pytest.raises(ValueError, raw.apply_function, bad_2, n_jobs=2)
+    with pytest.raises(TypeError, match='Return value must be an ndarray'):
+        raw.apply_function(bad_1)
+    with pytest.raises(ValueError, match='Return data must have shape'):
+        raw.apply_function(bad_2)
+    with pytest.raises(TypeError, match='Return value must be an ndarray'):
+        raw.apply_function(bad_1, n_jobs=2)
+    with pytest.raises(ValueError, match='Return data must have shape'):
+        raw.apply_function(bad_2, n_jobs=2)
+
+    # test return type when `channel_wise=False`
+    raw.apply_function(printer, channel_wise=False)
+    with pytest.raises(TypeError, match='Return value must be an ndarray'):
+        raw.apply_function(bad_1, channel_wise=False)
+    with pytest.raises(ValueError, match='Return data must have shape'):
+        raw.apply_function(bad_3, channel_wise=False)
 
     # check our arguments
     with catch_logging() as sio:
