@@ -6,7 +6,6 @@
 
 import hashlib
 import os.path as op
-from six import string_types
 
 
 import pytest
@@ -21,14 +20,15 @@ from mne.io import (read_fiducials, write_fiducials, _coil_trans_to_loc,
                     _loc_to_coil_trans, read_raw_fif, read_info, write_info,
                     anonymize_info)
 from mne.io.constants import FIFF
-from mne.io.write import DATE_NONE
-from mne.io.meas_info import (Info, create_info, _write_dig_points,
-                              _read_dig_points, _make_dig_points, _merge_info,
+from mne.io.write import DATE_NONE, _generate_meas_id
+from mne.io.meas_info import (Info, create_info, _merge_info,
                               _force_update_info, RAW_INFO_FIELDS,
                               _bad_chans_comp, _get_valid_units)
+from mne._digitization._utils import (_write_dig_points, _read_dig_points,
+                                      _make_dig_points,)
 from mne.io import read_raw_ctf
-from mne.utils import _TempDir, run_tests_if_main, catch_logging
-from mne.channels.montage import read_montage, read_dig_montage
+from mne.utils import run_tests_if_main, catch_logging, assert_object_equal
+from mne.channels import make_standard_montage
 
 base_dir = op.join(op.dirname(__file__), 'data')
 fiducials_fname = op.join(base_dir, 'fsaverage-fiducials.fif')
@@ -50,7 +50,7 @@ def test_get_valid_units():
     """Test the valid units."""
     valid_units = _get_valid_units()
     assert isinstance(valid_units, tuple)
-    assert all(isinstance(unit, string_types) for unit in valid_units)
+    assert all(isinstance(unit, str) for unit in valid_units)
     assert "n/a" in valid_units
 
 
@@ -70,7 +70,7 @@ def test_make_info():
     info = create_info(n_ch, 1000., 'eeg')
     assert set(info.keys()) == set(RAW_INFO_FIELDS)
 
-    coil_types = set([ch['coil_type'] for ch in info['chs']])
+    coil_types = {ch['coil_type'] for ch in info['chs']}
     assert FIFF.FIFFV_COIL_EEG in coil_types
 
     pytest.raises(TypeError, create_info, ch_names='Test Ch', sfreq=1000)
@@ -79,49 +79,46 @@ def test_make_info():
                   ch_types=['eeg', 'eeg'])
     pytest.raises(TypeError, create_info, ch_names=[np.array([1])],
                   sfreq=1000)
-    pytest.raises(TypeError, create_info, ch_names=['Test Ch'], sfreq=1000,
+    pytest.raises(KeyError, create_info, ch_names=['Test Ch'], sfreq=1000,
                   ch_types=np.array([1]))
     pytest.raises(KeyError, create_info, ch_names=['Test Ch'], sfreq=1000,
                   ch_types='awesome')
     pytest.raises(TypeError, create_info, ['Test Ch'], sfreq=1000,
                   ch_types=None, montage=np.array([1]))
-    m = read_montage('biosemi32')
+    m = make_standard_montage('biosemi32')
     info = create_info(ch_names=m.ch_names, sfreq=1000., ch_types='eeg',
                        montage=m)
     ch_pos = [ch['loc'][:3] for ch in info['chs']]
-    assert_array_equal(ch_pos, m.pos)
+    ch_pos_mon = m._get_ch_pos()
+    ch_pos_mon = np.array(
+        [ch_pos_mon[ch_name] for ch_name in info['ch_names']])
+    # transform to head
+    ch_pos_mon += (0., 0., 0.04014)
+    assert_allclose(ch_pos, ch_pos_mon, atol=1e-5)
 
-    names = ['nasion', 'lpa', 'rpa', '1', '2', '3', '4', '5']
-    d = read_dig_montage(hsp_fname, None, elp_fname, names, unit='m',
-                         transform=False)
-    info = create_info(ch_names=m.ch_names, sfreq=1000., ch_types='eeg',
-                       montage=d)
-    idents = [p['ident'] for p in info['dig']]
-    assert FIFF.FIFFV_POINT_NASION in idents
-
-    info = create_info(ch_names=m.ch_names, sfreq=1000., ch_types='eeg',
-                       montage=[d, m])
-    ch_pos = [ch['loc'][:3] for ch in info['chs']]
-    assert_array_equal(ch_pos, m.pos)
-    idents = [p['ident'] for p in info['dig']]
-    assert (FIFF.FIFFV_POINT_NASION in idents)
-    info = create_info(ch_names=m.ch_names, sfreq=1000., ch_types='eeg',
-                       montage=[d, 'biosemi32'])
-    ch_pos = [ch['loc'][:3] for ch in info['chs']]
-    assert_array_equal(ch_pos, m.pos)
-    idents = [p['ident'] for p in info['dig']]
-    assert (FIFF.FIFFV_POINT_NASION in idents)
-    assert info['meas_date'] is None
+    with pytest.deprecated_call():
+        info = create_info(ch_names=m.ch_names, sfreq=1000., ch_types='eeg',
+                           montage=[m, m, m, m, m, m, m])
 
 
-def test_fiducials_io():
+def test_duplicate_name_correction():
+    """Test duplicate channel names with running number."""
+    # When running number is possible
+    info = create_info(['A', 'A', 'A'], 1000., verbose='error')
+    assert info['ch_names'] == ['A-0', 'A-1', 'A-2']
+
+    # When running number is not possible
+    with pytest.raises(ValueError, match='Adding a running number'):
+        create_info(['A', 'A', 'A-0'], 1000., verbose='error')
+
+
+def test_fiducials_io(tmpdir):
     """Test fiducials i/o."""
-    tempdir = _TempDir()
     pts, coord_frame = read_fiducials(fiducials_fname)
     assert pts[0]['coord_frame'] == FIFF.FIFFV_COORD_MRI
     assert pts[0]['ident'] == FIFF.FIFFV_POINT_CARDINAL
 
-    temp_fname = op.join(tempdir, 'test.fif')
+    temp_fname = tmpdir.join('test.fif')
     write_fiducials(temp_fname, pts, coord_frame)
     pts_1, coord_frame_1 = read_fiducials(temp_fname)
     assert coord_frame == coord_frame_1
@@ -142,8 +139,7 @@ def test_info():
     event_id, tmin, tmax = 1, -0.2, 0.5
     events = read_events(event_name)
     event_id = int(events[0, 2])
-    epochs = Epochs(raw, events[:1], event_id, tmin, tmax, picks=None,
-                    baseline=(None, 0))
+    epochs = Epochs(raw, events[:1], event_id, tmin, tmax, picks=None)
 
     evoked = epochs.average()
 
@@ -160,7 +156,17 @@ def test_info():
         info_str = '%s' % obj.info
         assert len(info_str.split('\n')) == len(obj.info.keys()) + 2
         assert all(k in info_str for k in obj.info.keys())
-        assert '2002-12-03 19:01:10 GMT' in repr(obj.info), repr(obj.info)
+        rep = repr(obj.info)
+        assert '2002-12-03 19:01:10 GMT' in rep, rep
+        assert '146 items (3 Cardinal, 4 HPI, 61 EEG, 78 Extra)' in rep
+        dig_rep = repr(obj.info['dig'][0])
+        assert 'LPA' in dig_rep, dig_rep
+        assert '(-71.4, 0.0, 0.0) mm' in dig_rep, dig_rep
+        assert 'head frame' in dig_rep, dig_rep
+        # Test our BunchConstNamed support
+        for func in (str, repr):
+            assert '4 (FIFFV_COORD_HEAD)' == \
+                func(obj.info['dig'][0]['coord_frame'])
 
     # Test read-only fields
     info = raw.info.copy()
@@ -189,11 +195,10 @@ def test_info():
     assert info == info2
 
 
-def test_read_write_info():
+def test_read_write_info(tmpdir):
     """Test IO of info."""
-    tempdir = _TempDir()
     info = read_info(raw_fname)
-    temp_file = op.join(tempdir, 'info.fif')
+    temp_file = str(tmpdir.join('info.fif'))
     # check for bug `#1198`
     info['dev_head_t']['trans'] = np.eye(4)
     t1 = info['dev_head_t']['trans']
@@ -233,25 +238,26 @@ def test_read_write_info():
     with open(temp_file, 'rb') as fid:
         m1.update(fid.read())
     m1 = m1.hexdigest()
-    temp_file_2 = op.join(tempdir, 'info2.fif')
+    temp_file_2 = tmpdir.join('info2.fif')
     assert temp_file_2 != temp_file
     write_info(temp_file_2, info)
     m2 = hashlib.md5()
-    with open(temp_file_2, 'rb') as fid:
+    with open(str(temp_file_2), 'rb') as fid:
         m2.update(fid.read())
     m2 = m2.hexdigest()
     assert m1 == m2
 
 
-def test_io_dig_points():
+def test_io_dig_points(tmpdir):
     """Test Writing for dig files."""
-    tempdir = _TempDir()
     points = _read_dig_points(hsp_fname)
 
-    dest = op.join(tempdir, 'test.txt')
-    dest_bad = op.join(tempdir, 'test.mne')
-    pytest.raises(ValueError, _write_dig_points, dest, points[:, :2])
-    pytest.raises(ValueError, _write_dig_points, dest_bad, points)
+    dest = str(tmpdir.join('test.txt'))
+    dest_bad = str(tmpdir.join('test.mne'))
+    with pytest.raises(ValueError, match='must be of shape'):
+        _write_dig_points(dest, points[:, :2])
+    with pytest.raises(ValueError, match='extension'):
+        _write_dig_points(dest_bad, points)
     _write_dig_points(dest, points)
     points1 = _read_dig_points(dest, unit='m')
     err = "Dig points diverged after writing and reading."
@@ -259,7 +265,8 @@ def test_io_dig_points():
 
     points2 = np.array([[-106.93, 99.80], [99.80, 68.81]])
     np.savetxt(dest, points2, delimiter='\t', newline='\n')
-    pytest.raises(ValueError, _read_dig_points, dest)
+    with pytest.raises(ValueError, match='must be of shape'):
+        _read_dig_points(dest)
 
 
 def test_make_dig_points():
@@ -465,7 +472,7 @@ def _is_anonymous(inst):
                             anonymous_annotations)
 
 
-def test_anonymize():
+def test_anonymize(tmpdir):
     """Test that sensitive information can be anonymized."""
     pytest.raises(TypeError, anonymize_info, 'foo')
 
@@ -486,7 +493,7 @@ def test_anonymize():
 
     # Test instance method
     events = read_events(event_name)
-    epochs = Epochs(raw, events[:1], 2, 0., 0.1)
+    epochs = Epochs(raw, events[:1], 2, 0., 0.1, baseline=None)
 
     assert not any(_is_anonymous(raw))
     raw.anonymize()
@@ -498,14 +505,13 @@ def test_anonymize():
 
     # When we write out with raw.save, these get overwritten with the
     # new save time
-    tempdir = _TempDir()
-    out_fname = op.join(tempdir, 'test_subj_info_raw.fif')
+    out_fname = tmpdir.join('test_subj_info_raw.fif')
     raw.save(out_fname, overwrite=True)
     assert all(_is_anonymous(read_raw_fif(out_fname)))
 
 
 @testing.requires_testing_data
-def test_csr_csc():
+def test_csr_csc(tmpdir):
     """Test CSR and CSC."""
     info = read_info(sss_ctc_fname)
     info = pick_info(info, pick_types(info, meg=True, exclude=[]))
@@ -513,8 +519,7 @@ def test_csr_csc():
     ct = sss_ctc['decoupler'].copy()
     # CSC
     assert isinstance(ct, sparse.csc_matrix)
-    tempdir = _TempDir()
-    fname = op.join(tempdir, 'test.fif')
+    fname = tmpdir.join('test.fif')
     write_info(fname, info)
     info_read = read_info(fname)
     ct_read = info_read['proc_history'][0]['max_info']['sss_ctc']['decoupler']
@@ -525,7 +530,7 @@ def test_csr_csc():
     assert isinstance(csr, sparse.csr_matrix)
     assert_array_equal(csr.toarray(), ct.toarray())
     info['proc_history'][0]['max_info']['sss_ctc']['decoupler'] = csr
-    fname = op.join(tempdir, 'test1.fif')
+    fname = tmpdir.join('test1.fif')
     write_info(fname, info)
     info_read = read_info(fname)
     ct_read = info_read['proc_history'][0]['max_info']['sss_ctc']['decoupler']
@@ -561,4 +566,20 @@ def test_check_compensation_consistency():
             assert'Removing 5 compensators' in log.getvalue()
 
 
+def test_field_round_trip(tmpdir):
+    """Test round-trip for new fields."""
+    info = create_info(1, 1000., 'eeg')
+    for key in ('file_id', 'meas_id'):
+        info[key] = _generate_meas_id()
+    info['device_info'] = dict(
+        type='a', model='b', serial='c', site='d')
+    info['helium_info'] = dict(
+        he_level_raw=1., helium_level=2., orig_file_guid='e', meas_date=(1, 2))
+    fname = tmpdir.join('temp-info.fif')
+    write_info(fname, info)
+    info_read = read_info(fname)
+    info_read['dig'] = None  # XXX eventually this should go away
+    assert_object_equal(info, info_read)
+
+
 run_tests_if_main()
