@@ -63,10 +63,7 @@ def infomax(data, weights=None, l_rate=None, block=None, w_change=1e-12,
         Defaults to 1.
     max_iter : int
         The maximum number of iterations. Defaults to 200.
-    random_state : int | np.random.RandomState
-        If random_state is an int, use random_state to seed the random number
-        generator. If random_state is already a np.random.RandomState instance,
-        use random_state as random number generator.
+    %(random_state)s
     blowup : float
         The maximum difference allowed between two successive estimations of
         the unmixing matrix. Defaults to 10000.
@@ -82,9 +79,7 @@ def infomax(data, weights=None, l_rate=None, block=None, w_change=1e-12,
     use_bias : bool
         This quantity indicates if the bias should be computed.
         Defaults to True.
-    verbose : bool, str, int, or None
-        If not None, override default verbosity level (see :func:`mne.verbose`
-        and :ref:`Logging documentation <tut_logging>` for more).
+    %(verbose)s
 
     Returns
     -------
