@@ -13,8 +13,7 @@ from ..decoding import TransformerMixin, BaseEstimator
 from ..epochs import BaseEpochs
 from ..io import BaseRaw
 from ..io.pick import _pick_data_channels, pick_info
-from ..utils import logger
-from ..externals.six import iteritems, itervalues
+from ..utils import logger, _check_option
 
 
 def _construct_signal_from_epochs(epochs, events, sfreq, tmin):
@@ -143,7 +142,8 @@ def _fit_xdawn(epochs_data, y, n_components, reg=None, signal_cov=None,
     evokeds : array, shape (n_class, n_components, n_times)
         The independent evoked responses per condition.
     """
-    n_epochs, n_channels, n_times = epochs_data.shape
+    if not isinstance(epochs_data, np.ndarray) or epochs_data.ndim != 3:
+        raise ValueError('epochs_data must be 3D ndarray')
 
     classes = np.unique(y)
 
@@ -318,7 +318,6 @@ class _XdawnTransformer(BaseEstimator, TransformerMixin):
         """
         # Check size
         X, _ = self._check_Xy(X)
-        n_components, n_channels = self.patterns_.shape
         n_epochs, n_comp, n_times = X.shape
         if n_comp != (self.n_components * len(self.classes_)):
             raise ValueError('X must have %i components, got %i instead' % (
@@ -352,7 +351,7 @@ class Xdawn(_XdawnTransformer):
 
     Parameters
     ----------
-    n_components : int (default 2)
+    n_components : int, (default 2)
         The number of components to decompose the signals.
     signal_cov : None | Covariance | ndarray, shape (n_channels, n_channels)
         (default None). The signal covariance used for whitening of the data.
@@ -376,9 +375,9 @@ class Xdawn(_XdawnTransformer):
     patterns_ : dict of ndarray
         If fit, the Xdawn patterns used to restore the signals for each event
         type, else empty.
-    evokeds_ : dict of evoked instance
+    evokeds_ : dict of Evoked
         If fit, the evoked response for each event type.
-    event_id_ : dict of event id
+    event_id_ : dict
         The event id.
     correct_overlap_ : bool
         Whether overlap correction was applied.
@@ -410,8 +409,8 @@ class Xdawn(_XdawnTransformer):
         """Init."""
         super(Xdawn, self).__init__(n_components=n_components,
                                     signal_cov=signal_cov, reg=reg)
-        if correct_overlap not in ['auto', True, False]:
-            raise ValueError('correct_overlap must be a bool or "auto"')
+        _check_option('correct_overlap', correct_overlap,
+                      ['auto', True, False])
         self.correct_overlap = correct_overlap
 
     def fit(self, epochs, y=None):
@@ -419,14 +418,14 @@ class Xdawn(_XdawnTransformer):
 
         Parameters
         ----------
-        epochs : Epochs object
+        epochs : instance of Epochs
             An instance of Epoch on which Xdawn filters will be fitted.
         y : ndarray | None (default None)
             If None, used epochs.events[:, 2].
 
         Returns
         -------
-        self : Xdawn instance
+        self : instance of Xdawn
             The Xdawn instance.
         """
         # Check data
@@ -472,7 +471,7 @@ class Xdawn(_XdawnTransformer):
         filters = filters.reshape(-1, n_components, filters.shape[-1])
         patterns = patterns.reshape(-1, n_components, patterns.shape[-1])
         self.filters_, self.patterns_, self.evokeds_ = dict(), dict(), dict()
-        idx = np.argsort([value for _, value in iteritems(epochs.event_id)])
+        idx = np.argsort([value for _, value in epochs.event_id.items()])
         for eid, this_filter, this_pattern, this_evo in zip(
                 epochs.event_id, filters[idx], patterns[idx], evokeds[idx]):
             self.filters_[eid] = this_filter.T
@@ -508,7 +507,7 @@ class Xdawn(_XdawnTransformer):
             raise ValueError('Data input must be of Epoch type or numpy array')
 
         filters = [filt[:self.n_components]
-                   for filt in itervalues(self.filters_)]
+                   for filt in self.filters_.values()]
         filters = np.concatenate(filters, axis=0)
         X = np.dot(filters, X)
         if X.ndim == 3:
@@ -541,7 +540,7 @@ class Xdawn(_XdawnTransformer):
 
         Returns
         -------
-        out : dict of instance
+        out : dict
             A dict of instance (from the same type as inst input) for each
             event type in event_id.
         """
