@@ -1,4 +1,4 @@
-# Authors: Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
+# Authors: Alexandre Gramfort <alexandre.gramfort@inria.fr>
 #
 # License: BSD (3-clause)
 
@@ -6,7 +6,7 @@ import numpy as np
 from scipy import linalg
 
 from . import io, Epochs
-from .utils import check_fname, logger, verbose
+from .utils import check_fname, logger, verbose, _check_option
 from .io.pick import pick_types, pick_types_forward
 from .io.proj import Projection, _has_eeg_average_ref_proj
 from .event import make_fixed_length_events
@@ -16,6 +16,7 @@ from .forward import (is_fixed_orient, _subject_from_forward,
                       convert_forward_solution)
 from .source_estimate import SourceEstimate, VolSourceEstimate
 from .io.proj import make_projector, make_eeg_average_ref_proj
+from .rank import _get_rank_sss
 
 
 def read_proj(fname):
@@ -70,12 +71,30 @@ def write_proj(fname, projs):
 
 
 @verbose
-def _compute_proj(data, info, n_grad, n_mag, n_eeg, desc_prefix, verbose=None):
-    mag_ind = pick_types(info, meg='mag', ref_meg=False, exclude='bads')
+def _compute_proj(data, info, n_grad, n_mag, n_eeg, desc_prefix,
+                  meg='separate', verbose=None):
     grad_ind = pick_types(info, meg='grad', ref_meg=False, exclude='bads')
+    mag_ind = pick_types(info, meg='mag', ref_meg=False, exclude='bads')
     eeg_ind = pick_types(info, meg=False, eeg=True, ref_meg=False,
                          exclude='bads')
 
+    _check_option('meg', meg, ['separate', 'combined'])
+    if meg == 'combined':
+        _get_rank_sss(info, msg='meg="combined" can only be used with '
+                      'Maxfiltered data', verbose=False)
+        if n_grad != n_mag:
+            raise ValueError('n_grad (%d) must be equal to n_mag (%d) when '
+                             'using meg="combined"')
+        kinds = ['meg', '', 'eeg']
+        n_mag = 0
+        grad_ind = pick_types(info, meg=True, ref_meg=False, exclude='bads')
+        if (n_grad > 0) and len(grad_ind) == 0:
+            logger.info("No MEG channels found for joint estimation. "
+                        "Forcing n_grad=n_mag=0")
+            n_grad = 0
+    else:
+        kinds = ['planar', 'axial', 'eeg']
+
     if (n_grad > 0) and len(grad_ind) == 0:
         logger.info("No gradiometers found. Forcing n_grad to 0")
         n_grad = 0
@@ -95,7 +114,7 @@ def _compute_proj(data, info, n_grad, n_mag, n_eeg, desc_prefix, verbose=None):
     for n, ind, names, desc in zip([n_grad, n_mag, n_eeg],
                                    [grad_ind, mag_ind, eeg_ind],
                                    [grad_names, mag_names, eeg_names],
-                                   ['planar', 'axial', 'eeg']):
+                                   kinds):
         if n == 0:
             continue
         data_ind = data[ind][:, ind]
@@ -119,7 +138,7 @@ def _compute_proj(data, info, n_grad, n_mag, n_eeg, desc_prefix, verbose=None):
 
 @verbose
 def compute_proj_epochs(epochs, n_grad=2, n_mag=2, n_eeg=2, n_jobs=1,
-                        desc_prefix=None, verbose=None):
+                        desc_prefix=None, meg='separate', verbose=None):
     """Compute SSP (spatial space projection) vectors on Epochs.
 
     Parameters
@@ -132,14 +151,19 @@ def compute_proj_epochs(epochs, n_grad=2, n_mag=2, n_eeg=2, n_jobs=1,
         Number of vectors for magnetometers
     n_eeg : int
         Number of vectors for EEG channels
-    n_jobs : int
+    %(n_jobs)s
         Number of jobs to use to compute covariance
     desc_prefix : str | None
         The description prefix to use. If None, one will be created based on
         the event_id, tmin, and tmax.
-    verbose : bool, str, int, or None
-        If not None, override default verbose level (see :func:`mne.verbose`
-        and :ref:`Logging documentation <tut_logging>` for more).
+    meg : str
+        Can be 'separate' (default) or 'combined' to compute projectors
+        for magnetometers and gradiometers separately or jointly.
+        If 'combined', ``n_mag == n_grad`` is required and the number of
+        projectors computed for MEG will be ``n_mag``.
+
+        .. versionadded:: 0.18
+    %(verbose)s
 
     Returns
     -------
@@ -161,7 +185,8 @@ def compute_proj_epochs(epochs, n_grad=2, n_mag=2, n_eeg=2, n_jobs=1,
         event_id = 'Multiple-events'
     if desc_prefix is None:
         desc_prefix = "%s-%-.3f-%-.3f" % (event_id, epochs.tmin, epochs.tmax)
-    return _compute_proj(data, epochs.info, n_grad, n_mag, n_eeg, desc_prefix)
+    return _compute_proj(data, epochs.info, n_grad, n_mag, n_eeg, desc_prefix,
+                         meg=meg)
 
 
 def _compute_cov_epochs(epochs, n_jobs):
@@ -180,7 +205,7 @@ def _compute_cov_epochs(epochs, n_jobs):
 
 @verbose
 def compute_proj_evoked(evoked, n_grad=2, n_mag=2, n_eeg=2, desc_prefix=None,
-                        verbose=None):
+                        meg='separate', verbose=None):
     """Compute SSP (spatial space projection) vectors on Evoked.
 
     Parameters
@@ -198,9 +223,14 @@ def compute_proj_evoked(evoked, n_grad=2, n_mag=2, n_eeg=2, desc_prefix=None,
         tmin and tmax.
 
         .. versionadded:: 0.17
-    verbose : bool, str, int, or None
-        If not None, override default verbose level (see :func:`mne.verbose`
-        and :ref:`Logging documentation <tut_logging>` for more).
+    meg : str
+        Can be 'separate' (default) or 'combined' to compute projectors
+        for magnetometers and gradiometers separately or jointly.
+        If 'combined', ``n_mag == n_grad`` is required and the number of
+        projectors computed for MEG will be ``n_mag``.
+
+        .. versionadded:: 0.18
+    %(verbose)s
 
     Returns
     -------
@@ -214,12 +244,14 @@ def compute_proj_evoked(evoked, n_grad=2, n_mag=2, n_eeg=2, desc_prefix=None,
     data = np.dot(evoked.data, evoked.data.T)  # compute data covariance
     if desc_prefix is None:
         desc_prefix = "%-.3f-%-.3f" % (evoked.times[0], evoked.times[-1])
-    return _compute_proj(data, evoked.info, n_grad, n_mag, n_eeg, desc_prefix)
+    return _compute_proj(data, evoked.info, n_grad, n_mag, n_eeg, desc_prefix,
+                         meg=meg)
 
 
 @verbose
 def compute_proj_raw(raw, start=0, stop=None, duration=1, n_grad=2, n_mag=2,
-                     n_eeg=0, reject=None, flat=None, n_jobs=1, verbose=None):
+                     n_eeg=0, reject=None, flat=None, n_jobs=1, meg='separate',
+                     verbose=None):
     """Compute SSP (spatial space projection) vectors on Raw.
 
     Parameters
@@ -244,11 +276,16 @@ def compute_proj_raw(raw, start=0, stop=None, duration=1, n_grad=2, n_mag=2,
         Epoch rejection configuration (see Epochs).
     flat : dict | None
         Epoch flat configuration (see Epochs).
-    n_jobs : int
+    %(n_jobs)s
         Number of jobs to use to compute covariance.
-    verbose : bool, str, int, or None
-        If not None, override default verbose level (see :func:`mne.verbose`
-        and :ref:`Logging documentation <tut_logging>` for more).
+    meg : str
+        Can be 'separate' (default) or 'combined' to compute projectors
+        for magnetometers and gradiometers separately or jointly.
+        If 'combined', ``n_mag == n_grad`` is required and the number of
+        projectors computed for MEG will be ``n_mag``.
+
+        .. versionadded:: 0.18
+    %(verbose)s
 
     Returns
     -------
@@ -260,11 +297,14 @@ def compute_proj_raw(raw, start=0, stop=None, duration=1, n_grad=2, n_mag=2,
     compute_proj_epochs, compute_proj_evoked
     """
     if duration is not None:
+        duration = np.round(duration * raw.info['sfreq']) / raw.info['sfreq']
         events = make_fixed_length_events(raw, 999, start, stop, duration)
         picks = pick_types(raw.info, meg=True, eeg=True, eog=True, ecg=True,
                            emg=True, exclude='bads')
-        epochs = Epochs(raw, events, None, tmin=0., tmax=duration,
-                        picks=picks, reject=reject, flat=flat)
+        epochs = Epochs(raw, events, None, tmin=0.,
+                        tmax=duration - 1. / raw.info['sfreq'],
+                        picks=picks, reject=reject, flat=flat,
+                        baseline=None, proj=False)
         data = _compute_cov_epochs(epochs, n_jobs)
         info = epochs.info
         if not stop:
@@ -283,10 +323,12 @@ def compute_proj_raw(raw, start=0, stop=None, duration=1, n_grad=2, n_mag=2,
         stop = stop / raw.info['sfreq']
 
     desc_prefix = "Raw-%-.3f-%-.3f" % (start, stop)
-    projs = _compute_proj(data, info, n_grad, n_mag, n_eeg, desc_prefix)
+    projs = _compute_proj(data, info, n_grad, n_mag, n_eeg, desc_prefix,
+                          meg=meg)
     return projs
 
 
+@verbose
 def sensitivity_map(fwd, projs=None, ch_type='grad', mode='fixed', exclude=[],
                     verbose=None):
     """Compute sensitivity map.
@@ -310,9 +352,7 @@ def sensitivity_map(fwd, projs=None, ch_type='grad', mode='fixed', exclude=[],
     exclude : list of string | str
         List of channels to exclude. If empty do not exclude any (default).
         If 'bads', exclude channels in fwd['info']['bads'].
-    verbose : bool, str, int, or None
-        If not None, override default verbose level (see :func:`mne.verbose`
-        and :ref:`Logging documentation <tut_logging>` for more).
+    %(verbose)s
 
     Returns
     -------
@@ -321,12 +361,9 @@ def sensitivity_map(fwd, projs=None, ch_type='grad', mode='fixed', exclude=[],
         for visualization.
     """
     # check strings
-    if ch_type not in ['eeg', 'grad', 'mag']:
-        raise ValueError("ch_type should be 'eeg', 'mag' or 'grad (got %s)"
-                         % ch_type)
-    if mode not in ['free', 'fixed', 'ratio', 'radiality', 'angle',
-                    'remaining', 'dampening']:
-        raise ValueError('Unknown mode type (got %s)' % mode)
+    _check_option('ch_type', ch_type, ['eeg', 'grad', 'mag'])
+    _check_option('mode', mode, ['free', 'fixed', 'ratio', 'radiality',
+                                 'angle', 'remaining', 'dampening'])
 
     # check forward
     if is_fixed_orient(fwd, orig=True):
