@@ -2,7 +2,8 @@
 
 from .parametric import (f_threshold_mway_rm, f_mway_rm, f_oneway,
                          _parametric_ci, ttest_1samp_no_p)
-from .permutations import permutation_t_test, _ci, _bootstrap_ci
+from .permutations import (permutation_t_test, _ci,
+                           bootstrap_confidence_interval)
 from .cluster_level import (
     permutation_cluster_test, permutation_cluster_1samp_test,
     spatio_temporal_cluster_test, spatio_temporal_cluster_1samp_test,
