@@ -3,17 +3,18 @@
 # License: BSD 3 clause
 
 from copy import deepcopy
-from os import remove
+from os import remove, makedirs
 import os.path as op
 from shutil import copy
 
 import numpy as np
 import pytest
 from numpy.testing import assert_equal, assert_allclose
+import matplotlib.pyplot as plt
 
 from mne import (make_bem_model, read_bem_surfaces, write_bem_surfaces,
                  make_bem_solution, read_bem_solution, write_bem_solution,
-                 make_sphere_model, Transform, Info)
+                 make_sphere_model, Transform, Info, write_surface)
 from mne.preprocessing.maxfilter import fit_sphere_to_headshape
 from mne.io.constants import FIFF
 from mne.transforms import translation
@@ -26,9 +27,6 @@ from mne.bem import (_ico_downsample, _get_ico_map, _order_surfaces,
 from mne.surface import read_surface
 from mne.io import read_info
 
-import matplotlib
-matplotlib.use('Agg')  # for testing don't use X server
-
 fname_raw = op.join(op.dirname(__file__), '..', 'io', 'tests', 'data',
                     'test_raw.fif')
 subjects_dir = op.join(testing.data_path(download=False), 'subjects')
@@ -99,8 +97,8 @@ def test_make_sphere_model():
     pytest.raises(ValueError, make_sphere_model, 'auto', 'auto', None)
     pytest.raises(ValueError, make_sphere_model, 'auto', 'auto', info,
                   relative_radii=(), sigmas=())
-    pytest.raises(ValueError, make_sphere_model, 'auto', 'auto', info,
-                  relative_radii=(1,))  # wrong number of radii
+    with pytest.raises(ValueError, match='relative_radii.*must match.*sigmas'):
+        make_sphere_model('auto', 'auto', info, relative_radii=(1,))
     # here we just make sure it works -- the functionality is actually
     # tested more extensively e.g. in the forward and dipole code
     with catch_logging() as log:
@@ -118,15 +116,14 @@ def test_make_sphere_model():
     bem = make_sphere_model('auto', None, info)
     assert 'no layers' in repr(bem)
     assert 'Sphere ' in repr(bem)
-    pytest.raises(ValueError, make_sphere_model, sigmas=(0.33,),
-                  relative_radii=(1.0,))
+    with pytest.raises(ValueError, match='at least 2 sigmas.*head_radius'):
+        make_sphere_model(sigmas=(0.33,), relative_radii=(1.0,))
 
 
 @testing.requires_testing_data
-def test_bem_model():
+def test_make_bem_model(tmpdir):
     """Test BEM model creation from Python with I/O."""
-    tempdir = _TempDir()
-    fname_temp = op.join(tempdir, 'temp-bem.fif')
+    fname_temp = tmpdir.join('temp-bem.fif')
     for kwargs, fname in zip((dict(), dict(conductivity=[0.3])),
                              [fname_bem_3, fname_bem_1]):
         model = make_bem_model('sample', ico=2, subjects_dir=subjects_dir,
@@ -137,8 +134,31 @@ def test_bem_model():
         model_read = read_bem_surfaces(fname_temp)
         _compare_bem_surfaces(model, model_c)
         _compare_bem_surfaces(model_read, model_c)
-    pytest.raises(ValueError, make_bem_model, 'sample',  # bad conductivity
-                  conductivity=[0.3, 0.006], subjects_dir=subjects_dir)
+    # bad conductivity
+    with pytest.raises(ValueError, match='conductivity must be'):
+        make_bem_model('sample', 4, [0.3, 0.006], subjects_dir=subjects_dir)
+
+
+@testing.requires_testing_data
+def test_bem_model_topology(tmpdir):
+    """Test BEM model topological checks."""
+    # bad topology (not enough neighboring tris)
+    makedirs(tmpdir.join('foo', 'bem'))
+    for fname in ('inner_skull', 'outer_skull', 'outer_skin'):
+        fname += '.surf'
+        copy(op.join(subjects_dir, 'sample', 'bem', fname),
+             str(tmpdir.join('foo', 'bem', fname)))
+    outer_fname = tmpdir.join('foo', 'bem', 'outer_skull.surf')
+    rr, tris = read_surface(outer_fname)
+    tris = tris[:-1]
+    write_surface(outer_fname, rr, tris[:-1], overwrite=True)
+    with pytest.raises(RuntimeError, match='Surface outer skull is not compl'):
+        make_bem_model('foo', None, subjects_dir=tmpdir)
+    # Now get past this error to reach gh-6127 (not enough neighbor tris)
+    rr_bad = np.concatenate([rr, np.mean(rr, axis=0, keepdims=True)], axis=0)
+    write_surface(outer_fname, rr_bad, tris, overwrite=True)
+    with pytest.raises(RuntimeError, match='Surface outer skull.*triangles'):
+        make_bem_model('foo', None, subjects_dir=tmpdir)
 
 
 @pytest.mark.slowtest
@@ -343,7 +363,6 @@ def test_fit_sphere_to_headshape():
 @testing.requires_testing_data
 def test_make_flash_bem():
     """Test computing bem from flash images."""
-    import matplotlib.pyplot as plt
     tmp = _TempDir()
     bemdir = op.join(subjects_dir, 'sample', 'bem')
     flash_path = op.join(subjects_dir, 'sample', 'mri', 'flash')
