@@ -9,8 +9,7 @@ from numpy.testing import assert_allclose
 from scipy.interpolate import interp1d
 import pytest
 
-from mne import (pick_types, Dipole, make_sphere_model, make_forward_dipole,
-                 pick_info)
+from mne import pick_types, pick_info
 from mne.io import (read_raw_fif, read_raw_artemis123, read_raw_ctf, read_info,
                     RawArray)
 from mne.io.constants import FIFF
@@ -19,10 +18,9 @@ from mne.chpi import (_calculate_chpi_positions, _calculate_chpi_coil_locs,
                       read_head_pos, write_head_pos, filter_chpi,
                       _get_hpi_info, _get_hpi_initial_fit)
 from mne.transforms import rot_to_quat, _angle_between_quats
-from mne.simulation import simulate_raw
-from mne.utils import run_tests_if_main, _TempDir, catch_logging
+from mne.simulation import add_chpi
+from mne.utils import run_tests_if_main, catch_logging, assert_meg_snr
 from mne.datasets import testing
-from mne.tests.common import assert_meg_snr
 
 base_dir = op.join(op.dirname(__file__), '..', 'io', 'tests', 'data')
 test_fif_fname = op.join(base_dir, 'test_raw.fif')
@@ -87,10 +85,9 @@ def test_chpi_adjust():
 
 
 @testing.requires_testing_data
-def test_read_write_head_pos():
+def test_read_write_head_pos(tmpdir):
     """Test reading and writing head position quaternion parameters."""
-    tempdir = _TempDir()
-    temp_name = op.join(tempdir, 'temp.pos')
+    temp_name = op.join(str(tmpdir), 'temp.pos')
     # This isn't a 100% valid quat matrix but it should be okay for tests
     head_pos_rand = np.random.RandomState(0).randn(20, 10)
     # This one is valid
@@ -108,10 +105,9 @@ def test_read_write_head_pos():
 
 
 @testing.requires_testing_data
-def test_hpi_info():
+def test_hpi_info(tmpdir):
     """Test getting HPI info."""
-    tempdir = _TempDir()
-    temp_name = op.join(tempdir, 'temp_raw.fif')
+    temp_name = op.join(str(tmpdir), 'temp_raw.fif')
     for fname in (chpi_fif_fname, sss_fif_fname):
         raw = read_raw_fif(fname, allow_maxshield='yes').crop(0, 0.1)
         assert len(raw.info['hpi_subsystem']) > 0
@@ -283,7 +279,7 @@ def test_simulate_calculate_chpi_positions():
 
     info['hpi_subsystem'] = hpi_subsystem
     for l, freq in enumerate(coil_freq):
-            info['hpi_meas'][0]['hpi_coils'][l]['coil_freq'] = freq
+        info['hpi_meas'][0]['hpi_coils'][l]['coil_freq'] = freq
     picks = pick_types(info, meg=True, stim=True, eeg=False, exclude=[])
     info['sfreq'] = 100.  # this will speed it up a lot
     info = pick_info(info, picks)
@@ -298,14 +294,15 @@ def test_simulate_calculate_chpi_positions():
     ez = np.array([0, 0, 1])  # Unit vector in z-direction of head coordinates
 
     # Define some constants
-    duration = 30  # Time / s
+    duration = 10  # Time / s
 
     # Quotient of head position sampling frequency
     # and raw sampling frequency
-    head_pos_sfreq_quotient = 0.1
+    head_pos_sfreq_quotient = 0.01
 
     # Round number of head positions to the next integer
-    S = int(duration / (info['sfreq'] * head_pos_sfreq_quotient))
+    S = int(duration * info['sfreq'] * head_pos_sfreq_quotient)
+    assert S == 10
     dz = 0.001  # Shift in z-direction is 0.1mm for each step
 
     dev_head_pos = np.zeros((S, 10))
@@ -321,21 +318,7 @@ def test_simulate_calculate_chpi_positions():
     # Round number of samples to the next integer
     raw_data = np.zeros((len(picks), int(duration * info['sfreq'] + 0.5)))
     raw = RawArray(raw_data, info)
-
-    dip = Dipole(np.array([0.0, 0.1, 0.2]),
-                 np.array([[0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0]]),
-                 np.array([1e-9, 1e-9, 1e-9]),
-                 np.array([[1.0, 0.0, 0.0], [0.0, 1.0, 0.0], [0.0, 0.0, 1.0]]),
-                 np.array([1.0, 1.0, 1.0]), 'dip')
-    sphere = make_sphere_model('auto', 'auto', info=info,
-                               relative_radii=(1.0, 0.9), sigmas=(0.33, 0.3))
-    fwd, stc = make_forward_dipole(dip, sphere, info)
-    stc.resample(info['sfreq'])
-    raw = simulate_raw(raw, stc, None, fwd['src'], sphere, cov=None,
-                       blink=False, ecg=False, chpi=True,
-                       head_pos=dev_head_pos, mindist=1.0, interp='zero',
-                       verbose=None, use_cps=True)
-
+    add_chpi(raw, dev_head_pos)
     quats = _calculate_chpi_positions(
         raw, t_step_min=raw.info['sfreq'] * head_pos_sfreq_quotient,
         t_step_max=raw.info['sfreq'] * head_pos_sfreq_quotient, t_window=1.0)
@@ -370,7 +353,7 @@ def test_calculate_chpi_coil_locs():
     assert_allclose(cHPI_digs[2][0]['gof'], 0.9980471794552791, atol=1e-3)
     assert_allclose(cHPI_digs[2][0]['r'],
                     [-0.0157762, 0.06655744, 0.00545172], atol=1e-3)
-    with pytest.raises(ValueError, match='too_close must be'):
+    with pytest.raises(ValueError, match='too_close'):
         _calculate_chpi_coil_locs(raw, too_close='foo')
 
 
