@@ -1,12 +1,13 @@
-# Author: Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
+# Author: Alexandre Gramfort <alexandre.gramfort@inria.fr>
 #         Denis Engemann <denis.engemann@gmail.com>
 #         Andrew Dykstra <andrew.r.dykstra@gmail.com>
 #         Mads Jensen <mje.mads@gmail.com>
 #
 # License: BSD (3-clause)
 
-import os.path as op
 from copy import deepcopy
+import os.path as op
+import pickle
 
 import numpy as np
 from scipy import fftpack
@@ -15,13 +16,13 @@ from numpy.testing import (assert_array_almost_equal, assert_equal,
 import pytest
 
 from mne import (equalize_channels, pick_types, read_evokeds, write_evokeds,
-                 grand_average, combine_evoked, create_info, read_events,
+                 combine_evoked, create_info, read_events,
                  Epochs, EpochsArray)
 from mne.evoked import _get_peak, Evoked, EvokedArray
 from mne.io import read_raw_fif
+from mne.io.constants import FIFF
 from mne.utils import (_TempDir, requires_pandas, requires_version,
-                       run_tests_if_main)
-from mne.externals.six.moves import cPickle as pickle
+                       run_tests_if_main, grand_average)
 
 base_dir = op.join(op.dirname(__file__), '..', 'io', 'tests', 'data')
 fname = op.join(base_dir, 'test-ave.fif')
@@ -33,22 +34,32 @@ event_name = op.join(base_dir, 'test-eve.fif')
 def test_decim():
     """Test evoked decimation."""
     rng = np.random.RandomState(0)
-    n_epochs, n_channels, n_times = 5, 10, 20
+    n_channels, n_times = 10, 20
     dec_1, dec_2 = 2, 3
     decim = dec_1 * dec_2
-    sfreq = 1000.
+    sfreq = 10.
     sfreq_new = sfreq / decim
-    data = rng.randn(n_epochs, n_channels, n_times)
-    events = np.array([np.arange(n_epochs), [0] * n_epochs, [1] * n_epochs]).T
+    data = rng.randn(n_channels, n_times)
     info = create_info(n_channels, sfreq, 'eeg')
     info['lowpass'] = sfreq_new / float(decim)
-    epochs = EpochsArray(data, info, events)
-    data_epochs = epochs.copy().decimate(decim).get_data()
-    data_epochs_2 = epochs.copy().decimate(decim, offset=1).get_data()
-    data_epochs_3 = epochs.decimate(dec_1).decimate(dec_2).get_data()
-    assert_array_equal(data_epochs, data[:, :, ::decim])
-    assert_array_equal(data_epochs_2, data[:, :, 1::decim])
-    assert_array_equal(data_epochs, data_epochs_3)
+    evoked = EvokedArray(data, info, tmin=-1)
+    evoked_dec = evoked.copy().decimate(decim)
+    evoked_dec_2 = evoked.copy().decimate(decim, offset=1)
+    evoked_dec_3 = evoked.decimate(dec_1).decimate(dec_2)
+    assert_array_equal(evoked_dec.data, data[:, ::decim])
+    assert_array_equal(evoked_dec_2.data, data[:, 1::decim])
+    assert_array_equal(evoked_dec.data, evoked_dec_3.data)
+
+    # Check proper updating of various fields
+    assert evoked_dec.first == -1
+    assert evoked_dec.last == 2
+    assert_array_equal(evoked_dec.times, [-1, -0.4, 0.2, 0.8])
+    assert evoked_dec_2.first == -1
+    assert evoked_dec_2.last == 2
+    assert_array_equal(evoked_dec_2.times, [-0.9, -0.3, 0.3, 0.9])
+    assert evoked_dec_3.first == -1
+    assert evoked_dec_3.last == 2
+    assert_array_equal(evoked_dec_3.times, [-1, -0.4, 0.2, 0.8])
 
     # Now let's do it with some real data
     raw = read_raw_fif(raw_fname)
@@ -97,22 +108,51 @@ def test_hash_evoked():
     """Test evoked hashing."""
     ave = read_evokeds(fname, 0)
     ave_2 = read_evokeds(fname, 0)
-    assert_equal(hash(ave), hash(ave_2))
+    assert hash(ave) == hash(ave_2)
+    assert ave == ave_2
     # do NOT use assert_equal here, failing output is terrible
-    assert (pickle.dumps(ave) == pickle.dumps(ave_2))
+    assert pickle.dumps(ave) == pickle.dumps(ave_2)
 
     ave_2.data[0, 0] -= 1
     assert hash(ave) != hash(ave_2)
 
 
-@pytest.mark.slowtest
-def test_io_evoked():
-    """Test IO for evoked data (fif + gz) with integer and str args."""
-    tempdir = _TempDir()
-    ave = read_evokeds(fname, 0)
+def _aspect_kinds():
+    """Yield evoked aspect kinds."""
+    kinds = list()
+    for key in FIFF:
+        if not key.startswith('FIFFV_ASPECT_'):
+            continue
+        kinds.append(getattr(FIFF, str(key)))
+    return kinds
 
-    write_evokeds(op.join(tempdir, 'evoked-ave.fif'), ave)
-    ave2 = read_evokeds(op.join(tempdir, 'evoked-ave.fif'))[0]
+
+@pytest.mark.parametrize('aspect_kind', _aspect_kinds())
+def test_evoked_aspects(aspect_kind, tmpdir):
+    """Test handling of evoked aspects."""
+    # gh-6359
+    ave = read_evokeds(fname, 0)
+    ave._aspect_kind = aspect_kind
+    assert 'Evoked' in repr(ave)
+    # for completeness let's try a round-trip
+    temp_fname = op.join(str(tmpdir), 'test-ave.fif')
+    ave.save(temp_fname)
+    ave_2 = read_evokeds(temp_fname, condition=0)
+    assert_allclose(ave.data, ave_2.data)
+    assert ave.kind == ave_2.kind
+
+
+@pytest.mark.slowtest
+def test_io_evoked(tmpdir):
+    """Test IO for evoked data (fif + gz) with integer and str args."""
+    ave = read_evokeds(fname, 0)
+    ave_double = ave.copy()
+    ave_double.comment = ave.comment + ' doubled nave'
+    ave_double.nave = ave.nave * 2
+
+    write_evokeds(tmpdir.join('evoked-ave.fif'), [ave, ave_double])
+    ave2, ave_double = read_evokeds(op.join(tmpdir, 'evoked-ave.fif'))
+    assert ave2.nave * 2 == ave_double.nave
 
     # This not being assert_array_equal due to windows rounding
     assert (np.allclose(ave.data, ave2.data, atol=1e-16, rtol=1e-3))
@@ -140,8 +180,8 @@ def test_io_evoked():
     aves1 = read_evokeds(fname)[1::2]
     aves2 = read_evokeds(fname, [1, 3])
     aves3 = read_evokeds(fname, ['Right Auditory', 'Right visual'])
-    write_evokeds(op.join(tempdir, 'evoked-ave.fif'), aves1)
-    aves4 = read_evokeds(op.join(tempdir, 'evoked-ave.fif'))
+    write_evokeds(tmpdir.join('evoked-ave.fif'), aves1)
+    aves4 = read_evokeds(tmpdir.join('evoked-ave.fif'))
     for aves in [aves2, aves3, aves4]:
         for [av1, av2] in zip(aves1, aves):
             assert_array_almost_equal(av1.data, av2.data)
@@ -154,7 +194,7 @@ def test_io_evoked():
             assert_equal(av1.comment, av2.comment)
 
     # test warnings on bad filenames
-    fname2 = op.join(tempdir, 'test-bad-name.fif')
+    fname2 = tmpdir.join('test-bad-name.fif')
     with pytest.warns(RuntimeWarning, match='-ave.fif'):
         write_evokeds(fname2, ave)
     with pytest.warns(RuntimeWarning, match='-ave.fif'):
@@ -164,7 +204,7 @@ def test_io_evoked():
     pytest.raises(TypeError, Evoked, fname)
 
     # MaxShield
-    fname_ms = op.join(tempdir, 'test-ave.fif')
+    fname_ms = tmpdir.join('test-ave.fif')
     assert (ave.info['maxshield'] is False)
     ave.info['maxshield'] = True
     ave.save(fname_ms)
@@ -219,8 +259,10 @@ def test_evoked_resample():
     tempdir = _TempDir()
     # upsample, write it out, read it in
     ave = read_evokeds(fname, 0)
+    orig_lp = ave.info['lowpass']
     sfreq_normal = ave.info['sfreq']
     ave.resample(2 * sfreq_normal, npad=100)
+    assert ave.info['lowpass'] == orig_lp
     write_evokeds(op.join(tempdir, 'evoked-ave.fif'), ave)
     ave_up = read_evokeds(op.join(tempdir, 'evoked-ave.fif'), 0)
 
@@ -230,6 +272,7 @@ def test_evoked_resample():
     # and compare the original to the downsampled upsampled version
     ave_new = read_evokeds(op.join(tempdir, 'evoked-ave.fif'), 0)
     ave_new.resample(sfreq_normal, npad=100)
+    assert ave.info['lowpass'] == orig_lp
 
     assert_array_almost_equal(ave_normal.data, ave_new.data, 2)
     assert_array_almost_equal(ave_normal.times, ave_new.times)
@@ -244,6 +287,10 @@ def test_evoked_resample():
     assert (len(ave_up.times) == 2 * len(ave_normal.times))
     assert (ave_up.data.shape[1] == 2 * ave_normal.data.shape[1])
 
+    ave_new.resample(50)
+    assert ave_new.info['sfreq'] == 50.
+    assert ave_new.info['lowpass'] == 25.
+
 
 def test_evoked_filter():
     """Test filtering evoked data."""
@@ -279,6 +326,13 @@ def test_to_data_frame():
     assert_array_equal(df.values[:, 1], ave.data[0] * 1e13)
     assert_array_equal(df.values[:, 3], ave.data[2] * 1e15)
 
+    df = ave.to_data_frame(long_format=True)
+    assert(len(df) == ave.data.size)
+    assert("time" in df.columns)
+    assert("channel" in df.columns)
+    assert("ch_type" in df.columns)
+    assert("observation" in df.columns)
+
 
 def test_evoked_proj():
     """Test SSP proj operations."""
@@ -427,24 +481,30 @@ def test_arithmetic():
     # data should be added according to their `nave` weights
     # nave = ev1.nave + ev2.nave
     ev = combine_evoked([ev1, ev2], weights='nave')
-    assert_equal(ev.nave, ev1.nave + ev2.nave)
+    assert_allclose(ev.nave, ev1.nave + ev2.nave)
     assert_allclose(ev.data, 1. / 3. * np.ones_like(ev.data))
 
     # with same trial counts, a bunch of things should be equivalent
-    for weights in ('nave', 'equal', [0.5, 0.5]):
+    for weights in ('nave', [0.5, 0.5]):
         ev = combine_evoked([ev1, ev1], weights=weights)
         assert_allclose(ev.data, ev1.data)
-        assert_equal(ev.nave, 2 * ev1.nave)
+        assert_allclose(ev.nave, 2 * ev1.nave)
         ev = combine_evoked([ev1, -ev1], weights=weights)
         assert_allclose(ev.data, 0., atol=1e-20)
-        assert_equal(ev.nave, 2 * ev1.nave)
+        assert_allclose(ev.nave, 2 * ev1.nave)
+    # adding evoked to itself
+    ev = combine_evoked([ev1, ev1], weights='equal')
+    assert_allclose(ev.data, 2 * ev1.data)
+    assert_allclose(ev.nave, ev1.nave / 2)
+    # subtracting evoked from itself
     ev = combine_evoked([ev1, -ev1], weights='equal')
     assert_allclose(ev.data, 0., atol=1e-20)
-    assert_equal(ev.nave, 2 * ev1.nave)
+    assert_allclose(ev.nave, ev1.nave / 2)
+    # subtracting different evokeds
     ev = combine_evoked([ev1, -ev2], weights='equal')
-    expected = int(round(1. / (0.25 / ev1.nave + 0.25 / ev2.nave)))
-    assert_equal(expected, 27)  # this is reasonable
-    assert_equal(ev.nave, expected)
+    assert_allclose(ev.data, 2., atol=1e-20)
+    expected_nave = 1. / (1. / ev1.nave + 1. / ev2.nave)
+    assert_allclose(ev.nave, expected_nave)
 
     # default comment behavior if evoked.comment is None
     old_comment1 = ev1.comment
@@ -463,7 +523,7 @@ def test_arithmetic():
 
     # combine_evoked([ev1, ev2], weights=[1, 0]) should yield the same as ev1
     ev = combine_evoked([ev1, ev2], weights=[1, 0])
-    assert_equal(ev.nave, ev1.nave)
+    assert_allclose(ev.nave, ev1.nave)
     assert_allclose(ev.data, ev1.data)
 
     # simple subtraction (like in oddball)
@@ -484,11 +544,13 @@ def test_arithmetic():
     assert_equal(ch_names, gave.ch_names)
     assert_equal(gave.nave, 2)
     pytest.raises(TypeError, grand_average, [1, evoked1])
+    gave = grand_average([ev1, ev1, ev2])  # (1 + 1 + -1) / 3  =  1/3
+    assert_allclose(gave.data, np.full_like(gave.data, 1. / 3.))
 
     # test channel (re)ordering
     evoked1, evoked2 = read_evokeds(fname, condition=[0, 1], proj=True)
     data2 = evoked2.data  # assumes everything is ordered to the first evoked
-    data = (evoked1.data + evoked2.data) / 2
+    data = (evoked1.data + evoked2.data) / 2.
     evoked2.reorder_channels(evoked2.ch_names[::-1])
     assert not np.allclose(data2, evoked2.data)
     with pytest.warns(RuntimeWarning, match='reordering'):
@@ -535,8 +597,10 @@ def test_array_epochs():
     assert_equal(evoked1.nave, evoked3.nave)
 
     # test kind check
-    pytest.raises(TypeError, EvokedArray, data1, info, tmin=0, kind=1)
-    pytest.raises(ValueError, EvokedArray, data1, info, kind='mean')
+    with pytest.raises(ValueError, match='Invalid value'):
+        EvokedArray(data1, info, tmin=0, kind=1)
+    with pytest.raises(ValueError, match='Invalid value'):
+        EvokedArray(data1, info, kind='mean')
 
     # test match between channels info and data
     ch_names = ['EEG %03d' % (i + 1) for i in range(19)]
@@ -545,11 +609,17 @@ def test_array_epochs():
     pytest.raises(ValueError, EvokedArray, data1, info, tmin=-0.01)
 
 
-def test_time_as_index():
-    """Test time as index."""
-    evoked = read_evokeds(fname, condition=0).crop(-.1, .1)
+def test_time_as_index_and_crop():
+    """Test time as index and cropping."""
+    tmin, tmax = -0.1, 0.1
+    evoked = read_evokeds(fname, condition=0).crop(tmin, tmax)
+    delta = 1. / evoked.info['sfreq']
+    atol = 0.5 * delta
+    assert_allclose(evoked.times[[0, -1]], [tmin, tmax], atol=atol)
     assert_array_equal(evoked.time_as_index([-.1, .1], use_rounding=True),
                        [0, len(evoked.times) - 1])
+    evoked.crop(tmin, tmax, include_tmax=False)
+    assert_allclose(evoked.times[[0, -1]], [tmin, tmax - delta], atol=atol)
 
 
 def test_add_channels():
@@ -599,4 +669,30 @@ def test_evoked_baseline():
     assert_allclose(evoked.data, np.zeros_like(evoked.data))
 
 
+def test_hilbert():
+    """Test hilbert on raw, epochs, and evoked."""
+    raw = read_raw_fif(raw_fname).load_data()
+    raw.del_proj()
+    raw.pick_channels(raw.ch_names[:2])
+    events = read_events(event_name)
+    epochs = Epochs(raw, events)
+    with pytest.raises(RuntimeError, match='requires epochs data to be load'):
+        epochs.apply_hilbert()
+    epochs.load_data()
+    evoked = epochs.average()
+    raw_hilb = raw.apply_hilbert()
+    epochs_hilb = epochs.apply_hilbert()
+    evoked_hilb = evoked.copy().apply_hilbert()
+    evoked_hilb_2_data = epochs_hilb.get_data().mean(0)
+    assert_allclose(evoked_hilb.data, evoked_hilb_2_data)
+    # This one is only approximate because of edge artifacts
+    evoked_hilb_3 = Epochs(raw_hilb, events).average()
+    corr = np.corrcoef(np.abs(evoked_hilb_3.data.ravel()),
+                       np.abs(evoked_hilb.data.ravel()))[0, 1]
+    assert 0.96 < corr < 0.98
+    # envelope=True mode
+    evoked_hilb_env = evoked.apply_hilbert(envelope=True)
+    assert_allclose(evoked_hilb_env.data, np.abs(evoked_hilb.data))
+
+
 run_tests_if_main()
