@@ -1,11 +1,12 @@
 import numpy as np
 import os.path as op
 from numpy.testing import assert_array_almost_equal, assert_allclose
+from scipy.signal import welch
 import pytest
 
 from mne import pick_types, Epochs, read_events
 from mne.io import RawArray, read_raw_fif
-from mne.utils import run_tests_if_main
+from mne.utils import run_tests_if_main, requires_version
 from mne.time_frequency import psd_welch, psd_multitaper, psd_array_welch
 
 base_dir = op.join(op.dirname(__file__), '..', '..', 'io', 'tests', 'data')
@@ -159,6 +160,69 @@ def test_psd():
         assert (psds_ev.shape == (len(kws['picks']), len(freqs)))
 
 
+@requires_version('scipy', '1.2.0')
+@pytest.mark.parametrize('kind', ('raw', 'epochs', 'evoked'))
+def test_psd_welch_average_kwarg(kind):
+    """Test `average` kwarg of psd_welch()."""
+    raw = read_raw_fif(raw_fname)
+    picks_psd = [0, 1]
+
+    # Populate raw with sinusoids
+    rng = np.random.RandomState(40)
+    data = 0.1 * rng.randn(len(raw.ch_names), raw.n_times)
+    freqs_sig = [8., 50.]
+    for ix, freq in zip(picks_psd, freqs_sig):
+        data[ix, :] += 2 * np.sin(np.pi * 2. * freq * raw.times)
+    first_samp = raw._first_samps[0]
+    raw = RawArray(data, raw.info)
+
+    tmin, tmax = -0.5, 0.5
+    fmin, fmax = 0, np.inf
+    n_fft = 256
+    n_per_seg = 128
+    n_overlap = 0
+
+    event_id = 2
+    events = read_events(event_fname)
+    events[:, 0] -= first_samp
+
+    kws = dict(fmin=fmin, fmax=fmax, tmin=tmin, tmax=tmax, n_fft=n_fft,
+               n_per_seg=n_per_seg, n_overlap=n_overlap, picks=picks_psd)
+
+    if kind == 'raw':
+        inst = raw
+    elif kind == 'epochs':
+        inst = Epochs(raw, events[:10], event_id, tmin, tmax, picks=picks_psd,
+                      proj=False, preload=True, baseline=None)
+    elif kind == 'evoked':
+        inst = Epochs(raw, events[:10], event_id, tmin, tmax, picks=picks_psd,
+                      proj=False, preload=True, baseline=None).average()
+    else:
+        raise ValueError('Unknown parametrization passed to test, check test '
+                         'for typos.')
+
+    psds_mean, freqs_mean = psd_welch(inst=inst, average='mean', **kws)
+    psds_median, freqs_median = psd_welch(inst=inst, average='median', **kws)
+    psds_unagg, freqs_unagg = psd_welch(inst=inst, average=None, **kws)
+
+    # Frequencies should be equal across all "average" types, as we feed in
+    # the exact same data.
+    assert_allclose(freqs_mean, freqs_median)
+    assert_allclose(freqs_mean, freqs_unagg)
+
+    # For `average=None`, the last dimension contains the un-aggregated
+    # segments.
+    assert psds_mean.shape == psds_median.shape
+    assert psds_mean.shape == psds_unagg.shape[:-1]
+    assert_allclose(psds_mean, psds_unagg.mean(axis=-1))
+
+    # SciPy's welch() function corrects the median PSD for its bias relative to
+    # the mean.
+    from scipy.signal.spectral import _median_bias
+    median_bias = _median_bias(psds_unagg.shape[-1])
+    assert_allclose(psds_median, np.median(psds_unagg, axis=-1) / median_bias)
+
+
 @pytest.mark.slowtest
 def test_compares_psd():
     """Test PSD estimation on raw for plt.psd and scipy.signal.welch."""
@@ -182,10 +246,10 @@ def test_compares_psd():
     # Compute psds with plt.psd
     start, stop = raw.time_as_index([tmin, tmax])
     data, times = raw[picks, start:(stop + 1)]
-    from matplotlib.pyplot import psd
-    out = [psd(d, Fs=raw.info['sfreq'], NFFT=n_fft) for d in data]
-    freqs_mpl = out[0][1]
-    psds_mpl = np.array([o[0] for o in out])
+    out = [welch(d, fs=raw.info['sfreq'], nperseg=n_fft, noverlap=0)
+           for d in data]
+    freqs_mpl = out[0][0]
+    psds_mpl = np.array([o[1] for o in out])
 
     mask = (freqs_mpl >= fmin) & (freqs_mpl <= fmax)
     freqs_mpl = freqs_mpl[mask]
@@ -203,4 +267,5 @@ def test_compares_psd():
     assert (np.sum(psds_welch < 0) == 0)
     assert (np.sum(psds_mpl < 0) == 0)
 
+
 run_tests_if_main()
