@@ -1,36 +1,37 @@
 """Functions to plot ICA specific data (besides topographies)."""
-from __future__ import print_function
 
 # Authors: Denis Engemann <denis.engemann@gmail.com>
-#          Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
+#          Alexandre Gramfort <alexandre.gramfort@inria.fr>
 #          Teon Brooks <teon.brooks@gmail.com>
+#          Daniel McCloy <dan.mccloy@gmail.com>
 #
 # License: Simplified BSD
 
 from functools import partial
-from numbers import Integral
 
 import numpy as np
 
 from .utils import (tight_layout, _prepare_trellis, _select_bads,
-                    _layout_figure, _plot_raw_onscroll, _mouse_click,
-                    _helper_raw_resize, _plot_raw_onkey, plt_show)
+                    _plot_raw_onscroll, _mouse_click,
+                    _plot_raw_onkey, plt_show, _convert_psds)
 from .topomap import (_prepare_topo_plot, plot_topomap, _hide_frame,
                       _plot_ica_topomap)
-from .raw import _prepare_mne_browse_raw, _plot_raw_traces, _convert_psds
+from .raw import _prepare_mne_browse_raw, _plot_raw_traces
 from .epochs import _prepare_mne_browse_epochs, plot_epochs_image
 from .evoked import _butterfly_on_button_press, _butterfly_onpick
-from ..utils import warn, _validate_type
+from ..utils import warn, _validate_type, fill_doc
 from ..defaults import _handle_default
 from ..io.meas_info import create_info
-from ..io.pick import pick_types
-from ..externals.six import string_types
+from ..io.pick import (pick_types, _picks_to_idx, _get_channel_types,
+                       _DATA_CH_TYPES_ORDER_DEFAULT)
 from ..time_frequency.psd import psd_multitaper
+from ..utils import _reject_data_segments
 
 
-def plot_ica_sources(ica, inst, picks=None, exclude=None, start=None,
+@fill_doc
+def plot_ica_sources(ica, inst, picks=None, exclude='deprecated', start=None,
                      stop=None, title=None, show=True, block=False,
-                     show_first_samp=False):
+                     show_first_samp=False, show_scrollbars=True):
     """Plot estimated latent sources given the unmixing matrix.
 
     Typical usecases:
@@ -46,19 +47,18 @@ def plot_ica_sources(ica, inst, picks=None, exclude=None, start=None,
         The ICA solution.
     inst : instance of mne.io.Raw, mne.Epochs, mne.Evoked
         The object to plot the sources from.
-    picks : int | array_like of int | None.
-        The components to be displayed. If None, plot will show the
-        sources in the order as fitted.
-    exclude : array_like of int
-        The components marked for exclusion. If None (default), ICA.exclude
-        will be used.
+    %(picks_base)s all sources in the order as fitted.
+    exclude : 'deprecated'
+        The ``exclude`` parameter is deprecated and will be removed in version
+        0.20; specify excluded components using the ``ICA.exclude`` attribute
+        instead.
     start : int
         X-axis start index. If None, from the beginning.
     stop : int
         X-axis stop index. If None, next 20 are shown, in case of evoked to the
         end.
     title : str | None
-        The figure title. If None a default is provided.
+        The window title. If None a default is provided.
     show : bool
         Show figure if True.
     block : bool
@@ -67,10 +67,11 @@ def plot_ica_sources(ica, inst, picks=None, exclude=None, start=None,
         plotter. For evoked, this parameter has no effect. Defaults to False.
     show_first_samp : bool
         If True, show time axis relative to the ``raw.first_samp``.
+    %(show_scrollbars)s
 
     Returns
     -------
-    fig : instance of pyplot.Figure
+    fig : instance of Figure
         The figure.
 
     Notes
@@ -85,18 +86,24 @@ def plot_ica_sources(ica, inst, picks=None, exclude=None, start=None,
     from ..evoked import Evoked
     from ..epochs import BaseEpochs
 
-    if exclude is None:
-        exclude = ica.exclude
-    elif len(ica.exclude) > 0:
-        exclude = np.union1d(ica.exclude, exclude)
+    if exclude != 'deprecated':
+        warn('The "exclude" parameter is deprecated and will be removed in '
+             'version 0.20; specify excluded components using the ICA.exclude '
+             'attribute instead. Provided value of {} will be ignored; falling'
+             ' back to ICA.exclude'.format(exclude), DeprecationWarning)
+    exclude = ica.exclude
+    picks = _picks_to_idx(ica.n_components_, picks, 'all')
+
     if isinstance(inst, BaseRaw):
         fig = _plot_sources_raw(ica, inst, picks, exclude, start=start,
                                 stop=stop, show=show, title=title,
-                                block=block, show_first_samp=show_first_samp)
+                                block=block, show_first_samp=show_first_samp,
+                                show_scrollbars=show_scrollbars)
     elif isinstance(inst, BaseEpochs):
         fig = _plot_sources_epochs(ica, inst, picks, exclude, start=start,
                                    stop=stop, show=show, title=title,
-                                   block=block)
+                                   block=block,
+                                   show_scrollbars=show_scrollbars)
     elif isinstance(inst, Evoked):
         if start is not None or stop is not None:
             inst = inst.copy().crop(start, stop)
@@ -130,8 +137,12 @@ def _create_properties_layout(figsize=None):
 def _plot_ica_properties(pick, ica, inst, psds_mean, freqs, n_trials,
                          epoch_var, plot_lowpass_edge, epochs_src,
                          set_title_and_labels, plot_std, psd_ylabel,
-                         spectrum_std, topomap_args, image_args, fig, axes):
+                         spectrum_std, topomap_args, image_args, fig, axes,
+                         kind, dropped_indices):
     """Plot ICA properties (helper)."""
+    from mpl_toolkits.axes_grid1.axes_divider import make_axes_locatable
+    from scipy.stats import gaussian_kde
+
     topo_ax, image_ax, erp_ax, spec_ax, var_ax = axes
 
     # plotting
@@ -140,8 +151,19 @@ def _plot_ica_properties(pick, ica, inst, psds_mean, freqs, n_trials,
     _plot_ica_topomap(ica, pick, show=False, axes=topo_ax, **topomap_args)
 
     # image and erp
+    # we create a new epoch with dropped rows
+    epoch_data = epochs_src.get_data()
+    epoch_data = np.insert(arr=epoch_data,
+                           obj=(dropped_indices -
+                                np.arange(len(dropped_indices))).astype(int),
+                           values=0.0,
+                           axis=0)
+    from ..epochs import EpochsArray
+    epochs_src = EpochsArray(epoch_data, epochs_src.info, verbose=0)
+
     plot_epochs_image(epochs_src, picks=pick, axes=[image_ax, erp_ax],
-                      combine=None, colorbar=False, show=False, **image_args)
+                      combine=None, colorbar=False, show=False,
+                      **image_args)
 
     # spectrum
     spec_ax.plot(freqs, psds_mean, color='k')
@@ -154,24 +176,44 @@ def _plot_ica_properties(pick, ica, inst, psds_mean, freqs, n_trials,
                         color='k', alpha=0.2)
 
     # epoch variance
+    var_ax_divider = make_axes_locatable(var_ax)
+    hist_ax = var_ax_divider.append_axes("right", size="33%", pad="2.5%")
     var_ax.scatter(range(len(epoch_var)), epoch_var, alpha=0.5,
                    facecolor=[0, 0, 0], lw=0)
+    # rejected epochs in red
+    var_ax.scatter(dropped_indices, epoch_var[dropped_indices],
+                   alpha=1., facecolor=[1, 0, 0], lw=0)
+    # compute percentage of dropped epochs
+    var_percent = float(len(dropped_indices)) / float(len(epoch_var)) * 100.
+
+    # histogram & histogram
+    _, counts, _ = hist_ax.hist(epoch_var, orientation="horizontal",
+                                color="k", alpha=.5)
+
+    # kde
+    kde = gaussian_kde(epoch_var)
+    ymin, ymax = hist_ax.get_ylim()
+    x = np.linspace(ymin, ymax, 50)
+    kde_ = kde(x)
+    kde_ /= kde_.max()
+    kde_ *= hist_ax.get_xlim()[-1] * .9
+    hist_ax.plot(kde_, x, color="k")
+    hist_ax.set_ylim(ymin, ymax)
 
     # aesthetics
     # ----------
     topo_ax.set_title(ica._ica_names[pick])
 
-    set_title_and_labels(image_ax, 'Epochs image and ERP/ERF', [], 'Epochs')
+    set_title_and_labels(image_ax, kind + ' image and ERP/ERF', [], kind)
 
     # erp
-    set_title_and_labels(erp_ax, [], 'Time (s)', 'AU\n')
+    set_title_and_labels(erp_ax, [], 'Time (s)', 'AU')
     erp_ax.spines["right"].set_color('k')
     erp_ax.set_xlim(epochs_src.times[[0, -1]])
     # remove half of yticks if more than 5
     yt = erp_ax.get_yticks()
     if len(yt) > 5:
-        yt = yt[::2]
-        erp_ax.yaxis.set_ticks(yt)
+        erp_ax.yaxis.set_ticks(yt[::2])
 
     # remove xticks - erp plot shows xticks for both image and erp plot
     image_ax.xaxis.set_ticks([])
@@ -189,7 +231,12 @@ def _plot_ica_properties(pick, ica, inst, psds_mean, freqs, n_trials,
     image_ax.axhline(0, color='k', linewidth=.5)
 
     # epoch variance
-    set_title_and_labels(var_ax, 'Epochs variance', 'Epoch (index)', 'AU')
+    var_ax_title = 'Dropped segments: %.2f %%' % var_percent
+    set_title_and_labels(var_ax, var_ax_title, kind, 'Variance (AU)')
+
+    hist_ax.set_ylabel("")
+    hist_ax.set_yticks([])
+    set_title_and_labels(hist_ax, None, None, None)
 
     return fig
 
@@ -211,9 +258,10 @@ def _get_psd_label_and_std(this_psd, dB, ica, num_std):
     return psd_ylabel, psds_mean, spectrum_std
 
 
+@fill_doc
 def plot_ica_properties(ica, inst, picks=None, axes=None, dB=True,
                         plot_std=True, topomap_args=None, image_args=None,
-                        psd_args=None, figsize=None, show=True):
+                        psd_args=None, figsize=None, show=True, reject='auto'):
     """Display component properties.
 
     Properties include the topography, epochs image, ERP/ERF, power
@@ -225,10 +273,9 @@ def plot_ica_properties(ica, inst, picks=None, axes=None, dB=True,
         The ICA solution.
     inst: instance of Epochs or Raw
         The data to use in plotting properties.
-    picks : int | array-like of int | None
-        The components to be displayed. If None, plot will show the first
-        five sources. If more than one components were chosen in the picks,
-        each one will be plotted in a separate figure. Defaults to None.
+    %(picks_base)s the first five sources.
+        If more than one components were chosen in the picks,
+        each one will be plotted in a separate figure.
     axes: list of matplotlib axes | None
         List of five matplotlib axes to use in plotting: [topomap_axis,
         image_axis, erp_axis, spectrum_axis, variance_axis]. If None a new
@@ -236,10 +283,15 @@ def plot_ica_properties(ica, inst, picks=None, axes=None, dB=True,
     dB: bool
         Whether to plot spectrum in dB. Defaults to True.
     plot_std: bool | float
-        Whether to plot standard deviation in ERP/ERF and spectrum plots.
-        Defaults to True, which plots one standard deviation above/below.
-        If set to float allows to control how many standard deviations are
-        plotted. For example 2.5 will plot 2.5 standard deviation above/below.
+        Whether to plot standard deviation/confidence intervals in ERP/ERF and
+        spectrum plots.
+        Defaults to True, which plots one standard deviation above/below for
+        the spectrum. If set to float allows to control how many standard
+        deviations are plotted for the spectrum. For example 2.5 will plot 2.5
+        standard deviation above/below.
+        For the ERP/ERF, by default, plot the 95 percent parametric confidence
+        interval is calculated. To change this, use ``ci`` in ``ts_args`` in
+        ``image_args`` (see below).
     topomap_args : dict | None
         Dictionary of arguments to ``plot_topomap``. If None, doesn't pass any
         additional arguments. Defaults to None.
@@ -249,11 +301,17 @@ def plot_ica_properties(ica, inst, picks=None, axes=None, dB=True,
     psd_args : dict | None
         Dictionary of arguments to ``psd_multitaper``. If None, doesn't pass
         any additional arguments. Defaults to None.
-    figsize : array-like of size (2,) | None
+    figsize : array-like, shape (2,) | None
         Allows to control size of the figure. If None, the figure size
         defaults to [7., 6.].
     show : bool
         Show figure if True.
+    reject : 'auto' | dict | None
+        Allows to specify rejection parameters used to drop epochs
+        (or segments if continuous signal is passed as inst).
+        If None, no rejection is applied. The default is 'auto',
+        which applies the rejection parameters used when fitting
+        the ICA object.
 
     Returns
     -------
@@ -267,6 +325,7 @@ def plot_ica_properties(ica, inst, picks=None, axes=None, dB=True,
     from ..io.base import BaseRaw
     from ..epochs import BaseEpochs
     from ..preprocessing import ICA
+    from ..io import RawArray
 
     # input checks and defaults
     # -------------------------
@@ -282,8 +341,8 @@ def plot_ica_properties(ica, inst, picks=None, axes=None, dB=True,
                          'got %s instead' % type(plot_std))
 
     # if no picks given - plot the first 5 components
-    picks = list(range(min(5, ica.n_components_))) if picks is None else picks
-    picks = [picks] if isinstance(picks, Integral) else picks
+    limit = min(5, ica.n_components_) if picks is None else len(ica.ch_names)
+    picks = _picks_to_idx(ica.info, picks, 'all')[:limit]
     if axes is None:
         fig, axes = _create_properties_layout(figsize=figsize)
     else:
@@ -293,10 +352,17 @@ def plot_ica_properties(ica, inst, picks=None, axes=None, dB=True,
         from .utils import _validate_if_list_of_axes
         _validate_if_list_of_axes(axes, obligatory_len=5)
         fig = axes[0].get_figure()
+
     psd_args = dict() if psd_args is None else psd_args
     topomap_args = dict() if topomap_args is None else topomap_args
     image_args = dict() if image_args is None else image_args
     image_args["ts_args"] = dict(truncate_xaxis=False, show_sensors=False)
+    if plot_std:
+        from ..stats.parametric import _parametric_ci
+        image_args["ts_args"]["ci"] = _parametric_ci
+    elif "ts_args" not in image_args or "ci" not in image_args["ts_args"]:
+        image_args["ts_args"]["ci"] = False
+
     for item_name, item in (("psd_args", psd_args),
                             ("topomap_args", topomap_args),
                             ("image_args", image_args)):
@@ -306,14 +372,54 @@ def plot_ica_properties(ica, inst, picks=None, axes=None, dB=True,
 
     # calculations
     # ------------
+
     if isinstance(inst, BaseRaw):
+        # when auto, delegate reject to the ica
+        if reject == 'auto':
+            reject = getattr(ica, 'reject_', None)
+        else:
+            pass
+
+        if reject is None:
+            inst_rejected = inst
+            drop_inds = None
+        else:
+            data = inst.get_data()
+            data, drop_inds = _reject_data_segments(data, ica.reject_,
+                                                    flat=None, decim=None,
+                                                    info=inst.info,
+                                                    tstep=2.0)
+            inst_rejected = RawArray(data, inst.info)
+
         # break up continuous signal into segments
         from ..epochs import _segment_raw
+        inst_rejected = _segment_raw(inst_rejected,
+                                     segment_length=2.,
+                                     verbose=False,
+                                     preload=True)
         inst = _segment_raw(inst, segment_length=2., verbose=False,
                             preload=True)
+        kind = "Segment"
+    else:
+        drop_inds = None
+        inst_rejected = inst
+        kind = "Epochs"
 
-    epochs_src = ica.get_sources(inst)
-    ica_data = np.swapaxes(epochs_src.get_data()[:, picks, :], 0, 1)
+    epochs_src = ica.get_sources(inst_rejected)
+    data = epochs_src.get_data()
+
+    ica_data = np.swapaxes(data[:, picks, :], 0, 1)
+
+    # getting dropped epochs indexes
+    if drop_inds is not None:
+        dropped_indices = [(d[0] // len(inst.times)) + 1
+                           for d in drop_inds]
+    else:
+        dropped_indices = []
+
+    # getting ica sources from inst
+    dropped_src = ica.get_sources(inst).get_data()
+    dropped_src = np.swapaxes(dropped_src[:, picks, :], 0, 1)
 
     # spectrum
     Nyquist = inst.info['sfreq'] / 2.
@@ -347,12 +453,24 @@ def plot_ica_properties(ica, inst, picks=None, axes=None, dB=True,
         if idx > 0:
             fig, axes = _create_properties_layout(figsize=figsize)
 
+        # we reconstruct an epoch_variance with 0 where indexes where dropped
+        epoch_var = np.var(ica_data[idx], axis=1)
+        drop_var = np.var(dropped_src[idx], axis=1)
+        drop_indices_corrected = \
+            (dropped_indices -
+             np.arange(len(dropped_indices))).astype(int)
+        epoch_var = np.insert(arr=epoch_var,
+                              obj=drop_indices_corrected,
+                              values=drop_var[dropped_indices],
+                              axis=0)
+
         # the actual plot
         fig = _plot_ica_properties(
             pick, ica, inst, psds_mean, freqs, ica_data.shape[1],
-            np.var(ica_data[idx], axis=1), plot_lowpass_edge,
+            epoch_var, plot_lowpass_edge,
             epochs_src, set_title_and_labels, plot_std, psd_ylabel,
-            spectrum_std, topomap_args, image_args, fig, axes)
+            spectrum_std, topomap_args, image_args, fig, axes, kind,
+            dropped_indices)
         all_fig.append(fig)
 
     plt_show(show)
@@ -367,10 +485,8 @@ def _plot_ica_sources_evoked(evoked, picks, exclude, title, show, ica,
     ----------
     evoked : instance of mne.Evoked
         The Evoked to be used.
-    picks : int | array_like of int | None.
-        The components to be displayed. If None, plot will show the
-        sources in the order as fitted.
-    exclude : array_like of int
+    %(picks_base)s all sources in the order as fitted.
+    exclude : array-like of int
         The components marked for exclusion. If None (default), ICA.exclude
         will be used.
     title : str
@@ -381,6 +497,8 @@ def _plot_ica_sources_evoked(evoked, picks, exclude, title, show, ica,
         The ICA labels attribute.
     """
     import matplotlib.pyplot as plt
+    from matplotlib import patheffects
+
     if title is None:
         title = 'Reconstructed latent sources, time-locked'
 
@@ -392,8 +510,6 @@ def _plot_ica_sources_evoked(evoked, picks, exclude, title, show, ica,
     # plot unclassified sources and label excluded ones
     lines = list()
     texts = list()
-    if picks is None:
-        picks = np.arange(evoked.data.shape[0])
     picks = np.sort(picks)
     idxs = [picks]
 
@@ -418,11 +534,11 @@ def _plot_ica_sources_evoked(evoked, picks, exclude, title, show, ica,
 
     if labels is not None:
         # compute colors only based on label categories
-        unique_labels = set([k.split(' - ')[1] for k in exclude_labels if k])
+        unique_labels = {k.split(' - ')[1] for k in exclude_labels if k}
         label_colors = plt.cm.rainbow(np.linspace(0, 1, len(unique_labels)))
         label_colors = dict(zip(unique_labels, label_colors))
     else:
-        label_colors = dict((k, 'red') for k in exclude_labels)
+        label_colors = {k: 'red' for k in exclude_labels}
 
     for exc_label, ii in zip(exclude_labels, picks):
         if exc_label is not None:
@@ -434,7 +550,7 @@ def _plot_ica_sources_evoked(evoked, picks, exclude, title, show, ica,
             color = label_colors[key]
             # ... but display component number too
             lines.extend(ax.plot(times, evoked.data[ii].T, picker=3.,
-                         zorder=2, color=color, label=exc_label))
+                                 zorder=2, color=color, label=exc_label))
         else:
             lines.extend(ax.plot(times, evoked.data[ii].T, picker=3.,
                                  color='k', zorder=1))
@@ -456,7 +572,6 @@ def _plot_ica_sources_evoked(evoked, picks, exclude, title, show, ica,
     lines = [lines]
     ch_names = evoked.ch_names
 
-    from matplotlib import patheffects
     path_effects = [patheffects.withStroke(linewidth=2, foreground="w",
                                            alpha=0.75)]
     params = dict(axes=axes, texts=texts, lines=lines, idxs=idxs,
@@ -482,9 +597,9 @@ def plot_ica_scores(ica, scores, exclude=None, labels=None, axhline=None,
     ----------
     ica : instance of mne.preprocessing.ICA
         The ICA object.
-    scores : array_like of float, shape (n ica components) | list of arrays
+    scores : array-like of float, shape (n_ica_components,) | list of array
         Scores based on arbitrary metric to characterize ICA components.
-    exclude : array_like of int
+    exclude : array-like of int
         The components marked for exclusion. If None (default), ICA.exclude
         will be used.
     labels : str | list | 'ecg' | 'eog' | None
@@ -504,7 +619,7 @@ def plot_ica_scores(ica, scores, exclude=None, labels=None, axhline=None,
 
     Returns
     -------
-    fig : instance of matplotlib.pyplot.Figure
+    fig : instance of Figure
         The figure object
     """
     import matplotlib.pyplot as plt
@@ -529,7 +644,7 @@ def plot_ica_scores(ica, scores, exclude=None, labels=None, axhline=None,
     elif labels == 'eog':
         labels = [l for l in ica.labels_ if l.startswith('eog/')]
         labels.sort(key=lambda l: l.split('/')[1])  # sort by index
-    elif isinstance(labels, string_types):
+    elif isinstance(labels, str):
         if len(axes) > 1:
             raise ValueError('Need as many labels as axes (%i)' % len(axes))
         labels = [labels]
@@ -561,13 +676,14 @@ def plot_ica_scores(ica, scores, exclude=None, labels=None, axhline=None,
                 label = ', '.join(label.split('/'))
             ax.set_title('(%s)' % label)
         ax.set_xlabel('ICA components')
-        ax.set_xlim(0, len(this_scores))
+        ax.set_xlim(-0.6, len(this_scores) - 0.4)
 
     tight_layout(fig=fig)
     plt_show(show)
     return fig
 
 
+@fill_doc
 def plot_ica_overlay(ica, inst, exclude=None, picks=None, start=None,
                      stop=None, title=None, show=True):
     """Overlay of raw and cleaned signals given the unmixing matrix.
@@ -585,12 +701,10 @@ def plot_ica_overlay(ica, inst, exclude=None, picks=None, start=None,
         sources will be canceled out this display is sensitive to
         artifacts. If evoked input, butterfly plots for clean and raw
         signals will be superimposed.
-    exclude : array_like of int | None (default)
+    exclude : array-like of int | None (default)
         The components marked for exclusion. If None (default), ICA.exclude
         will be used.
-    picks : array-like of int | None (default)
-        Indices of channels to include (if None, all channels
-        are used that were included on fitting).
+    %(picks_base)s all channels that were included during fitting.
     start : int
         X-axis start index. If None from the beginning.
     stop : int
@@ -602,7 +716,7 @@ def plot_ica_overlay(ica, inst, exclude=None, picks=None, start=None,
 
     Returns
     -------
-    fig : instance of pyplot.Figure
+    fig : instance of Figure
         The figure.
     """
     # avoid circular imports
@@ -613,8 +727,9 @@ def plot_ica_overlay(ica, inst, exclude=None, picks=None, start=None,
     _validate_type(inst, (BaseRaw, Evoked), "inst", "Raw or Evoked")
     if title is None:
         title = 'Signals before (red) and after (black) cleaning'
-    if picks is None:
-        picks = [inst.ch_names.index(k) for k in ica.ch_names]
+    picks = ica.ch_names if picks is None else picks
+    picks = _picks_to_idx(inst.info, picks, exclude=())
+    ch_types_used = _get_channel_types(inst.info, picks=picks, unique=True)
     if exclude is None:
         exclude = ica.exclude
     if not isinstance(exclude, (np.ndarray, list)):
@@ -625,7 +740,6 @@ def plot_ica_overlay(ica, inst, exclude=None, picks=None, start=None,
             start = 0.0
         if stop is None:
             stop = 3.0
-        ch_types_used = [k for k in ['mag', 'grad', 'eeg'] if k in ica]
         start_compare, stop_compare = _check_start_stop(inst, start, stop)
         data, times = inst[picks, start_compare:stop_compare]
 
@@ -661,7 +775,7 @@ def _plot_ica_overlay_raw(data, data_cln, times, title, ch_types_used, show):
 
     Returns
     -------
-    fig : instance of pyplot.Figure
+    fig : instance of Figure
     """
     import matplotlib.pyplot as plt
     # Restore sensor space data and keep all PCA components
@@ -678,7 +792,7 @@ def _plot_ica_overlay_raw(data, data_cln, times, title, ch_types_used, show):
                  'grad': 'Gradiometers',
                  'eeg': 'EEG'}
     ch_types = ', '.join([_ch_types[k] for k in ch_types_used])
-    ax2.set_title('Average across channels ({0})'.format(ch_types))
+    ax2.set_title('Average across channels ({})'.format(ch_types))
     ax2.plot(times, data.mean(0), color='r')
     ax2.plot(times, data_cln.mean(0), color='k')
     ax2.set(xlabel='Time (s)', xlim=times[[0, -1]])
@@ -704,7 +818,7 @@ def _plot_ica_overlay_evoked(evoked, evoked_cln, title, show):
 
     Returns
     -------
-    fig : instance of pyplot.Figure
+    fig : instance of Figure
     """
     import matplotlib.pyplot as plt
     ch_types_used = [c for c in ['mag', 'grad', 'eeg'] if c in evoked]
@@ -735,14 +849,11 @@ def _plot_ica_overlay_evoked(evoked, evoked_cln, title, show):
 
 
 def _plot_sources_raw(ica, raw, picks, exclude, start, stop, show, title,
-                      block, show_first_samp):
+                      block, show_first_samp, show_scrollbars):
     """Plot the ICA components as raw array."""
     color = _handle_default('color', (0., 0., 0.))
     orig_data = ica._transform_raw(raw, 0, len(raw.times)) * 0.2
-    if picks is None:
-        picks = range(len(orig_data))
     types = ['misc' for _ in picks]
-    picks = list(sorted(picks))
     eog_chs = pick_types(raw.info, meg=False, eog=True, ref_meg=False)
     ecg_chs = pick_types(raw.info, meg=False, ecg=True, ref_meg=False)
     data = [orig_data[pick] for pick in picks]
@@ -791,7 +902,8 @@ def _plot_sources_raw(ica, raw, picks, exclude, start, stop, show, title,
                   ica=ica, n_channels=n_channels, times=times, types=types,
                   n_times=raw.n_times, bad_color=bad_color, picks=picks,
                   first_time=first_time, data_picks=[], decim=1,
-                  noise_cov=None, whitened_ch_names=())
+                  noise_cov=None, whitened_ch_names=(), clipping=None,
+                  use_scalebars=False, show_scrollbars=show_scrollbars)
     _prepare_mne_browse_raw(params, title, 'w', color, bad_color, inds,
                             n_channels)
     params['scale_factor'] = 1.0
@@ -800,7 +912,6 @@ def _plot_sources_raw(ica, raw, picks, exclude, start, stop, show, title,
     params['update_fun'] = partial(_update_data, params)
     params['pick_bads_fun'] = partial(_pick_bads, params=params)
     params['label_click_fun'] = partial(_label_clicked, params=params)
-    _layout_figure(params)
     # callbacks
     callback_key = partial(_plot_raw_onkey, params=params)
     params['fig'].canvas.mpl_connect('key_press_event', callback_key)
@@ -808,8 +919,6 @@ def _plot_sources_raw(ica, raw, picks, exclude, start, stop, show, title,
     params['fig'].canvas.mpl_connect('scroll_event', callback_scroll)
     callback_pick = partial(_mouse_click, params=params)
     params['fig'].canvas.mpl_connect('button_press_event', callback_pick)
-    callback_resize = partial(_helper_raw_resize, params=params)
-    params['fig'].canvas.mpl_connect('resize_event', callback_resize)
     callback_close = partial(_close_event, params=params)
     params['fig'].canvas.mpl_connect('close_event', callback_close)
     params['fig_proj'] = None
@@ -850,7 +959,7 @@ def _close_event(events, params):
 
 
 def _plot_sources_epochs(ica, epochs, picks, exclude, start, stop, show,
-                         title, block):
+                         title, block, show_scrollbars):
     """Plot the components as epochs."""
     data = ica._transform_epochs(epochs, concatenate=True)
     eog_chs = pick_types(epochs.info, meg=False, eog=True, ref_meg=False)
@@ -876,8 +985,6 @@ def _plot_sources_epochs(ica, epochs, picks, exclude, start, stop, show,
     info['bads'] = [c_names[x] for x in exclude]
     if title is None:
         title = 'ICA components'
-    if picks is None:
-        picks = list(range(ica.n_components_))
     if start is None:
         start = 0
     if stop is None:
@@ -892,12 +999,17 @@ def _plot_sources_epochs(ica, epochs, picks, exclude, start, stop, show,
                   bads=list(), bad_color=(1., 0., 0.),
                   t_start=start * len(epochs.times),
                   data_picks=list(), decim=1, whitened_ch_names=(),
-                  noise_cov=None)
+                  noise_cov=None, show_scrollbars=show_scrollbars,
+                  epoch_colors=None)
     params['label_click_fun'] = partial(_label_clicked, params=params)
+    # changing the order to 'misc' before 'eog' and 'ecg'
+    order = list(_DATA_CH_TYPES_ORDER_DEFAULT)
+    order.pop(order.index('misc'))
+    order.insert(order.index('eog'), 'misc')
     _prepare_mne_browse_epochs(params, projs=list(), n_channels=20,
                                n_epochs=n_epochs, scalings=scalings,
                                title=title, picks=picks,
-                               order=['misc', 'eog', 'ecg'])
+                               order=order, info=info)
     params['plot_update_proj_callback'] = _update_epoch_data
     _update_epoch_data(params)
     params['hsel_patch'].set_x(params['t_start'])
