@@ -1,9 +1,7 @@
 # -*- coding: utf-8 -*-
 """Functions to make simple plots with M/EEG data."""
 
-from __future__ import print_function
-
-# Authors: Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
+# Authors: Alexandre Gramfort <alexandre.gramfort@inria.fr>
 #          Denis Engemann <denis.engemann@gmail.com>
 #          Martin Luessi <mluessi@nmr.mgh.harvard.edu>
 #          Eric Larson <larson.eric.d@gmail.com>
@@ -24,19 +22,21 @@ import numpy as np
 from scipy import linalg
 
 from ..defaults import DEFAULTS
+from ..rank import compute_rank
 from ..surface import read_surface
-from ..externals.six import string_types
 from ..io.proj import make_projector
-from ..io.pick import _DATA_CH_TYPES_SPLIT, pick_types
+from ..io.pick import (_DATA_CH_TYPES_SPLIT, pick_types, pick_info,
+                       pick_channels)
 from ..source_space import read_source_spaces, SourceSpaces
-from ..utils import logger, verbose, get_subjects_dir, warn
+from ..utils import (logger, verbose, get_subjects_dir, warn, _check_option,
+                     _mask_to_onsets_offsets)
 from ..io.pick import _picks_by_type
 from ..filter import estimate_ringing_samples
 from .utils import tight_layout, _get_color_list, _prepare_trellis, plt_show
 
 
 @verbose
-def plot_cov(cov, info, exclude=[], colorbar=True, proj=False, show_svd=True,
+def plot_cov(cov, info, exclude=(), colorbar=True, proj=False, show_svd=True,
              show=True, verbose=None):
     """Plot Covariance data.
 
@@ -58,25 +58,42 @@ def plot_cov(cov, info, exclude=[], colorbar=True, proj=False, show_svd=True,
         type. We show square roots ie. standard deviations.
     show : bool
         Show figure if True.
-    verbose : bool, str, int, or None
-        If not None, override default verbose level (see :func:`mne.verbose`
-        and :ref:`Logging documentation <tut_logging>` for more).
+    %(verbose)s
 
     Returns
     -------
-    fig_cov : instance of matplotlib.pyplot.Figure
+    fig_cov : instance of matplotlib.figure.Figure
         The covariance plot.
-    fig_svd : instance of matplotlib.pyplot.Figure | None
+    fig_svd : instance of matplotlib.figure.Figure | None
         The SVD spectra plot of the covariance.
+
+    See Also
+    --------
+    mne.compute_rank
+
+    Notes
+    -----
+    For each channel type, the rank is estimated using
+    :func:`mne.compute_rank`.
+
+    .. versionchanged:: 0.19
+       Approximate ranks for each channel type are shown with red dashed lines.
     """
+    from ..cov import Covariance
+    import matplotlib.pyplot as plt
+    from matplotlib.colors import Normalize
+
     if exclude == 'bads':
         exclude = info['bads']
+    info = pick_info(info, pick_channels(info['ch_names'], cov['names'],
+                                         exclude))
+    del exclude
     picks_list = \
         _picks_by_type(info, meg_combined=False, ref_meg=False,
-                       exclude=exclude)
+                       exclude=())
     picks_by_type = dict(picks_list)
 
-    ch_names = [n for n in cov.ch_names if n not in exclude]
+    ch_names = [n for n in cov.ch_names if n in info['ch_names']]
     ch_idx = [cov.ch_names.index(n) for n in ch_names]
 
     info_ch_names = info['ch_names']
@@ -84,15 +101,16 @@ def plot_cov(cov, info, exclude=[], colorbar=True, proj=False, show_svd=True,
     for ch_type, sel in picks_by_type.items():
         idx_by_type[ch_type] = [ch_names.index(info_ch_names[c])
                                 for c in sel if info_ch_names[c] in ch_names]
-
     idx_names = [(idx_by_type[key],
                   '%s covariance' % DEFAULTS['titles'][key],
                   DEFAULTS['units'][key],
-                  DEFAULTS['scalings'][key])
+                  DEFAULTS['scalings'][key],
+                  key)
                  for key in _DATA_CH_TYPES_SPLIT
                  if len(idx_by_type[key]) > 0]
     C = cov.data[ch_idx][:, ch_idx]
 
+    projs = []
     if proj:
         projs = copy.deepcopy(info['projs'])
 
@@ -109,12 +127,9 @@ def plot_cov(cov, info, exclude=[], colorbar=True, proj=False, show_svd=True,
             logger.info('    The projection vectors do not apply to these '
                         'channels.')
 
-    import matplotlib.pyplot as plt
-    from matplotlib.colors import Normalize
-
     fig_cov, axes = plt.subplots(1, len(idx_names), squeeze=False,
                                  figsize=(3.8 * len(idx_names), 3.7))
-    for k, (idx, name, _, _) in enumerate(idx_names):
+    for k, (idx, name, _, _, _) in enumerate(idx_names):
         vlim = np.max(np.abs(C[idx][:, idx]))
         im = axes[0, k].imshow(C[idx][:, idx], interpolation="nearest",
                                norm=Normalize(vmin=-vlim, vmax=vlim),
@@ -134,14 +149,27 @@ def plot_cov(cov, info, exclude=[], colorbar=True, proj=False, show_svd=True,
     if show_svd:
         fig_svd, axes = plt.subplots(1, len(idx_names), squeeze=False,
                                      figsize=(3.8 * len(idx_names), 3.7))
-        for k, (idx, name, unit, scaling) in enumerate(idx_names):
-            s = linalg.svd(C[idx][:, idx], compute_uv=False)
+        for k, (idx, name, unit, scaling, key) in enumerate(idx_names):
+            this_C = C[idx][:, idx]
+            s = linalg.svd(this_C, compute_uv=False)
+            this_C = Covariance(this_C, [info['ch_names'][ii] for ii in idx],
+                                [], [], 0)
+            this_info = pick_info(info, idx)
+            this_info['projs'] = []
+            this_rank = compute_rank(this_C, info=this_info)
             # Protect against true zero singular values
             s[s <= 0] = 1e-10 * s[s > 0].min()
             s = np.sqrt(s) * scaling
-            axes[0, k].plot(s)
+            axes[0, k].plot(s, color='k', zorder=3)
+            this_rank = this_rank[key]
+            axes[0, k].axvline(this_rank - 1, ls='--', color='r',
+                               alpha=0.5, zorder=4, clip_on=False)
+            axes[0, k].text(this_rank - 1, axes[0, k].get_ylim()[1],
+                            'rank ≈ %d' % (this_rank,), ha='right', va='top',
+                            color='r', alpha=0.5, zorder=4)
             axes[0, k].set(ylabel=u'Noise σ (%s)' % unit, yscale='log',
-                           xlabel='Eigenvalue index', title=name)
+                           xlabel='Eigenvalue index', title=name,
+                           xlim=[0, len(s) - 1])
         tight_layout(fig=fig_svd)
 
     plt_show(show)
@@ -379,7 +407,7 @@ def plot_bem(subject=None, subjects_dir=None, orientation='coronal',
 
     Returns
     -------
-    fig : Instance of matplotlib.figure.Figure
+    fig : instance of matplotlib.figure.Figure
         The figure.
 
     See Also
@@ -410,7 +438,7 @@ def plot_bem(subject=None, subjects_dir=None, orientation='coronal',
             surfaces.append((surf_fname, color))
 
     if brain_surfaces is not None:
-        if isinstance(brain_surfaces, string_types):
+        if isinstance(brain_surfaces, str):
             brain_surfaces = (brain_surfaces,)
         for surf_name in brain_surfaces:
             for hemi in ('lh', 'rh'):
@@ -421,7 +449,7 @@ def plot_bem(subject=None, subjects_dir=None, orientation='coronal',
                 else:
                     raise IOError("Surface %s does not exist." % surf_fname)
 
-    if isinstance(src, string_types):
+    if isinstance(src, str):
         if not op.exists(src):
             src_ = op.join(subjects_dir, subject, 'bem', src)
             if op.exists(src_):
@@ -454,19 +482,20 @@ def plot_events(events, sfreq=None, first_samp=0, color=None, event_id=None,
         The sample frequency. If None, data will be displayed in samples (not
         seconds).
     first_samp : int
-        The index of the first sample. Typically the raw.first_samp
-        attribute. It is needed for recordings on a Neuromag
-        system as the events are defined relative to the system
-        start and not to the beginning of the recording.
+        The index of the first sample. Recordings made on Neuromag systems
+        number samples relative to the system start (not relative to the
+        beginning of the recording). In such cases the ``raw.first_samp``
+        attribute can be passed here. Default is 0.
     color : dict | None
-        Dictionary of event_id value and its associated color. If None,
+        Dictionary of event_id integers as keys and colors as values. If None,
         colors are automatically drawn from a default list (cycled through if
-        number of events longer than list of default colors).
+        number of events longer than list of default colors). Color can be any
+        valid :doc:`matplotlib color <tutorials/colors/colors>`.
     event_id : dict | None
-        Dictionary of event label (e.g. 'aud_l') and its associated
-        event_id value. Label used to plot a legend. If None, no legend is
-        drawn.
-    axes : instance of matplotlib.axes.AxesSubplot
+        Dictionary of event labels (e.g. 'aud_l') as keys and their associated
+        event_id values. Labels are used to plot a legend. If None, no legend
+        is drawn.
+    axes : instance of Axes
        The subplot handle.
     equal_spacing : bool
         Use equal spacing between events in y-axis.
@@ -494,7 +523,7 @@ def plot_events(events, sfreq=None, first_samp=0, color=None, event_id=None,
     if event_id is not None:
         # get labels and unique event ids from event_id dict,
         # sorted by value
-        event_id_rev = dict((v, k) for k, v in event_id.items())
+        event_id_rev = {v: k for k, v in event_id.items()}
         conditions, unique_events_id = zip(*sorted(event_id.items(),
                                                    key=lambda x: x[1]))
 
@@ -526,8 +555,7 @@ def plot_events(events, sfreq=None, first_samp=0, color=None, event_id=None,
         ev_mask = events[:, 2] == ev
         kwargs = {}
         if event_id is not None:
-            event_label = '{0} ({1})'.format(event_id_rev[ev],
-                                             np.sum(ev_mask))
+            event_label = '{} ({})'.format(event_id_rev[ev], np.sum(ev_mask))
             kwargs['label'] = event_label
         if ev in color:
             kwargs['color'] = color[ev]
@@ -582,9 +610,9 @@ def plot_dipole_amplitudes(dipoles, colors=None, show=True):
 
     Parameters
     ----------
-    dipoles : list of instance of Dipoles
+    dipoles : list of instance of Dipole
         The dipoles whose amplitudes should be shown.
-    colors: list of colors | None
+    colors: list of color | None
         Color to plot with each dipole. If None default colors are used.
     show : bool
         Show figure if True.
@@ -639,9 +667,13 @@ def _filter_ticks(lims, fscale):
         return None, None  # let matplotlib handle it
     lims = np.array(lims)
     ticks = list()
+    if lims[1] > 20 * lims[0]:
+        base = np.array([1, 2, 4])
+    else:
+        base = np.arange(1, 11)
     for exp in range(int(np.floor(np.log10(lims[0]))),
                      int(np.floor(np.log10(lims[1]))) + 1):
-        ticks += (np.array([1, 2, 4]) * (10 ** exp)).tolist()
+        ticks += (base * (10 ** exp)).tolist()
     ticks = np.array(ticks)
     ticks = ticks[(ticks >= lims[0]) & (ticks <= lims[1])]
     ticklabels = [('%g' if t < 1 else '%d') % t for t in ticks]
@@ -669,13 +701,17 @@ def _get_flim(flim, fscale, freq, sfreq=None):
 
 def _check_fscale(fscale):
     """Check for valid fscale."""
-    if not isinstance(fscale, string_types) or fscale not in ('log', 'linear'):
+    if not isinstance(fscale, str) or fscale not in ('log', 'linear'):
         raise ValueError('fscale must be "log" or "linear", got %s'
                          % (fscale,))
 
 
+_DEFAULT_ALIM = (-80, 10)
+
+
 def plot_filter(h, sfreq, freq=None, gain=None, title=None, color='#1f77b4',
-                flim=None, fscale='log', alim=(-60, 10), show=True):
+                flim=None, fscale='log', alim=_DEFAULT_ALIM, show=True,
+                compensate=False):
     """Plot properties of a filter.
 
     Parameters
@@ -705,6 +741,16 @@ def plot_filter(h, sfreq, freq=None, gain=None, title=None, color='#1f77b4',
         The y-axis amplitude limits (dB) to use (default: (-60, 10)).
     show : bool
         Show figure if True (default).
+    compensate : bool
+        If True, compensate for the filter delay (phase will not be shown).
+
+        - For linear-phase FIR filters, this visualizes the filter coefficients
+          assuming that the output will be shifted by ``N // 2``.
+        - For IIR filters, this changes the filter coefficient display
+          by filtering backward and forward, and the frequency response
+          by squaring it.
+
+        .. versionadded:: 0.18
 
     Returns
     -------
@@ -720,10 +766,12 @@ def plot_filter(h, sfreq, freq=None, gain=None, title=None, color='#1f77b4',
     -----
     .. versionadded:: 0.14
     """
-    from scipy.signal import freqz, group_delay
+    from scipy.signal import (
+        freqz, group_delay, lfilter, filtfilt, sosfilt, sosfiltfilt)
     import matplotlib.pyplot as plt
+    from matplotlib.ticker import FormatStrFormatter, NullFormatter
     sfreq = float(sfreq)
-    _check_fscale(fscale)
+    _check_option('fscale', fscale, ['log', 'linear'])
     flim = _get_flim(flim, fscale, freq, sfreq)
     if fscale == 'log':
         omega = np.logspace(np.log10(flim[0]), np.log10(flim[1]), 1000)
@@ -732,62 +780,98 @@ def plot_filter(h, sfreq, freq=None, gain=None, title=None, color='#1f77b4',
     omega /= sfreq / (2 * np.pi)
     if isinstance(h, dict):  # IIR h.ndim == 2:  # second-order sections
         if 'sos' in h:
-            from scipy.signal import sosfilt
-            h = h['sos']
             H = np.ones(len(omega), np.complex128)
             gd = np.zeros(len(omega))
-            for section in h:
+            for section in h['sos']:
                 this_H = freqz(section[:3], section[3:], omega)[1]
                 H *= this_H
-                with warnings.catch_warnings(record=True):  # singular GD
-                    warnings.simplefilter('ignore')
-                    gd += group_delay((section[:3], section[3:]), omega)[1]
-            n = estimate_ringing_samples(h)
+                if compensate:
+                    H *= this_H.conj()  # time reversal is freq conj
+                else:
+                    # Assume the forward-backward delay zeros out, which it
+                    # mostly should
+                    with warnings.catch_warnings(record=True):  # singular GD
+                        warnings.simplefilter('ignore')
+                        gd += group_delay((section[:3], section[3:]), omega)[1]
+            n = estimate_ringing_samples(h['sos'])
             delta = np.zeros(n)
             delta[0] = 1
-            h = sosfilt(h, delta)
+            if compensate:
+                delta = np.pad(delta, [(n - 1, 0)], 'constant')
+                func = sosfiltfilt
+                gd += (len(delta) - 1) // 2
+            else:
+                func = sosfilt
+            h = func(h['sos'], delta)
         else:
-            from scipy.signal import lfilter
-            n = estimate_ringing_samples((h['b'], h['a']))
-            delta = np.zeros(n)
-            delta[0] = 1
             H = freqz(h['b'], h['a'], omega)[1]
+            if compensate:
+                H *= H.conj()
             with warnings.catch_warnings(record=True):  # singular GD
                 warnings.simplefilter('ignore')
                 gd = group_delay((h['b'], h['a']), omega)[1]
-            h = lfilter(h['b'], h['a'], delta)
-        title = 'SOS (IIR) filter' if title is None else title
+                if compensate:
+                    gd += group_delay(h['b'].conj(), h['a'].conj(), omega)[1]
+            n = estimate_ringing_samples((h['b'], h['a']))
+            delta = np.zeros(n)
+            delta[0] = 1
+            if compensate:
+                delta = np.pad(delta, [(n - 1, 0)], 'constant')
+                func = filtfilt
+            else:
+                func = lfilter
+            h = func(h['b'], h['a'], delta)
+        if title is None:
+            title = 'SOS (IIR) filter'
+        if compensate:
+            title += ' (forward-backward)'
     else:
         H = freqz(h, worN=omega)[1]
         with warnings.catch_warnings(record=True):  # singular GD
             warnings.simplefilter('ignore')
             gd = group_delay((h, [1.]), omega)[1]
         title = 'FIR filter' if title is None else title
-    gd /= sfreq
+        if compensate:
+            title += ' (delay-compensated)'
     # eventually axes could be a parameter
     fig, (ax_time, ax_freq, ax_delay) = plt.subplots(3)
-    t = np.arange(len(h)) / sfreq
+    t = np.arange(len(h))
+    if compensate:
+        n_shift = (len(h) - 1) // 2
+        t -= n_shift
+        assert t[0] == -t[-1]
+        gd -= n_shift
+    t = t / sfreq
+    gd = gd / sfreq
     f = omega * sfreq / (2 * np.pi)
     ax_time.plot(t, h, color=color)
     ax_time.set(xlim=t[[0, -1]], xlabel='Time (s)',
                 ylabel='Amplitude', title=title)
     mag = 10 * np.log10(np.maximum((H * H.conj()).real, 1e-20))
-    ax_freq.plot(f, mag, color=color, linewidth=2, zorder=4)
-    if freq is not None and gain is not None:
-        plot_ideal_filter(freq, gain, ax_freq, fscale=fscale,
-                          title=None, show=False)
-    ax_freq.set(ylabel='Magnitude (dB)', xlabel='', xscale=fscale)
     sl = slice(0 if fscale == 'linear' else 1, None, None)
+    # Magnitude
+    ax_freq.plot(f[sl], mag[sl], color=color, linewidth=2, zorder=4)
+    if freq is not None and gain is not None:
+        plot_ideal_filter(freq, gain, ax_freq, fscale=fscale, show=False)
+    ax_freq.set(ylabel='Magnitude (dB)', xlabel='', xscale=fscale)
+    # Delay
     ax_delay.plot(f[sl], gd[sl], color=color, linewidth=2, zorder=4)
+    # shade nulled regions
+    for start, stop in zip(*_mask_to_onsets_offsets(mag <= -39.9)):
+        ax_delay.axvspan(f[start], f[stop - 1], facecolor='k', alpha=0.05,
+                         zorder=5)
     ax_delay.set(xlim=flim, ylabel='Group delay (s)', xlabel='Frequency (Hz)',
                  xscale=fscale)
     xticks, xticklabels = _filter_ticks(flim, fscale)
-    dlim = [0, 1.05 * gd[1:].max()]
+    dlim = np.abs(t).max() / 2.
+    dlim = [-dlim, dlim]
     for ax, ylim, ylabel in ((ax_freq, alim, 'Amplitude (dB)'),
                              (ax_delay, dlim, 'Delay (s)')):
         if xticks is not None:
             ax.set(xticks=xticks)
             ax.set(xticklabels=xticklabels)
+        ax.xaxis.set_major_formatter(FormatStrFormatter('%0.1f'))
+        ax.xaxis.set_minor_formatter(NullFormatter())
         ax.set(xlim=flim, ylim=ylim, xlabel='Frequency (Hz)', ylabel=ylabel)
     adjust_axes([ax_time, ax_freq, ax_delay])
     tight_layout()
@@ -796,7 +880,7 @@ def plot_filter(h, sfreq, freq=None, gain=None, title=None, color='#1f77b4',
 
 
 def plot_ideal_filter(freq, gain, axes=None, title='', flim=None, fscale='log',
-                      alim=(-60, 10), color='r', alpha=0.5, linestyle='--',
+                      alim=_DEFAULT_ALIM, color='r', alpha=0.5, linestyle='--',
                       show=True):
     """Plot an ideal filter response.
 
@@ -806,7 +890,7 @@ def plot_ideal_filter(freq, gain, axes=None, title='', flim=None, fscale='log',
         The ideal response frequencies to plot (must be in ascending order).
     gain : array-like or None
         The ideal response gains to plot.
-    axes : instance of matplotlib.axes.AxesSubplot | None
+    axes : instance of Axes | None
         The subplot handle. With None (default), axes are created.
     title : str
         The title to use, (default: '').
@@ -828,7 +912,7 @@ def plot_ideal_filter(freq, gain, axes=None, title='', flim=None, fscale='log',
 
     Returns
     -------
-    fig : Instance of matplotlib.figure.Figure
+    fig : instance of matplotlib.figure.Figure
         The figure.
 
     See Also
@@ -851,23 +935,20 @@ def plot_ideal_filter(freq, gain, axes=None, title='', flim=None, fscale='log',
 
     """
     import matplotlib.pyplot as plt
-    xs, ys = list(), list()
     my_freq, my_gain = list(), list()
     if freq[0] != 0:
         raise ValueError('freq should start with DC (zero) and end with '
                          'Nyquist, but got %s for DC' % (freq[0],))
     freq = np.array(freq)
     # deal with semilogx problems @ x=0
-    _check_fscale(fscale)
+    _check_option('fscale', fscale, ['log', 'linear'])
     if fscale == 'log':
         freq[0] = 0.1 * freq[1] if flim is None else min(flim[0], freq[1])
     flim = _get_flim(flim, fscale, freq)
+    transitions = list()
     for ii in range(len(freq)):
-        xs.append(freq[ii])
-        ys.append(alim[0])
         if ii < len(freq) - 1 and gain[ii] != gain[ii + 1]:
-            xs += [freq[ii], freq[ii + 1]]
-            ys += [alim[1]] * 2
+            transitions += [[freq[ii], freq[ii + 1]]]
             my_freq += np.linspace(freq[ii], freq[ii + 1], 20,
                                    endpoint=False).tolist()
             my_gain += np.linspace(gain[ii], gain[ii + 1], 20,
@@ -878,8 +959,8 @@ def plot_ideal_filter(freq, gain, axes=None, title='', flim=None, fscale='log',
     my_gain = 10 * np.log10(np.maximum(my_gain, 10 ** (alim[0] / 10.)))
     if axes is None:
         axes = plt.subplots(1)[1]
-    xs = np.maximum(xs, flim[0])
-    axes.fill_between(xs, alim[0], ys, color=color, alpha=0.1)
+    for transition in transitions:
+        axes.axvspan(*transition, color=color, alpha=0.1)
     axes.plot(my_freq, my_gain, color=color, linestyle=linestyle, alpha=0.5,
               linewidth=4, zorder=3)
     xticks, xticklabels = _filter_ticks(flim, fscale)
@@ -889,6 +970,8 @@ def plot_ideal_filter(freq, gain, axes=None, title='', flim=None, fscale='log',
         axes.set(xticks=xticks)
         axes.set(xticklabels=xticklabels)
     axes.set(xlim=flim)
+    if title:
+        axes.set(title=title)
     adjust_axes(axes)
     tight_layout()
     plt_show(show)
@@ -908,8 +991,8 @@ def _handle_event_colors(unique_events, color, unique_events_id):
     else:
         for this_event in color:
             if this_event not in unique_events_id:
-                raise ValueError('%s from color is not present in events '
-                                 'or event_id.' % this_event)
+                raise ValueError('Event ID %s is in the color dict but is not '
+                                 'present in events or event_id.' % this_event)
 
         for this_event in unique_events_id:
             if this_event not in color:
@@ -950,7 +1033,7 @@ def plot_csd(csd, info=None, mode='csd', colorbar=True, cmap=None,
 
     Returns
     -------
-    fig : list of matplotlib figures
+    fig : list of Figure
         The figures created by this function.
     """
     import matplotlib.pyplot as plt
