@@ -1,4 +1,4 @@
-# Authors: Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
+# Authors: Alexandre Gramfort <alexandre.gramfort@inria.fr>
 #          Denis Engemann <denis.engemann@gmail.com>
 #          Martin Luessi <mluessi@nmr.mgh.harvard.edu>
 #          Eric Larson <larson.eric.d@gmail.com>
@@ -10,6 +10,8 @@ from collections import namedtuple
 
 import numpy as np
 import pytest
+import matplotlib
+import matplotlib.pyplot as plt
 
 from mne import read_events, Epochs, pick_channels_evoked, read_cov
 from mne.channels import read_layout
@@ -25,11 +27,6 @@ from mne.viz.utils import _fake_click
 from mne.viz.topo import (_plot_update_evoked_topo_proj, iter_topography,
                           _imshow_tfr)
 
-# Set our plotters to test mode
-import matplotlib
-matplotlib.use('Agg')  # for testing don't use X server
-
-
 base_dir = op.join(op.dirname(__file__), '..', '..', 'io', 'tests', 'data')
 evoked_fname = op.join(base_dir, 'test-ave.fif')
 raw_fname = op.join(base_dir, 'test_raw.fif')
@@ -73,14 +70,9 @@ def _get_epochs_delayed_ssp():
     return epochs_delayed_ssp
 
 
-def test_plot_topo():
-    """Test plotting of ERP topography."""
-    import matplotlib.pyplot as plt
-    # Show topography
+def test_plot_joint():
+    """Test joint plot."""
     evoked = _get_epochs().average()
-    # should auto-find layout
-    plot_evoked_topo([evoked, evoked], merge_grads=True, background_color='w')
-    # Test jointplot
     evoked.plot_joint(ts_args=dict(time_unit='s'),
                       topomap_args=dict(time_unit='s'))
 
@@ -96,9 +88,18 @@ def test_plot_topo():
     axes = plt.subplots(nrows=3)[-1].flatten().tolist()
     evoked.plot_joint(times=[0], picks=[6, 7, 8],  ts_args=dict(axes=axes[0]),
                       topomap_args={"axes": axes[1:], "time_unit": "s"})
-    plt.close()
-    pytest.raises(ValueError, evoked.plot_joint, picks=[6, 7, 8],
-                  ts_args=dict(axes=axes[0]), topomap_args=dict(axes=axes[2:]))
+    with pytest.raises(ValueError, match='array of length 6'):
+        evoked.plot_joint(picks=[6, 7, 8], ts_args=dict(axes=axes[0]),
+                          topomap_args=dict(axes=axes[2:]))
+    plt.close('all')
+
+
+def test_plot_topo():
+    """Test plotting of ERP topography."""
+    # Show topography
+    evoked = _get_epochs().average()
+    # should auto-find layout
+    plot_evoked_topo([evoked, evoked], merge_grads=True, background_color='w')
 
     picked_evoked = evoked.copy().pick_channels(evoked.ch_names[:3])
     picked_evoked_eeg = evoked.copy().pick_types(meg=False, eeg=True)
@@ -137,10 +138,12 @@ def test_plot_topo():
     # Test RMS plot of grad pairs
     picked_evoked.plot_topo(merge_grads=True, background_color='w')
     plt.close('all')
-    for ax, idx in iter_topography(evoked.info):
+    for ax, idx in iter_topography(evoked.info, legend=True):
         ax.plot(evoked.data[idx], color='red')
         # test status bar message
-        assert (evoked.ch_names[idx] in ax.format_coord(.5, .5))
+        if idx != -1:
+            assert (evoked.ch_names[idx] in ax.format_coord(.5, .5))
+    assert idx == -1
     plt.close('all')
     cov = read_cov(cov_fname)
     cov['projs'] = []
@@ -153,12 +156,20 @@ def test_plot_topo():
                         evoked.times)
     plt.close('all')
 
+    for ax, idx in iter_topography(evoked.info):  # brief test with false
+        ax.plot([0, 1, 2])
+        break
+    plt.close('all')
+
 
 def test_plot_topo_single_ch():
     """Test single channel topoplot with time cursor."""
-    import matplotlib.pyplot as plt
     evoked = _get_epochs().average()
-    fig = plot_evoked_topo(evoked, background_color='w')
+    evoked2 = evoked.copy()
+    # test plotting several evokeds on different time grids
+    evoked.crop(-.19, 0)
+    evoked2.crop(.05, .19)
+    fig = plot_evoked_topo([evoked, evoked2], background_color='w')
     # test status bar message
     ax = plt.gca()
     assert ('MEG 0113' in ax.format_coord(.065, .63))
@@ -176,7 +187,6 @@ def test_plot_topo_single_ch():
 
 def test_plot_topo_image_epochs():
     """Test plotting of epochs image topography."""
-    import matplotlib.pyplot as plt
     title = 'ERF images - MNE sample data'
     epochs = _get_epochs()
     epochs.load_data()
@@ -189,13 +199,19 @@ def test_plot_topo_image_epochs():
     num_figures_before = len(plt.get_fignums())
     _fake_click(fig, fig.axes[0], (0.08, 0.64))
     assert num_figures_before + 1 == len(plt.get_fignums())
+    # test for auto-showing a colorbar when only 1 sensor type
+    ep = epochs.copy().pick_types(meg=False, eeg=True)
+    fig = plot_topo_image_epochs(ep, vmin=None, vmax=None, colorbar=None,
+                                 cmap=cmap)
+    ax = [x for x in fig.get_children() if isinstance(x, matplotlib.axes.Axes)]
+    qm_cmap = [y.cmap for x in ax for y in x.get_children()
+               if isinstance(y, matplotlib.collections.QuadMesh)]
+    assert qm_cmap[0] is cmap
     plt.close('all')
 
 
 def test_plot_tfr_topo():
     """Test plotting of TFR data."""
-    import matplotlib.pyplot as plt
-
     epochs = _get_epochs()
     n_freqs = 3
     nave = 1
@@ -209,7 +225,7 @@ def test_plot_tfr_topo():
     # test opening tfr by clicking
     num_figures_before = len(plt.get_fignums())
     # could use np.reshape(fig.axes[-1].images[0].get_extent(), (2, 2)).mean(1)
-    with pytest.warns(None):  # on old mpl there is a warning
+    with pytest.warns(None):  # on old mpl (at least 2.0) there is a warning
         _fake_click(fig, fig.axes[0], (0.08, 0.65))
     assert num_figures_before + 1 == len(plt.get_fignums())
     plt.close('all')
